/******** <lalVerbatim file="KnownPulsarDemodCV"> ********
Author: Mendell, Greg A.
$Id: KnownPulsarDemod.c,v 1.19 2003/02/05 01:36:46 gmendell Exp $
********* </lalVerbatim> ********/

/* Notes: */
/*  DFT  = Discrete Fourier Transform, e.g., as computed by FFT routines. */
/*  SFT  = short time baseline DFT;  No demodulation of the data is done */
/*  DeFT = stitched together SFTs forming demodulated longer time baseline DFT. */
/*  DeFTs remove phase modulation due to Earth's motion and that intristic to the source */
/*  Eventually DeFTs will account for amplitude modulation due to beam pattern sensitivity of the detector */

/* 09/26/01 Code first submitted to lalwrapper cvs. */
/* Revisons: */
/*  09/27/01 gam Debugged code to produce correct SFT output.  Was adding data to multiDimData using complex_8; needed complex_16.; */
/*  09/28/01 gam Produce SFTs using LALForwardRealFFT rather than LALCOMPLEX8VectorFFT. */
/*  10/08/01 gam Copy in latest tdb function form LALDemodTest; update definitions and calls to funcName. */
/*  10/10/01 gam Change deltaT and tSFT to type REAL8; Check that input data interval and sample rate agrees with that requested; get units from input data. */
/*  10/10/01 gam Add error checking that data is found (unless generating test data) */
/*  10/10/01 gam Set up time intervals using input params->gpsStartTime */
/*  10/11/01 gam Add nSFT and mCohSFTPerCall as command line filter parameters.  */
/*  10/11/01 gam Clear up confusion between using N - 1 and N to compute tSFT, tCoh, duration, etc.... */
/*  NOTE THE FOLLOWING ASSUMPTIONS MUST BE TRUE FOR THIS CODE TO WORK */
/*  tSFT = Time between start of one SFT and next SFT for sequential SFTs */
/*  The difference between the times associated with the first and last data points in an n second frame file must correspond to n - deltaT seconds */
/*  The number of data points in the file is N = (n - deltaT)/deltaT + 1 = n/deltaT = n*sampleRate */
/*  The difference between the gpsStartTime and gpsStopTime in a frame file is deltaT*N not deltaT*(N - 1) */
/*  For DFTs that df = 1/N*deltaT */
/*  10/12/01 gam Move declaration and allocation of memory into InitSearch and ConditionData; Deallocate when finished with variable or in FinalizeSearch */
/*  10/15/01 gam Move option to generate test data into ConditionData. */
/*  10/15/01 gam Move declaration of variables used only in LALDemod into LALDemod section */
/*  10/15/01 gam Add code to compute mCohSFTPerCall per call to ApplySearch; return correct output->fracRemaining each time */
/*  10/16/01 gam Do not set up creator_db, process_id, or event_id; input filter_id, ifo, and target_name as command line filter parameters */
/*  10/17/01 gam Make duration database column type REAL4. */
/*  10/18/01 gam Do not set up filter_id column */
/*  10/26/01 gam Change measured from 1 to 0 when creating FFTPlan to remove warnings:
     LALMalloc warning: zero size allocation and LALFree warning: tried to free a freed pointer.
     These warning occur only when measure = 1 and the FFTPlan is for a power of 2 input data points.
     Note: use measure equal 0 for faster estimated plan, nonzero for more exact plan. */
/*  10/30/01 gam When params->generateTestData == 'y' handle case when params->vecInputData not yet created. */
/*  11/02/01 gam Replace int with INT4 in MoreTimes */
/*  11/02/01 gam Add testTypes 3 and 4
    0 = sinusoidal test signal only;
    1 = 0 + gaussian noise (mean = 0, std dev = 1);
    2 = add test signal to input data;
    3 = same as 0 but without phase or amplitude modulation due to Earth's motion;
    4 = same as 2 but without phase or amplitude modulation due to Earth's motion; */
/*  11/02/01 gam Add drop out percentage into dcHistory structure for each SFT. */
/*  11/05/01 gam Add more meta data into the dcHistory structure for each SFT. */
/*  11/05/01 gam Change filterID to dataCondDesc; this is a string with data conditioning info */
/*  11/05/01 gam add dsoName and percentPad as parameters; add these and dataConDesc to dcHistory structure for each SFT */
/*  11/05/01 gam Find power in freq bin closest to f0 = pwF0 and std def pwStdDev; temporarily set SNR = (pwF0 - pwMean)/pwStdDev. */
/*  11/05/01 gam Replace pwMaxOverpwMean with pwF0OverpwMean = pwF0/pwMean. Set Confidence = 1 - exp(-pwF0/pwMean)  */
/*  11/05/01 gam Replace phaseForPWMax with phaseForPWF0 */
/*  11/05/01 gam Replace freqPWMax with freqPWF0 */
/*  11/10/01 gam Change template and test RA, Dec, Freq, Freq derivatives, and polarization from REAL4 to REAL8. */
/*  11/10/01 gam Change database entries for RA, Dec, Freq from REAL4 to REAL8. */
/*  11/10/01 gam Change KNOWNPULSARDEMODH_LIGOBANDMIN to 0.0 Hz and KNOWNPULSARDEMODH_LIGOBANDMAX to 2048 Hz. */
/*  11/10/01 gam Add an SFTStats data structure; store max, mean, and stddev stats for each SFT in this structure; store values in SFT history */
/*  11/10/01 gam Add static function CalcDemodPowerStats to calculate SFTStats */
/*  11/10/01 gam Add parameter nDeFT =(params->if0Max-params->if0Min)*params->mCohSFT = Number of bins in a DeFT. */
/*  11/11/01 gam Add code to handle time delays introduced into by resampling and filtering. */
/*  11/17/01 gam Replace double with REAL8 in MoreTimes */
/*  11/17/01 gam Do not vet input stop time, since vetting inputN and deltaT is enough.  Stop time since it can be ambigous,
             e.g, can = start time + N*deltaT or start time + (N - 1)*deltaT. */
/*  11/17/01 gam Instead of putting stop time in the SFT, put number of input data point = nSFT. This is less ambiguous than the stop time. */
/*  11/17/01 gam Adjust order of meta data in SFT output history structure */
/*  11/18/01 gam Allow channel as an input parameter */
/*  11/27/01 gam Add code to pad data with mean value between cutoffs using a quality channel. */
/*  11/27/01 gam Initialize percentPad as -1, which mean percentPad was not set since dropout and quality was not handled */
/*  12/05/01 gam Add error checking iStartTime = index to gpsStartTime is not negative or past end of input data */
/*  12/05/01 gam Fix errors in checking for errors in declination, frequency derivatives, and mCohSFTPerCall */
/*  12/06/01 gam Updated basic documentation */
/*  12/13/01 gam Add column "search" to BuildOutput for SGNL_DPERIDIC database table */
/*  01/28/02 gam Change code to handle new definition of LALUnitPair. */
/*  02/07/02 gam Fix bug in quality channel code; cutoffs were incorrectly applied; disable use of cutoffs for now. */
/*  03/06/02 gam Save SFTs as COMPLEX8 rather than as COMPLEX16 data. */
/*  04/16/02 gam Complet code change to handle new definition of LALUnitPair. */
/*  05/13/02 gam Temporarily comment out INCLUDE_DEMOD_CODE */
/*  05/13/02 gam Uncomment INCLUDE_DEMOD_CODE to work on including new version of LALDemod */
/*  05/14/02 gam FFT data structure now holds COMPLEX8 frequency series; change xHat and SFTData to hold COMPLEX8 rather than COMPLEX16 data. */
/*  05/16/02 gam Change xHat to type DeFTPeriodogram which contains a REAL8FrequencySeries and a ParameterSet.  The ParameterSet contains SkyPos and Spindown. */
/*               !!! NOTE that xHat now contains the F statistic from Jaranowski, Krolak, and Schutz (I think Eq. 99) */
/*               !!! Also, I think LALDemod returns F in unit of |SFT units|^2 which currently would be count^2 per Hz. */
/*               !!! To make F dimensionless we would need to divide by S_h(f_0), the noise spectral density, in these same units */
/*  05/17/02 gam Add 1 to definition of nDeFT to include starting bin; set length of data in xHat. */
/*  05/17/02 gam Include code for amplitude demodulation; copied from LALDemodTest.c */
/*  06/02/02 gam Continue to update code to work with latest LALDemod code; add fA and fB to xHat; complete call to amp demod code; add calls to barycenter code */
/*  06/02/02 gam WARNING: Using Ephemeris data from earth98.dat and sun98.dat; Using detector info in lalCachedDetectors array. NEED TO CHECK THIS IS OK!!! */
/*  06/02/02 gam WARNING: Is FloatToTime function correct ??? */
/*  06/02/02 gam WARNING: Is midTS[k] set up corrrectly */
/*  06/06/02 gam Try to solve problem with finding LALInitBarycenter. */
/*  06/11/02 gam Try to solve problem with finding LALInitBarycenter; part of problem was code was not linking to liblalsupport.so. */
/*               Still do not know how to do this correctly.  Also, need paths to earth02.dat and sun02.dat; need to file names input parameters. */
/*  07/01/02 gam Copying local version of LALInitBarycenter for stand-alone code; in dataPipeline datacon should read in earth02.ilwd and sun02.ilwd. */
/*               Code will have to written to unpack the data from the datacon. */
/*  07/07/02 gam Working on debugging new code */
/*  07/07/02 gam Change doDemod to type INT2 */
/*               Case 0 means compute SFTS, case 1 means compute SFTs and then do demod; case >= 2 means unpack SFTs and ephemeris data from input and do demod */
/*  07/07/02 gam Adding code to unpack SFT data from input structure */
/*  07/09/02 gam Add code to unpack input SFTs */
/*                NOTE HARD CODED to look for sft in sequence COMMENT field; SHOULD CHANGE THIS BEFORE FINALIZING CODE */
/*  07/09/02 gam Put 1 inside () in definition of nDeFT to include starting bin; used to set length of data in xHat. */
/*  07/09/02 gam temporarily set SNR = sqrt(pwF0/pwMean) = sqrt(pwF0OverpwMean). THIS IS REALLY JKS sqrt(Fmax/Fmean) */
/*  07/10/02 gam Write LALWRAPPERInitBarycenter function to transfer ephemeris data from inout LDAS structure into LAL structures; based on LALInitBarycenter. */
/*  07/10/02 gam Add EphemerisData *edat = pointer to ephemeris data to DemodSearchParams */
/*  07/12/02 gam Add code to write to the search summary table and search summary vars table. */
/*  07/12/02 gam Add params->numNodes and params->numDBOutput parameters */
/*  07/12/02 gam Add BuildDemodFrameOutput similar to BuildFrameOutput. Output Demod JKS F Power Spectrum in Frame Format */
/*  07/18/02 gam String data, for some reason, no longer set up OK in sngl_dperiodic database table */
/*  07/18/02 gam Need to Always call LALWRAPPERInitBarycenter if doDemod > 0 and if INCLUDE_LALWRAPPERINITBARYCENTER is defined. */
/*  07/18/02 gam f start, f stop, and df not set up in xHat.  For DeFTs use f0min, f0max, and df = 1.0/params->tCoh. */
/*  07/18/02 gam IFO in sngl_dperiodic can only be 2 characters. */
/*  07/18/02 gam dataConDesc is obsolete; change this to 2 character ifoNickName = H1, H2, or L1; note IFO is the site = LHO, LLO, GEO, etc.... */
/*  07/18/02 gam Use ifoNickName in sngl_dperiodic for ifo column. */
/*  08/06/02 gam Include new parameter:  demParams->sftPerCoh = params->sftPerCoh; Array with alpha index and mCohSFT, unless gaps are present. */
/*               Will compute sftPerCoh in MoreTimes, as in LALDemodTest times2 */
/*               Note that new parameter amParams->mObsSFT is in LALDemodTest.c but not LALComputeAM.h ! (= params->sftPerCoh[params->mObsCoh]); */
/*  09/18/02 gam Parameter amParams->mObsSFT is now in LALComputeAM.h (= params->sftPerCoh[params->mObsCoh]). */
/*  09/18/02 gam Add in params: amParams->ts0 and amParams->tsEnd in LALDemodTest.c */
/*  10/10/02 gam Fix round-off error problems when computing percentPad and percent clean lock */
/*  10/15/02 gam Add normalizeSFTs parameter. Default 0 means do nothing; 1 means normalize with S_h = 2*<|SFT|> in DeFT band; 2 means calibrate first, then normalize. */
/*  10/16/02 gam Add option if doDemod < 0 (and qc1Found > 0) then remove mean when generating SFTs. Works only when a quality channel is provided. */
/*  10/16/02 gam Update confidence for F stat; add stat_name = JKS F Stat and stat_value = F columns to signal_dperiodic table.  */
/*  10/17/02 gam Play with code in standalone mode by uncommenting INCLUDE_LALINITBARYCENTER and commenting INCLUDE_LALWRAPPERINITBARYCENTER. */
/*  10/22/02 gam Add option params->testType = 5 to read in test data from a file. Used in stand-alone mode only, must uncomment INCLUDE_TEST_DATA_FROM_FILE_CODE below. */
/*  10/31/02 gam Add code to calibrate SFTs when normalizeSFTs == 2; calibrated SFTs will then be normalized. */
/*  01/15/03 gam Change summvars output value to time REAL8. */
/*  01/15/03 gam Remove factor of 2 from definition of power spectral density, S_h, in EstimateMeanSpectralAmplitude */
/*  01/15/03 gam Fix possible bug in computing pwF0 */
/*  01/15/03 gam If normalizeSFTs > 1 then calibrate; if normalizeSFTs == 3 then calibrate only; do not normalize. */
/*               Thus if normalizeSFTs == 1 normalize only; if normalizeSFTs == 2 then calibrate and normalize; if normalizeSFTs == 3 calibrate only */
/*  02/03/03 gam If inputing SFTs, use input metadata from LDAS rather MoreTimes. */
/*  02/03/03 gam If doDemod == -2 then high pass filter and window time series data before generating SFTs. (Make doDemod == -1 then remove mean) */

/* TO DO LIST */
/* Improved quality channel code to better handle quality and cutoffs GW_CHANNEL_MAX_VALUE_EXPECTED and GW_CHANNEL_MIN_VALUE_EXPECTED */
/* Enhance MoreTimes to handle arbitary data drop outs and to zero pad missing data; rename to SetTimesHandleDropOuts */
/* Drop out includes data drop out, interferometer drop out of lock, and other data quality issues */
/*  Example data structure that gap_info is stored in:

                    <ilwd name='LDAS_History'>
                      <ilwd name='gap_info' size='5'>
                        <int_4u name='fill_method'>0</int_4u>
                        <int_4u dims='4' name='time_range'>688011799 999511718 688011801 0</int_4u>
                        <int_4u dims='4' name='time_range'>688011802 0 688011803 0</int_4u>
                        <int_4u dims='4' name='time_range'>688011804 0 688011805 0</int_4u>
                        <int_4u dims='4' name='time_range'>688011808 0 688011809 999999999</int_4u>
                      </ilwd>
                    </ilwd>

*/
/* Add amplitude modulation to testTypes 0 and 1 */

#define INCLUDE_DEMOD_CODE
#define INCLUDE_TEST_DATA_CODE
/* #define INCLUDE_TEST_DATA_FROM_FILE_CODE */

/* #define DEBUG_KNOWNPULSARDEMOD */
/* #define DEBUG_SFT_CODE */
/* #define DEBUG_OUTPUT_SFT_FILES */
/* #define DEBUG_DEMOD_CODE */
/* #define DEBUG_KNOWNPULSARDEMODINPUTFILES */
/* #define DEBUG_KNOWNPULSARDEMODOUTPUTFILES */
/* #define DEBUG_BUILDOUTPUT_CODE */
/* #define DEBUG_BUILDFRAMEOUTPUT_CODE */
/* #define DEBUG_BUILDDEMODFRAMEOUTPUT_CODE */
/* #define DEBUG_TEST_CODE */
/* #define DEBUG_TEST_DATA_FROM_FILE_CODE */
/* #define DEBUG_FRACREMAINING_CODE */
/* #define DEBUG_TIMEDELAY_CODE */
/* #define DEBUG_QUALITYCHANNEL_CODE */
/* #define DEBUG_INPUTSFT_CODE */
/* #define DEBUG_LALWRAPPERINITBARYCENTER */
/* #define DEBUG_NORMALIZESFTS */
/* #define DEBUG_CALIBRATESFTS */
/* #define DEBUG_HIGHPASSFILTERANDWINDOW */
/* UNCOMMENT ONLY ONE OF INCLUDE_LALINITBARYCENTER OR INCLUDE_LALWRAPPERINITBARYCENTER AT A TIME */
/* #define INCLUDE_LALINITBARYCENTER */
#define INCLUDE_LALWRAPPERINITBARYCENTER

#include <stdio.h>
#include <math.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
/* 10/31/02 gam Next two are used when calibrating SFTs */
#include <ExtractSeries.h>
#include <lal/VectorOps.h>
#include <lal/RealFFT.h>
#include <lal/LALConstants.h>
/* #include <lal/LALInitBarycenter.h> */ /* 07/01/02 gam Need to make this a local function in stand alone mode only. */
#include <lal/LALDemod.h>
#include <lal/Units.h>
#include <KnownPulsarDemod.h>
/* #include <BuildDB.h> */ /* 07/12/02 gam Not using LALCreateSearchSummaryDataBase */
/*  02/03/03 gam If doDemod == -2 then high pass filter and window time series data before generating SFTs. (Make doDemod == -1 then remove mean) */
#include <lal/BandPassTimeSeries.h>

NRCSID( KNOWNPULSARDEMODC, "$Id: KnownPulsarDemod.c,v 1.19 2003/02/05 01:36:46 gmendell Exp $");

/********************** prototypes for local static function definitions ********************/

#ifdef INCLUDE_LALINITBARYCENTER
 /*  07/01/02 gam */ /* Copied from LALInitBarycenter.c in the LAL support package.  Author: Curt Cutler */
 void LALInitBarycenter(LALStatus *stat, EphemerisData *edat);
#endif
/* 07/10/02 gam */ /* Function to transfer ephemeris data input as ilwd from LALSearchInput structure into LAL structures; based on LALInitBarycenter. */
void LALWRAPPERInitBarycenter(LALStatus *stat, LALSearchInput *inout, EphemerisData *edat);
/* void tdb(REAL8 alpha, REAL8 delta, REAL8 t_GPS, REAL8 *T, REAL8 *Tdot, const CHAR *sw); */
void tdb(REAL8 alpha, REAL8 delta, REAL8 t_GPS, REAL8 *T, REAL8 *Tdot); /*  10/08/01 gam */
/* void times(REAL8 tSFT, int howMany, LIGOTimeGPS *ts, INT4 sw); */
/* void MoreTimes(REAL8 tSFT, int howMany, LIGOTimeGPS *ts, gpsTimeInterval *timeIntervals, INT4 sw); */ /* 10/10/01 gam */
/* void MoreTimes(REAL8 tSFT, int howMany, UINT4 gpsStartTime, LIGOTimeGPS *ts, gpsTimeInterval *timeIntervals, INT4 sw); */
/* void MoreTimes(REAL8 tSFT, INT4 howMany, UINT4 gpsStartTime, LIGOTimeGPS *ts, gpsTimeInterval *timeIntervals, INT4 sw); */ /* 11/02/01 gam */
/* 08/06/02 gam */ /* Will compute sftPerCoh in MoreTimes, as in LALDemodTest times2 */
void MoreTimes(REAL8 tSFT, INT4 howMany, UINT4 gpsStartTime, LIGOTimeGPS *ts, INT4 **sftPerCoh, gpsTimeInterval *timeIntervals, INT4 sw); /* 11/02/01 gam */ /* 08/06/02 gam */
/* 11/10/01 gam */ /* Calculate stats associated with frequency series */
/* void CalcDemodPowerStats( const COMPLEX16Vector *fsVec, DemodPowerStats *stats, REAL8 fStart, REAL8 df ); */ /* 05/14/02 gam */
static void TimeToFloat(REAL8 *f, LIGOTimeGPS *tgps); /* 06/02/02 gam  Copied from LALDemodTest.c */
static void FloatToTime(LIGOTimeGPS *tgps, REAL8 *f); /* 06/02/02 gam  Copied from LALDemodTest.c */
void CalcDemodPowerStats( const COMPLEX8Vector *fsVec, DemodPowerStats *stats, REAL8 fStart, REAL8 df ); /* 05/14/02 gam */
/* 10/15/02 gam Next is used when params->normalizeSFTs > 0; finds mean spectral amplitude in a band specified by if0Min and if0Max from an input frequency series. */
void EstimateMeanSpectralAmplitude( const COMPLEX8Vector *fsVec, INT4 if0Min, INT4 if0Max, REAL4 *meanSpectralAmplitude ); /* 10/15/02 gam */
/* 11/27/01 gam */  /* Calculate the mean value of a time series with cutoffs */
/* void CalcMeanWithCutoffs( const REAL4Vector *tInputVec, REAL4 cutoffMin, REAL4 cutoffMax, REAL4 *meanWithCuttoffs); */
/* 11/11/01 gam */ /* Find index of input data that corresponds to gpsStartTime */
void GetStartTimeIndex(const gpsTimeInterval *times, const UINT4 *gpsStartTime, UINT4 *iStartTime);
REAL8 ComputePhase(REAL8 x, REAL8 y);
static void BuildOutput(LALStatus*            const status,
	    		const DemodSearchParams* const params,
	    		LALSearchOutput*      const output);
/*
static void BuildFrameOutput(LALStatus* const status,
	    const DemodSearchParams*    const params,
	    FFT                         **SFTData,
            gpsTimeInterval             *timeIntervals,
	    LALSearchOutput*            const output);
*/
/* 11/10/01 gam */ /* Add in SFTStats */
static void BuildFrameOutput(LALStatus* const status,
	    const DemodSearchParams*    const params,
	    FFT                         **SFTData,
            DemodPowerStats             **SFTStats,
            gpsTimeInterval             *timeIntervals,
	    LALSearchOutput*            const output);

/* 07/12/02 gam */ /* Similar to BuildFrameOutput above */ /* Output Demod JKS F Power Spectrum in Frame Format */
static void BuildDemodFrameOutput(LALStatus* const status,
	    const DemodSearchParams*    const params,
	    DeFTPeriodogram             **xHat,
            gpsTimeInterval             *timeIntervals,
	    LALSearchOutput*            const output);

/*  02/03/03 gam If doDemod == -2 then high pass filter and window time series data before generating SFTs. (Make doDemod == -1 then remove mean) */
void HighPassFilterAndWindow(LALStatus* const status, REAL4Vector *tvec, const DemodSearchParams* const params);

/* Initialization routine */
void
LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{
  /* Declare variables */
  DemodSearchParams *params;
  UINT2 i;
  REAL8 totalDeltaT; /* 11/10/01 gam  Changed from REAL4 to REAL8 */
  REAL8 shiftedFreq; /* 11/10/01 gam  Changed from REAL4 to REAL8 */

  INITSTATUS( status, "LALInitSearch", KNOWNPULSARDEMODC );

  #ifdef DEBUG_KNOWNPULSARDEMOD
  	fprintf(stdout, "\nLALInitSearch\n");
  	fflush(stdout);
  #endif

  /* Attach DemodSearchParams to searchParams */
  *searchParams = params = LALMalloc(sizeof( DemodSearchParams ));

  /* ----------------- parse the filter parameter arguments --------------*/
  /* ----------------- and initialise other parameters --------------------*/

  #ifdef DEBUG_KNOWNPULSARDEMOD
  	printf("\nargc = %i\n",initSearchParams->argc);
  #endif

  /* Parameters passed as command line arguments */
  /*  INT2    doDemod;                  */ /*  Case 0: compute SFTS, case 1 compute SFTs and do demod; case >= 2 input SFTs and do demod */ /* 07/07/02 gam */
  /*  INT4    mObsCoh                   Number of DeFTs to compute from input observation time */
  /*  UINT4   gpsStartTime              GPS start time of data requested */
  /*  REAL8   deltaT;                   time step in seconds = 1/sample rate */
  /*  UINT4   inputN;                   number of input data points */
  /*  REAL8   tSFT;                     duration in seconds of SFTs (= nSFT*deltaT = Time between start of one SFT and next SFT for sequential SFTs) */
  /*  INT4    nSFT;                     number of time domain data points used to compute one SFTs */ /* 10/11/01 gam */
  /*  UINT4   mCohSFTPerCall;            number of SFTs to compute during each call to ApplySearch */ /* 10/11/01 gam */
  /*  REAL8   templateRA;               Right Ascension of template */
  /*  REAL8   templateDec;              Declination of the template */
  /*  REAL8   templatePhase0;           initial or midpoint phase of template */
  /*  REAL8   templateFreq0;            initial or midpoint frequency of template */
  /*  REAL8   templateFreq0Band;        Band around f0 to compute DeFT */
  /*  REAL8   templateFreqDeriv0;       initial or midpoint first deriv of freq of template */
  /*  REAL8   templateFreqSecondDeriv0; initial or midpoint second deriv of freq of template */
  /*  REAL8   templateFreqThirdDeriv0;  initial or midpoint third deriv of freq of template */
  /*  REAL8   templateFreqFourthDeriv0; initial or midpoint fourth deriv of freq of template */
  /*  REAL8   templateFreqFifthDeriv0;  initial or midpoint fifth deriv of freq of template */
  /*  CHAR    generateTestData;         Generate test data: y = yes, n = no. */
  /*  INT2    testType;                 0 = signal only; 1 = with noise; 2 = add test signal to input data */
  /*  REAL8   testRA;                   Right Ascension of test */
  /*  REAL8   testDec;                  Declination of the test */
  /*  REAL8   testAmplitude;            Amplitude of the test */
  /*  REAL8   testPhase0;               phase of test */
  /*  REAL8   testFreq0;                frequency of test */
  /*  REAL8   testFreqDeriv0;           first deriv of freq of test */
  /*  REAL8   testFreqSecondDeriv0;     second deriv of freq of test */
  /*  REAL8   testFreqThirdDeriv0;      third deriv of freq of test */
  /*  REAL8   testFreqFourthDeriv0;     fourth deriv of freq of test */
  /*  REAL8   testFreqFifthDeriv0;      fifth deriv freq of test */
  /*  REAL8   testPolarization;         Polarization of test; NOT YET IMPLEMENTED */
  /*  INT4    testDeletions;            If deletions > 1 then gaps of deletions*tSFT occur between tSFT segments of test data */
  /*  CHAR    *ifoNickName;             07/18/02 gam dataConDesc is obsolete; change this to 2 character ifoNickName = H1, H2, or L1; note IFO is the site = LHO, LLO, GEO, etc.... */
  /*  CHAR    *IFO;                     Identifies the interferometer name */ /* 10/16/01 gam */
  /*  CHAR    *targetName;              Identifies the target of the search */ /* 10/16/01 gam */
  /*  CHAR    channel[dbNameLimit];     Channel Name */ /* 11/18/01 gam */
  /* INT4    inputSFTm;                 Number of input SFTs Normally this is the same as mCohSFT, but could be smaller if gaps are present */
  /* INT4    inputSFTn;                 Number of data points in the input SFT in the input frequency band */ /* 07/09/02 gam */
  /* REAL8   inputSFTf0;                Start frequency of the input SFTs */ /* 07/09/02 gam */
  /* REAL8   inputSFTband;              Band width of the input SFTs. */ /* 07/09/02 gam */
  /* INT2    outputDeFT;                if > 0 then will output DeFT into frame file */ /* 07/12/02 gam */
  /* INT2    normalizeSFTs;             10/15/02 gam Add normalizeSFTs parameter. Default 0 means do nothing; 1 means normalize with S_h = 2*<|SFT|> in DeFT band; 2 means calibrate first, then normalize. */

/* 07/09/02 gam */ /* Initialize */
 params->inputSFTm = 0;      /* Number of input SFTs */ /* 07/09/02 gam */ /* Normally this is the same as mCohSFT, but could be smaller if gaps are present */
 params->inputSFTn = 0;      /* Number of data points in the input SFT in the input frequency band */ /* 07/09/02 gam */
 params->inputSFTf0 = 0.0;   /* Start frequency of the input SFTs */ /* 07/09/02 gam */
 params->inputSFTband = 0.0; /* Band width of the input SFTs. */ /* 07/09/02 gam */
 params->outputDeFT = 0;     /* Default is 0; if > 0 then will output DeFT into frame file */

 params->normalizeSFTs = 0;  /* 10/15/02 gam Default 0 means do nothing; 1 means normalize with S_h = 2*<|SFT|> in DeFT band; 2 means calibrate first, then normalize. */

 /* 10/16/01 gam */
 params->ifoNickName = NULL; /* 11/05/01 gam */
 params->IFO = NULL;
 params->targetName = NULL;

 /* Set up name of this DSO */
 params->dsoName = NULL; /* 11/05/01 gam */
 params->dsoName = (CHAR *) LALMalloc( (strlen( "KPD_DSO" ) + 1) * sizeof(CHAR) );
 strcpy(params->dsoName, "KPD_DSO");
 strcpy(params->channel, "NotSet"); /* 11/18/01 gam Initialize */

 for (i = 1; i < initSearchParams->argc; i++) {

	#ifdef DEBUG_KNOWNPULSARDEMOD
		/* printf("\nargv[%i] = %g\n",i,atof(initSearchParams->argv[i])); */
	#endif

	switch(i) {
		/* case  1: params->doDemod = (BOOLEAN)atol(initSearchParams->argv[i]); break; */
		case  1: params->doDemod = (INT2)atoi(initSearchParams->argv[i]); break; /* 07/07/02 gam */
		case  2: params->mObsCoh = (INT4)atol(initSearchParams->argv[i]); break;
		case  3: params->gpsStartTime = (UINT4)atol(initSearchParams->argv[i]); break;
		case  4: params->deltaT = (REAL8)atof(initSearchParams->argv[i]); break; /* 10/10/01 gam */
		case  5: params->inputN = (UINT4)atol(initSearchParams->argv[i]); break;
		case  6: params->tSFT = (REAL8)atof(initSearchParams->argv[i]); break; /* 10/10/01 gam */
		case  7: params->nSFT = (INT4)atol(initSearchParams->argv[i]); break; /* 10/11/01 gam */
		case  8: params->mCohSFTPerCall = (UINT4)atol(initSearchParams->argv[i]); break; /* 10/11/01 gam */
		case  9: params->templateRA = (REAL8)atof(initSearchParams->argv[i]); break;
		case 10: params->templateDec = (REAL8)atof(initSearchParams->argv[i]); break;
		case 11: params->templatePhase0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 12: params->templateFreq0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 13: params->templateFreq0Band = (REAL8)atof(initSearchParams->argv[i]); break;
		case 14: params->templateFreqDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 15: params->templateFreqSecondDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 16: params->templateFreqThirdDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 17: params->templateFreqFourthDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 18: params->templateFreqFifthDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 19: params->generateTestData = initSearchParams->argv[i][0]; break;
		case 20: params->testType = (INT2)atoi(initSearchParams->argv[i]); break;
		case 21: params->testRA = (REAL8)atof(initSearchParams->argv[i]); break;
		case 22: params->testDec = (REAL8)atof(initSearchParams->argv[i]); break;
		case 23: params->testAmplitude = (REAL8)atof(initSearchParams->argv[i]); break;
		case 24: params->testPhase0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 25: params->testFreq0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 26: params->testFreqDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 27: params->testFreqSecondDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 28: params->testFreqThirdDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 29: params->testFreqFourthDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 30: params->testFreqFifthDeriv0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 31: params->testPolarization = (REAL8)atof(initSearchParams->argv[i]); break;
		case 32: params->testDeletions = (INT4)atol(initSearchParams->argv[i]); break;
                /* 11/05/01 gam */
		case 33: params->ifoNickName = (CHAR *) LALMalloc( (strlen( initSearchParams->argv[i] ) + 1) * sizeof(CHAR) );
		         strcpy(params->ifoNickName, initSearchParams->argv[i]); break;
		case 34: params->IFO = (CHAR *) LALMalloc( (strlen( initSearchParams->argv[i] ) + 1) * sizeof(CHAR) );
		         strcpy(params->IFO, initSearchParams->argv[i]); break;
		case 35: params->targetName = (CHAR *) LALMalloc( (strlen( initSearchParams->argv[i] ) + 1) * sizeof(CHAR) );
		         strcpy(params->targetName, initSearchParams->argv[i]); break;
		case 36: strcpy(params->channel, initSearchParams->argv[i]); break;
		case 37: params->inputSFTm = (INT4)atol(initSearchParams->argv[i]); break; /* 07/09/01 gam */
		case 38: params->inputSFTn = (INT4)atol(initSearchParams->argv[i]); break; /* 07/09/01 gam */
		case 39: params->inputSFTf0 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 40: params->inputSFTband = (REAL8)atof(initSearchParams->argv[i]); break;
		case 41: params->outputDeFT = (INT2)atoi(initSearchParams->argv[i]); break; /* 07/12/02 gam */
		case 42: params->normalizeSFTs = (INT2)atoi(initSearchParams->argv[i]); break; /* 10/15/02 gam */
	}
  } /* end for (i=1; i<argc; i++)  */

  #ifdef DEBUG_TEST_CODE
  	fprintf(stdout, "\nparams->generateTestData = %c\n",params->generateTestData);
  	fflush(stdout);
  #endif

  #ifdef DEBUG_INPUTSFT_CODE
	fprintf(stdout,"\n\nparams->inputSFTm = %i \n", params->inputSFTm);
	fprintf(stdout,"params->inputSFTn = %i \n", params->inputSFTn);
 	fprintf(stdout,"params->inputSFTf0 = %g \n", params->inputSFTf0);
  	fprintf(stdout,"params->inputSFTband = %g \n", params->inputSFTband);
   	fflush(stdout);
 #endif


  params->searchMaster = initSearchParams->nodeClass;
  params->rank         = 0;
  params->numSlaves    = 0;
  params->curSlaves    = 0;
  params->numNodes     = 0;
  /* strcpy(params->channel, ""); */ /* 11/18/01 gam Moved above */
  params->numDBOutput  = 0;  /* 07/12/02 gam */ /* Total number of rows going into all databases */
  params->vecInputData  = NULL;
  params->freqPWF0 = 0.0;       /* freq detected with maximum power */
  params->pwMax = 0.0;          /* power = absolute square of the amplitude at this freq */
  params->phaseForPWF0 = 0.0;   /* phase that goes with amplitude at max power */
  params->pwMean = 0.0;         /* mean power detected (mean computed less max power) */
  params->pwF0OverpwMean = 0.0; /* ratio of powMax to pwMean */ /* 11/05/01 gam */
  params->pwF0 = 0.0;           /* Power in the bin closest to templateFreq0 */   /* 11/05/01 gam */
  params->pwStdDev = 0.0;       /* Standard deviation in power (less pwF0) */ /* 11/05/01 gam */

  /* params->percentPad = 0.0; */ /* 11/27/01 gam */  /* Percent of input time series to produce each SFT was zero padded to fill in gaps */ /* 11/05/01 gam */
  params->percentPad = -0.01; /* 11/27/01 gam  will = -1% means not set*/ /* Percent of input time series to produce each SFT was zero padded to fill in gaps */ /* 11/05/01 gam */

  /* parameters for keeping track of which SFTs to do, and if all SFTs are done; used with mCohSFTPerCall */
  params->firstK = 0;                 /* Index of the first SFT to compute and store in the SFTData structure this call to ApplySearch */
  params->lastK = 0;                  /* Index of the last SFT to compute and store in the SFTData Structure this call to ApplySearch */
  params->finishedSFTs = 0;   /* Set equal to true when all SFTS for this job have been computed */

  params->ifMin = 0;      /* Index of minimum frequency in SFT band */
  params->fMin = 0.0;     /* min freq in SFT band */
  params->ifMax = 0;      /* Index of maximum frequency in SFT band */
  params->if0Min = 0;     /* Min search freq index in coarse-grained frequency resolution */
  params->f0Min = 0.0;    /* Min freq in coarse-grained freq resolution */
  params->if0Max = 0;     /* Max search freq index in coarse-grained frequency resolution */
  params->f0Max = 0.0;    /* Max freq in coarse-grained freq resolution */
  params->nDeFT = 0;     /* Number of bins in a DeFT */ /* 11/10/01 gam */

  params->edat = NULL;   /* 07/10/02 gam */ /* Initialize pointer to ephemeris data */

  if ( strstr(params->channel, "NotSet") ) {
      strcpy(params->channel, "H2\\:LSC-AS_Q"); /* This is the default */
  }

  /* Validate parameters */

  if (params->mObsCoh != 1) {
    ABORT( status, KNOWNPULSARDEMODH_EMOBSCOH, KNOWNPULSARDEMODH_MSGEMOBSCOH);
  }
  if (params->deltaT <= 0.0) {
    ABORT( status, KNOWNPULSARDEMODH_EDELTAT, KNOWNPULSARDEMODH_MSGEDELTAT);
  }
  if (params->inputN <= 0) {
    ABORT( status, KNOWNPULSARDEMODH_EINPUTN, KNOWNPULSARDEMODH_MSGEINPUTN);
  }

  /* if (params->tSFT <= 0.0 || params->tSFT > params->deltaT*((REAL4)(params->inputN - 1))) */
  /* 10/10/01 gam */
  /* if (params->tSFT <= 0.0 || params->tSFT > params->deltaT*((REAL8)(params->inputN - 1))) */
  /* 10/11/01 gam */
  if (params->tSFT <= 0.0 || params->tSFT > params->deltaT*((REAL8)params->inputN)) {
    ABORT( status, KNOWNPULSARDEMODH_ETSFT, KNOWNPULSARDEMODH_MSGETSFT);
  }

  /* 10/11/01 gam */
  if (params->nSFT <= 0 || params->nSFT > (INT4)params->inputN) {
    ABORT( status, KNOWNPULSARDEMODH_ENSFT, KNOWNPULSARDEMODH_MSGENSFT);
  }

  /* 10/11/01 gam */
  /* if ( params->tSFT != params->deltaT*(REAL8)params->nSFT ) */
  if (fabs((REAL8)((REAL8)params->tSFT - (REAL8)(params->deltaT*(REAL8)params->nSFT))) > 1.e-9*params->tSFT ) {
    ABORT( status, KNOWNPULSARDEMODH_ENTSFT, KNOWNPULSARDEMODH_MSGENTSFT);
  }
  /* Recompute tSFT to make it consistent with nSFT to double precision */
  params->tSFT = (REAL8)(((REAL8)params->deltaT)*((REAL8)params->nSFT));
  /* Check again! */
  if (fabs((REAL8)((REAL8)params->tSFT - (REAL8)(params->deltaT*(REAL8)params->nSFT))) > 1.e-9*params->tSFT ) {
    ABORT( status, KNOWNPULSARDEMODH_ENTSFT, KNOWNPULSARDEMODH_MSGENTSFT);
  }

  /* 10/11/01 gam */
  /* if (params->mCohSFTPerCall < 0) {} */
  /* 12/05/01 gam */
  if (params->mCohSFTPerCall <= 0) {
    ABORT( status, KNOWNPULSARDEMODH_EMCOHSFTPERCALL, KNOWNPULSARDEMODH_MSGEMCOHSFTPERCALL);
  }

  if (params->templateRA < 0 || params->templateRA > (REAL8)LAL_TWOPI) {
    /* Could be OK, but for now do not allow */
    ABORT( status, KNOWNPULSARDEMODH_ERA, KNOWNPULSARDEMODH_MSGERA);
  }

  /* 12/05/01 gam */
  /* if (params->templateDec < -1.0*(REAL8)LAL_PI_2 || params->templateRA > (REAL8)LAL_PI_2) {} */
  if (params->templateDec < -1.0*(REAL8)LAL_PI_2 || params->templateDec > (REAL8)LAL_PI_2) {
    ABORT( status, KNOWNPULSARDEMODH_EDEC, KNOWNPULSARDEMODH_MSGEDEC);
  }

  if (params->templateFreq0 - 0.5*params->templateFreq0Band < (REAL8)KNOWNPULSARDEMODH_LIGOBANDMIN ||
       params->templateFreq0 + 0.5*params->templateFreq0Band > (REAL8)KNOWNPULSARDEMODH_LIGOBANDMAX ||
       params->templateFreq0 + 0.5*params->templateFreq0Band > 0.5/params->deltaT) {

       ABORT( status, KNOWNPULSARDEMODH_EFREQ, KNOWNPULSARDEMODH_MSGEFREQ);
  }

  /* Estimate maximum shift in frequency during demodulation */

  /* totalDeltaT = params->deltaT*((REAL4)(params->inputN - 1)); */
  /* totalDeltaT = params->deltaT*((REAL8)(params->inputN - 1)); */ /* 10/10/01 gam */
  totalDeltaT = params->deltaT*((REAL8)params->inputN); /* 10/11/01 gam */
  shiftedFreq = params->templateFreq0;
  for (i = 1; i <= 5; i++) {
     /* shiftedFreq += ((REAL4)atof(initSearchParams->argv[23+i]))*pow(totalDeltaT,(REAL4)i); */
     /* shiftedFreq += ((REAL8)atof(initSearchParams->argv[25+i]))*pow(totalDeltaT,(REAL8)i); */ /* 10/11/01 gam */
     shiftedFreq += ((REAL8)atof(initSearchParams->argv[13+i]))*pow(totalDeltaT,(REAL8)i); /* 12/05/01 gam */
  }

  if  (shiftedFreq - 0.5*params->templateFreq0Band < (REAL8)KNOWNPULSARDEMODH_LIGOBANDMIN ||
       shiftedFreq + 0.5*params->templateFreq0Band > (REAL8)KNOWNPULSARDEMODH_LIGOBANDMAX ||
       shiftedFreq + 0.5*params->templateFreq0Band > 0.5/params->deltaT) {

       ABORT( status, KNOWNPULSARDEMODH_EFREQDERIV, KNOWNPULSARDEMODH_MSGEFREQDERIV);
  }

  if (params->generateTestData != 'y' && params->generateTestData != 'n') {
     ABORT( status, KNOWNPULSARDEMODH_EGENTEST, KNOWNPULSARDEMODH_MSGEGENTEST);
  }

  /* if (params->testType != 0 && params->testType != 1 && params->testType != 2) {} */ /* 11/02/01 gam */
  /* Note: testType is UINT2; cannot be negative */
  /* "Invalid test data type" */
  /* if (params->testType > 4) */ /* 10/22/02 gam */
  if (params->testType > 5) {
     ABORT( status, KNOWNPULSARDEMODH_ETESTTYPE, KNOWNPULSARDEMODH_MSGETESTTYPE);
  }

  /* 10/16/01 gam */

  /* "Invalid or null dataCondDesc" */ /* 11/05/01 gam */
  /* 07/18/02 gam dataConDesc is obsolete; changed this to 2 character ifoNickName */
  if ( !(params->ifoNickName) ){
    ABORT( status, KNOWNPULSARDEMODH_EDCONDESC, KNOWNPULSARDEMODH_MSGEDCONDESC);
  }

  /* "Invalid or null IFO" */
  if ( !(params->IFO) ){
    ABORT( status, KNOWNPULSARDEMODH_EIFO, KNOWNPULSARDEMODH_MSGEIFO);
  }

  /* "Invalid or null Target Name" */
  if ( !(params->targetName) ){
    ABORT( status, KNOWNPULSARDEMODH_ETARGETNAME, KNOWNPULSARDEMODH_MSGETARGETNAME);
  }

  /* "Invalid or null channel" */
  if ( strstr(params->channel, "NotSet") ){
    ABORT( status, KNOWNPULSARDEMODH_ECHANNEL, KNOWNPULSARDEMODH_MSGECHANNEL);
  }

  RETURN( status );

}

/* ConditionData routine */
void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams	  *mpiParams
    )
{

 DemodSearchParams* const params = (DemodSearchParams *) searchParams;

 INITSTATUS( status, "LALConditionData", KNOWNPULSARDEMODC );
 ATTATCHSTATUSPTR (status);

 /* 10/12/01 gam */
 MPI_Comm_rank( *(mpiParams->comm), &params->rank );

 MPI_Comm_size( *(mpiParams->comm), &params->numNodes); /* 07/12/02 gam */

 /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
 /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */
 if (params->searchMaster) {

  BOOLEAN dataFound = 0;

  /* BOOLEAN gapsFound = 0; */ /* FOR FUTURE USE: Found gap_info in the input mdd history record  */ /* 11/27/01 gam */
  BOOLEAN qc1Found = 0;  /* Found quality channel qc1 */ /* 11/27/01 gam */
  UINT4 qc1Index = 0;    /* The quality channel was found in inout->sequences[qc1Index].data.real4 */ /* 11/27/01 gam */
  BOOLEAN inputSFTsFound = 0; /* SFT data was found in inout->sequences[qc1Index].data.complex8 */ /* 07/07/02 gam */

  UINT4 i = 0;      /* all purpose index */
  UINT4 k = 0;      /* another all purpose index */
  UINT4 nData = 0;  /* 10/10/01 gam */
  UINT4 iStartTime = 0; /* 11/11/01 gam */

  /* Variables used to compute SFTs that are only needed in ConditionData */
  /* const INT4 measure = 1; */ /* Use 0 for faster estimated plan, nonzero for more exact plan. Used in new version LALCreate...FFTPlan() */
  const INT4 measure = 0;  /* 10/26/01 gam */ /* Use 0 for faster estimated plan, nonzero for more exact plan. Used in new version LALCreate...FFTPlan() */
  INT4 deletions=1;        /* Used when computing timeStamps.  For now hard code with default value.  BUT RESET IF GENERATING TEST DATA */


  for (i = 0; i < inout->numberSequences; ++i)
  {
    /* if (strstr(inout->sequences[i].name, "H2\\:LSC-AS_Q")) */ /* 11/18/01 gam */
    if (strstr(inout->sequences[i].name, params->channel))
    {

      params->times = inout->sequences[i].range.dTime;
      /* 11/11/01 gam */ /* Find index of input data that corresponds to gpsStartTime */
      GetStartTimeIndex(&params->times, &params->gpsStartTime, &iStartTime);
      #ifdef DEBUG_TIMEDELAY_CODE
  	fprintf(stdout, "\nFound channel, iStartTime = %i \n",iStartTime);
  	fflush(stdout);
      #endif
      /* 12/05/01 gam */
      if (iStartTime < 0 || iStartTime >= inout->sequences[i].dimensions[0]) {
          ABORT( status, KNOWNPULSARDEMODH_EISTARTTIME, KNOWNPULSARDEMODH_MSGEISTARTTIME);
      }

      /* const UINT4 nData = inout->sequences[i].dimensions[0]; */
      /* nData = inout->sequences[i].dimensions[0]; */ /* 10/10/01 gam */
      /* 11/11/01 gam */
      nData = inout->sequences[i].dimensions[0] - iStartTime;
      params->times.startSec = params->gpsStartTime;
      params->times.startNan = 0;

      LALSCreateVector(status->statusPtr, &(params->vecInputData), nData);

      /*
      memcpy(params->vecInputData->data, inout->sequences[i].data.real4,
	     sizeof(REAL4)*nData);
      */

      memcpy(params->vecInputData->data, (inout->sequences[i].data.real4 + iStartTime),
	     sizeof(REAL4)*nData);

      /* strcpy(params->channel, "H2:LSC-AS_Q"); */ /* 11/18/01 gam */

      params->unitSFT = lalADCCountUnit;  /* 10/12/01 gam */ /* HARD CODE FOR NOW; BUT SHOULD TRY SOMETHING LIKE CODE BELOW */
      /* const CHARVector *inputUnitString  = inout->sequences[i].units; */
      /* LALParseUnitString(status->statusPtr,&params->unitSFT,inputUnitString); */
      /* CHECKSTATUSPTR (status); */

      dataFound = 1;
      /* break; */ /* 11/27/01 gam */
      if (qc1Found && dataFound) {
      	break;
      }
    }
    else if (strstr(inout->sequences[i].name, "ifodmro"))
    {
      /* const INT2* dummyI2 = inout->sequences[i].data.int2s; */ /* 11/11/01 gam */
      INT2* dummyI2 = inout->sequences[i].data.int2s;

      params->times = inout->sequences[i].range.dTime;
      /* 11/11/01 gam */ /* Find index of input data that corresponds to gpsStartTime */
      GetStartTimeIndex(&params->times, &params->gpsStartTime, &iStartTime);
      #ifdef DEBUG_TIMEDELAY_CODE
  	fprintf(stdout, "\nFound ifodmro, iStartTime = %i \n",iStartTime);
  	fflush(stdout);
      #endif
      /* 12/05/01 gam */
      if (iStartTime < 0 || iStartTime >= inout->sequences[i].dimensions[0]) {
          ABORT( status, KNOWNPULSARDEMODH_EISTARTTIME, KNOWNPULSARDEMODH_MSGEISTARTTIME);
      }

      /* UINT4 k = 0; */ /* 10/12/01 gam */

      /* const UINT4 nData = inout->sequences[i].dimensions[0]; */
      /* nData = inout->sequences[i].dimensions[0]; */ /* 10/10/01 gam */

      /* 11/11/01 gam */
      nData = inout->sequences[i].dimensions[0] - iStartTime;
      params->times.startSec = params->gpsStartTime;
      params->times.startNan = 0;

      /*
       * All we want to do is convert the IFODMRO data from int to
       * float and get it to the ApplySearch routine
       */

      LALSCreateVector(status->statusPtr, &(params->vecInputData), nData);

      /*
       * Copy from int2s to real4
       */
      *dummyI2 = *dummyI2 + iStartTime; /* 11/11/01 gam */
      for (k = 0; k < params->vecInputData->length; ++k)
      {
	params->vecInputData->data[k] = (REAL4) *dummyI2++;
      }

      strcpy(params->channel, "ifodmro");

      params->unitSFT = lalVoltUnit;  /* 10/12/01 gam */ /* HARD CODE FOR NOW; BUT SHOULD TRY SOMETHING LIKE CODE BELOW */
      /* const CHARVector *inputUnitString  = inout->sequences[i].units; */
      /* LALParseUnitString(status->statusPtr,&params->unitSFT,inputUnitString); */
      /* CHECKSTATUSPTR (status); */

      dataFound = 1;
      /* break; */ /* 11/27/01 gam */
      if (qc1Found && dataFound) {
      	break;
      }
    }
    else if (strstr(inout->sequences[i].name, "qc1"))
    {
      /* 11/27/01 */ /* Found the quality channel qc1 */

      qc1Index = i;
      qc1Found = 1;
      if (qc1Found && dataFound) {
      	break;
      }

    }
    else if (strstr(inout->sequences[i].comment, "sft"))
    {
        /* 07/09/02 gam */ /* NOTE HARD CODED COMMENT ABOVE; SHOULD CHANGE */ /* SFT data was found. Check that this is what was expected and set flags */

        #ifdef DEBUG_INPUTSFT_CODE
    	  fprintf(stdout,"\n FOUND SFT DATA \n");
          fflush(stdout);
        #endif

        if (params->doDemod > 1) {
            /* params->doDemod > 1 means SFT data should be input; so set flags */
            params->unitSFT = lalADCCountUnit;  /* 10/12/01 gam */ /* HARD CODE FOR NOW; BUT SHOULD TRY SOMETHING LIKE CODE BELOW */
	    /* Need to set these now for error checking; but then reset below after SFTs are unpacked */
            /* params->times = inout->sequences[i].range.dTime; */ /* 02/03/03 gam range should be dFreq not dTime */ /* 07/09/02 gam */
	    /* 02/03/03 gam */ /* Next two lines were already revised below based on the actual SFTs times input (allows for missing SFTs)  */
            /* params->times.startSec = params->gpsStartTime; */
            /* params->times.startNan = 0; */
            /* params->times.timeStepSize = params->deltaT; */ /* 02/03/03 gam Moved below */
            dataFound = 1;
            inputSFTsFound = 1;    /* Set flag to unpack the data below after allocating memory */
	    break;  /* Will unpack SFT below */
	}

    }
    /* END if (strstr(inout->sequences[i].name, "H2\\:LSC-AS_Q")) elseif ... */
   }  /* END for (i = 0; i < inout->numberSequences; ++i) */

   /* "No input data was found and not generating test data" */ /* 07/09/02 gam */ /* Copied and modified from below */
   if (!dataFound && params->generateTestData == 'n') {
    		ABORT( status, KNOWNPULSARDEMODH_ENODATA, KNOWNPULSARDEMODH_MSGENODATA);
   }


    /* 10/12/01 gam */
    /* Finish initializing variables used to compute SFTs */

        /* IMPORTANT, note that I define nSFT, which is related to nDeltaF in LALDeModTest */
        /* IMPORTANT, note that I define nDeltaF = nSFT/2 + 1 */   /* 09/28/01 gam */
        /* nSFT is the number of time domain data points in the input to LALFORWARDREAL and nDeltaF is the number of output data points */
        /* IMPORTANT, note that I assume deltaT is the same as dtEFF in LALDeModTest */

         /* params->pfwd = NULL; */     /* Plan for computing SFTs */
         params->pfwdReal = NULL;       /* Plan for computing FFTs from real input. */ /* 09/28/01 gam */

         params->tCoh = params->deltaT*((REAL8)params->inputN); /* Duration of DeFTs.  For now, this is taken as the time input into this code */

         params->dfSFT = 1.0/params->tSFT;                        /* Freq resolution of SFT = 1/duration of SFTs  */

         /* params->mCohSFT = floor(params->tCoh/params->tSFT); */  /* 07/09/02 gam */ /* Moved below */ /* Number of SFTs in one DeFT. */
         /* params->mObsSFT = params->mCohSFT*params->mObsCoh;  */  /* 07/09/02 gam */ /* Moved below */  /* Total number of SFTs in the observation time. */

         /* params->nDeltaF = params->nSFT/2 + 1;  */ /* 07/09/02 gam */ /* Moved below */ /* Number of output data points in one SFT */ /* 09/28/01 gam */

         params->tvec = NULL;            /* Temporary holder in input when computing each SFT */ /* 09/28/01 gam */
         params->fvec = NULL;         /* Temporary holder of output when computing each SFT */

      /* If SFT data was found in inout->sequences[qc1Index].data.complex8 BEGIN UNPACKING SFT DATA HERE else will do as before. */ /* 07/07/02 gam */
      if (inputSFTsFound) {

	  /* 02/03/03 gam Define these temporary constants used below when setting SFT times */
	  REAL8 tempReal8tSFT1 = 0.0;
	  REAL8 tempReal8tSFT2 = 0.0;
	  LIGOTimeGPS tempLIGOTimeGPStSFT;

	 /* 07/09/02 gam */ /* Moved from above */
         params->mCohSFT = params->inputSFTm;                 /* Number of SFTs in one DeFT. */
         params->mObsSFT = params->mCohSFT*params->mObsCoh;   /* Total number of SFTs in the observation time. */

         params->nDeltaF = params->inputSFTn;                         /* Number of output data points in one SFT */ /* 09/28/01 gam */

          /* 10/12/01 gam */
          /* Finish allocating memory and computing parameters that only get computed once when computing SFTs */

          /***** START CALCULATE USEFUL QUANTITIES BASED ON LALDemodTest.c *****/

	  params->ifMin = 0;
	  /* params->ifMax = params->nSFT - 1; */
	  params->ifMax = params->nDeltaF - 1; /* 09/28/01 gam */

	  /* params->fMin=params->inputSFTf0 + params->dfSFT*params->ifMin; */   /* Minimum frequency for which search is conducted */
	  params->fMin=params->inputSFTf0;    /* 02/03/03 gam The start frequency of the SFTs in input as a filter parameter */  /* Minimum frequency for which search is conducted */
	  params->if0Max = ceil((params->templateFreq0 + params->templateFreq0Band/2.0 - params->inputSFTf0)/params->dfSFT);  /* Index of maximum frequency to include in DeFT */
	  params->f0Max=params->inputSFTf0 + params->dfSFT*params->if0Max;                        /* Max freq in DeFT */
          params->if0Min = floor((params->templateFreq0-params->templateFreq0Band/2.0 - params->inputSFTf0)/params->dfSFT);     /* Index of mimimum frequency to include in DeFT */
	  params->f0Min=params->inputSFTf0 + params->dfSFT*params->if0Min;                        /* Min freq in DeFT */

	  /* params->nDeFT = (params->if0Max-params->if0Min)*params->mCohSFT; */ /* Number of bins in a DeFT. */ /* 11/10/01 gam */
	  /* params->nDeFT = (params->if0Max-params->if0Min)*params->mCohSFT + 1; */ /* Number of bins in a DeFT. */ /* 11/10/01 gam */ /* 05/17/02 gam */
	  params->nDeFT = (params->if0Max-params->if0Min + 1)*params->inputSFTm; /* 07/09/02 gam Need 1 inside () */ /* Number of bins in a DeFT. */ /* 11/10/01 gam */ /* 05/17/02 gam */

	  params->funcName=tdb;    /* For now hard codes as is LALDemodTest */

 	  /* params->inputM = params->mObsSFT; */ /* 10/12/01 gam */ /* number of input data segments */ /* TO DO: Check that inputM is set up correctly */

         #ifdef DEBUG_INPUTSFT_CODE
          fprintf(stdout,"\n\nifMin = %i, ifMax = %i \n",params->ifMin, params->ifMax);
          fprintf(stdout,"if0Min = %i, if0Max = %i \n",params->if0Min,params->if0Max);
          fprintf(stdout,"f0Min = %g, f0Max = %g \n",params->f0Min, params->f0Max);
          /* fprintf(stdout,"nSFT = %i, tSFT = %g, dfSFT = %g, f0Band = %g \n",params->nSFT,params->tSFT,params->dfSFT,params->templateFreq0Band); */
          fprintf(stdout,"nSFT = %i, nDeltaF = %i, tSFT = %g, dfSFT = %g, f0Band = %g \n",params->nSFT,params->nDeltaF,params->tSFT,params->dfSFT,params->templateFreq0Band); /* 09/28/01 gam */
          fprintf(stdout,"mCohSFT = %i \n",params->mCohSFT);
          fprintf(stdout,"mObsSFT = %i, mObsCoh = %i \n",params->mObsSFT,params->mObsCoh);
          fprintf(stdout,"params->nDeFT %i \n\n",params->nDeFT);
          fflush(stdout);
         #endif

         /***** START CALL ROUTINE TO GENERATE TIMESTAMPS BASED ON LALDemodTest.c *****/

	  /* params->timeIntervals =(gpsTimeInterval *)LALMalloc(params->mObsSFT*sizeof(gpsTimeInterval)); */
	  params->timeIntervals =(gpsTimeInterval *)LALCalloc(params->mObsSFT, sizeof(gpsTimeInterval));

	  params->timeStamps=(LIGOTimeGPS *)LALMalloc(params->mObsSFT*sizeof(LIGOTimeGPS));
	  /* MoreTimes(params->tSFT, params->mObsSFT, params->gpsStartTime, params->timeStamps, params->timeIntervals, deletions); */ /* 10/10/01 gam */

	  /* MoreTimes(params->tSFT, params->mObsSFT, params->gpsStartTime, params->timeStamps, &params->sftPerCoh, params->timeIntervals, deletions); */ /* 08/06/02 gam */ /* 02/03/03 gam COMMENT OUT */

	  /* 02/03/03 gam  COMMENT OUT MoreTimes ABOVE.  Set params->sftPerCoh next as in MoreTimes. */
	                   /* Set params->timeStamps and params->timeIntervals from the SFT meta data itself when reading in the SFTs below */

          /* 02/03/03 gam  Set params->sftPerCoh next as in MoreTimes. */
	  /* 08/06/02 gam */ /* For now this is hard coded for just mObsCoh = 1 */
          params->sftPerCoh = (INT4 *)LALCalloc(1+1, sizeof(INT4));
          params->sftPerCoh[0]=0;
          params->sftPerCoh[1] = params->mObsSFT;

         /***** END CALL ROUTINE TO GENERATE TIMESTAMPS BASED ON LALDemodTest.c *****/

         /* 07/10/02 gam  */ /*  07/18/02 gam  All this moved below */
         /* ifdef INCLUDE_LALWRAPPERINITBARYCENTER */

	  /* params->edat = (EphemerisData *)LALMalloc(sizeof(EphemerisData)); */

	  /* 07/10/02 gam */ /* Call function to transfer ephemeris data input as ilwd from LALSearchInput structure into LAL structures; based on LALInitBarycenter. */
	  /* LALWRAPPERInitBarycenter(status->statusPtr, inout, params->edat); */
	  /* CHECKSTATUSPTR (status); */

	/* endif */

	 #ifdef DEBUG_INPUTSFT_CODE
    	  fprintf(stdout,"\n START UNPACKING SFT DATA \n\n");
          fflush(stdout);
         #endif

        /* Allocate memory for the SFTData structure */
    	params->SFTData=(FFT **)LALMalloc(params->mObsSFT*sizeof(FFT *));
    	for(i=0;i<params->mObsSFT;i++)
    	{
		params->SFTData[i]=(FFT*)LALMalloc(sizeof(FFT));

		/* 05/14/02 gam */
		/* params->SFTData[i]->fft=(COMPLEX16FrequencySeries *)LALMalloc(sizeof(COMPLEX16FrequencySeries));
		params->SFTData[i]->fft->data=(COMPLEX16Vector *)LALMalloc(sizeof(COMPLEX16Vector)); */
		params->SFTData[i]->fft=(COMPLEX8FrequencySeries *)LALMalloc(sizeof(COMPLEX8FrequencySeries));
		params->SFTData[i]->fft->data=(COMPLEX8Vector *)LALMalloc(sizeof(COMPLEX8Vector));

		/* params->SFTData[i]->fft->data->data=(COMPLEX16
			*)LALMalloc(params->nSFT*sizeof(COMPLEX16)); */
		/* params->SFTData[i]->fft->data->data=(COMPLEX16 *)LALMalloc(params->nDeltaF*sizeof(COMPLEX16)); */ /* 09/28/01 gam */ /* 05/14/02 gam */
		params->SFTData[i]->fft->data->data=(COMPLEX8 *)LALMalloc(params->nDeltaF*sizeof(COMPLEX8)); /* 05/14/02 gam */
    	} /* END for(i=0;i<params->mObsSFT;i++) */

         /* Loop through sequence again to find SFT data */
	 k = 0;  /* Use this as a counter of SFTs in the next loop. */
         params->firstK = 0;
         for (i = 0; i < inout->numberSequences; ++i)
         {
            if (strstr(inout->sequences[i].comment, "sft")) {
               nData = inout->sequences[i].dimensions[0];
	       params->lastK = params->firstK + nData/params->inputSFTn;
	       if (params->lastK > params->inputSFTm) params->lastK = params->inputSFTm;
    	       for(k=params->firstK; k<params->lastK; k++)
    	       {
                 memcpy(params->SFTData[k]->fft->data->data, (inout->sequences[i].data.complex8 + (k - params->firstK)*params->inputSFTn), params->inputSFTn*sizeof(COMPLEX8));

                 /* 02/03/03 gam START Assign meta data based on that input from LDAS */
	  	 if (k == params->firstK) {
		 	params->timeStamps[k].gpsSeconds = inout->sequences[i].range.dFreq.gpsStartTimeSec;
		 	params->timeStamps[k].gpsNanoSeconds = inout->sequences[i].range.dFreq.gpsStartTimeNan;

			TimeToFloat(&tempReal8tSFT1, &(params->timeStamps[k]));  /* Save this for k > firstK */

		 	params->timeIntervals[k].startSec = params->timeStamps[k].gpsSeconds;
                 	params->timeIntervals[k].startNan = params->timeStamps[k].gpsNanoSeconds;
                 	params->timeIntervals[k].stopSec = inout->sequences[i].range.dFreq.gpsStopTimeSec;
                 	params->timeIntervals[k].stopNan = inout->sequences[i].range.dFreq.gpsStopTimeNan;

			/* Verify meta data is correct */
     			if (inout->sequences[i].range.dFreq.startFreq != params->fMin) {
    				ABORT( status, KNOWNPULSARDEMODH_ESTARTFREQ, KNOWNPULSARDEMODH_MSGESTARTFREQ);
			}

     			if (inout->sequences[i].range.dFreq.freqStepSize != params->dfSFT) {
    				ABORT( status, KNOWNPULSARDEMODH_EFREQSTEPSIZE, KNOWNPULSARDEMODH_MSGEFREQSTEPSIZE);
			}

                 } else {
		        /* This is not used if SFT data arrives in separate containers for each SFT; only if concatinated together*/
          		tempReal8tSFT2 = tempReal8tSFT1 + ((REAL8)(k - params->firstK))*params->tSFT;
          		FloatToTime(&tempLIGOTimeGPStSFT, &tempReal8tSFT2);
		 	params->timeStamps[k].gpsSeconds = tempLIGOTimeGPStSFT.gpsSeconds;
		 	params->timeStamps[k].gpsNanoSeconds = tempLIGOTimeGPStSFT.gpsNanoSeconds;
          		tempReal8tSFT2 = tempReal8tSFT2 + params->tSFT;
          		FloatToTime(&tempLIGOTimeGPStSFT, &tempReal8tSFT2);
		 	params->timeIntervals[k].startSec = params->timeStamps[k].gpsSeconds;
                 	params->timeIntervals[k].startNan = params->timeStamps[k].gpsNanoSeconds;
                 	params->timeIntervals[k].stopSec = tempLIGOTimeGPStSFT.gpsSeconds;
                 	params->timeIntervals[k].stopNan = tempLIGOTimeGPStSFT.gpsNanoSeconds;
		 }
                 /* 02/03/03 gam END Assign meta data based on that input from LDAS */

		 params->SFTData[k]->fft->epoch=params->timeStamps[k];
		 params->SFTData[k]->fft->f0=params->fMin;
		 params->SFTData[k]->fft->deltaF=params->dfSFT;
		 params->SFTData[k]->fft->data->length=params->inputSFTn;
	       }
	       params->firstK = params->lastK;
	    }
            if (params->firstK >= params->inputSFTm) {
	      	params->finishedSFTs = 1;
	      	break;
	    }
	 }
         /* END reloop through sequences for SFT data: for (i = 0; i < inout->numberSequences; ++i) */

         /* Some requested input SFT data is missing */
         if (!params->finishedSFTs) {
    		ABORT( status, KNOWNPULSARDEMODH_EMISSINGSFTDATA, KNOWNPULSARDEMODH_MSGEMISSINGSFTDATA);
         }

	 /* 07/12/02 gam */ /* Revise these times based on time intervals actually found (allow for missing SFTs) */
         params->times.startSec = params->timeIntervals[0].startSec;
	 params->times.startNan = params->timeIntervals[0].startNan;
	 params->times.stopSec = params->timeIntervals[params->mObsSFT - 1].stopSec;
	 params->times.stopNan = params->timeIntervals[params->mObsSFT -1].stopNan;
         params->times.timeStepSize = params->deltaT; /* 02/03/03 gam Moved here from above */

	 #ifdef DEBUG_INPUTSFT_CODE
    	  fprintf(stdout,"\n FINISHED UNPACKING SFT DATA \n\n");
          fflush(stdout);
         #endif

        /* END UNPACKING SFTs.  Next carry on before if NOT inputSFTsFound */ /* 07/07/02 gam */
      } else {

         /* else do as before */ /* 07/09/02 gam */

	 /* 07/09/02 gam */ /* Moved from above */
         params->mCohSFT = floor(params->tCoh/params->tSFT);      /* Number of SFTs in one DeFT. */
         params->mObsSFT = params->mCohSFT*params->mObsCoh;       /* Total number of SFTs in the observation time. */

         params->nDeltaF = params->nSFT/2 + 1;                            /* Number of output data points in one SFT */ /* 09/28/01 gam */


        /* 10/12/01 gam */
        /* Finish allocating memory and computing parameters that only get computed once when computing SFTs */

        /***** START CALCULATE USEFUL QUANTITIES BASED ON LALDemodTest.c *****/

	  params->ifMin = 0;
	  /* params->ifMax = params->nSFT - 1; */
	  params->ifMax = params->nDeltaF - 1; /* 09/28/01 gam */

	  params->fMin=params->dfSFT*params->ifMin;                          /* Minimum frequency for which search is conducted */
	  params->if0Max = ceil((params->templateFreq0+params->templateFreq0Band/2.0)/params->dfSFT);      /* Index of maximum frequency to include in DeFT */
	  params->f0Max=params->dfSFT*params->if0Max;                        /* Max freq in DeFT */
          params->if0Min = floor((params->templateFreq0-params->templateFreq0Band/2.0)/params->dfSFT);     /* Index of mimimum frequency to include in DeFT */
	  params->f0Min=params->dfSFT*params->if0Min;                        /* Min freq in DeFT */

	  /* params->nDeFT = (params->if0Max-params->if0Min)*params->mCohSFT; */ /* Number of bins in a DeFT. */ /* 11/10/01 gam */
	  /* params->nDeFT = (params->if0Max-params->if0Min)*params->mCohSFT + 1; */ /* Number of bins in a DeFT. */ /* 11/10/01 gam */ /* 05/17/02 gam */
	  params->nDeFT = (params->if0Max-params->if0Min + 1)*params->mCohSFT; /* 07/09/02 gam Need 1 inside () */ /* Number of bins in a DeFT. */ /* 11/10/01 gam */ /* 05/17/02 gam */

	  params->funcName=tdb;    /* For now hard codes as is LALDemodTest */

 	  /* params->inputM = params->mObsSFT; */ /* 10/12/01 gam */ /* number of input data segments */ /* TO DO: Check that inputM is set up correctly */

         #ifdef DEBUG_DEMOD_CODE
          fprintf(stdout,"\n\nifMin = %i, ifMax = %i \n",params->ifMin, params->ifMax);
          fprintf(stdout,"if0Min = %i, if0Max = %i \n",params->if0Min,params->if0Max);
          fprintf(stdout,"f0Min = %g, f0Max = %g \n",params->f0Min, params->f0Max);
          /* fprintf(stdout,"nSFT = %i, tSFT = %g, dfSFT = %g, f0Band = %g \n",params->nSFT,params->tSFT,params->dfSFT,params->templateFreq0Band); */
          fprintf(stdout,"nSFT = %i, nDeltaF = %i, tSFT = %g, dfSFT = %g, f0Band = %g \n",params->nSFT,params->nDeltaF,params->tSFT,params->dfSFT,params->templateFreq0Band); /* 09/28/01 gam */
          fprintf(stdout,"mCohSFT = %i \n",params->mCohSFT);
          fprintf(stdout,"mObsSFT = %i, mObsCoh = %i \n",params->mObsSFT,params->mObsCoh);
          fprintf(stdout,"params->nDeFT %i \n\n",params->nDeFT);
          fflush(stdout);
         #endif

       /***** END CALCULATE USEFUL QUANTITIES BASED ON LALDemodTest.c *****/

       /***** START CALL ROUTINE TO GENERATE TIMESTAMPS BASED ON LALDemodTest.c *****/

	/* params->timeIntervals =(gpsTimeInterval *)LALMalloc(params->mObsSFT*sizeof(gpsTimeInterval)); */
	params->timeIntervals =(gpsTimeInterval *)LALCalloc(params->mObsSFT, sizeof(gpsTimeInterval));

	params->timeStamps=(LIGOTimeGPS *)LALMalloc(params->mObsSFT*sizeof(LIGOTimeGPS));
	/* MoreTimes(params->tSFT, params->mObsSFT, params->gpsStartTime, params->timeStamps, params->timeIntervals, deletions); */ /* 10/10/01 gam */
	MoreTimes(params->tSFT, params->mObsSFT, params->gpsStartTime, params->timeStamps, &params->sftPerCoh, params->timeIntervals, deletions); /* 08/06/02 gam */

      /***** END CALL ROUTINE TO GENERATE TIMESTAMPS BASED ON LALDemodTest.c *****/


        /* Create vector to hold frequency series */
    	LALCCreateVector(status->statusPtr, &params->fvec, (UINT4)params->nDeltaF); /* 09/28/01 gam */

        /* Create vector to hold time series */
    	LALSCreateVector(status->statusPtr, &params->tvec, (UINT4)params->nSFT);

        /* Allocate memory for the SFTData structure */
    	params->SFTData=(FFT **)LALMalloc(params->mObsSFT*sizeof(FFT *));
    	for(i=0;i<params->mObsSFT;i++)
    	{
		params->SFTData[i]=(FFT*)LALMalloc(sizeof(FFT));

		/* 05/14/02 gam */
		/* params->SFTData[i]->fft=(COMPLEX16FrequencySeries *)LALMalloc(sizeof(COMPLEX16FrequencySeries));
		params->SFTData[i]->fft->data=(COMPLEX16Vector *)LALMalloc(sizeof(COMPLEX16Vector)); */
		params->SFTData[i]->fft=(COMPLEX8FrequencySeries *)LALMalloc(sizeof(COMPLEX8FrequencySeries));
		params->SFTData[i]->fft->data=(COMPLEX8Vector *)LALMalloc(sizeof(COMPLEX8Vector));

		/* params->SFTData[i]->fft->data->data=(COMPLEX16
			*)LALMalloc(params->nSFT*sizeof(COMPLEX16)); */
		/* params->SFTData[i]->fft->data->data=(COMPLEX16 *)LALMalloc(params->nDeltaF*sizeof(COMPLEX16)); */ /* 09/28/01 gam */ /* 05/14/02 gam */
		params->SFTData[i]->fft->data->data=(COMPLEX8 *)LALMalloc(params->nDeltaF*sizeof(COMPLEX8)); /* 05/14/02 gam */
    	} /* END for(i=0;i<params->mObsSFT;i++) */

        /* 11/10/01 gam */ /* Allocate memory for the SFTStats structure */
    	params->SFTStats=(DemodPowerStats **)LALMalloc(params->mObsSFT*sizeof(DemodPowerStats *));
    	for(i=0;i<params->mObsSFT;i++)
    	{
		params->SFTStats[i]=(DemodPowerStats*)LALMalloc(sizeof(DemodPowerStats));
    	} /* END for(i=0;i<params->mObsSFT;i++) */

        /* Compute plan for FFTW */
        /* LALCreateForwardComplexFFTPlan(status->statusPtr, &params->pfwd, (UINT4)params->nSFT, measure); */
        LALCreateForwardRealFFTPlan( status->statusPtr, &params->pfwdReal, (UINT4)params->nSFT, measure ); /* 09/28/01 gam */

      }
      /* END if (inputSFTsFound) else... */

      /* 07/10/02 gam  */ /*  07/18/02 gam  Call LALWRAPPERInitBarycenter whenever doDemod > 0; moved here from above. */
      #ifdef INCLUDE_LALWRAPPERINITBARYCENTER

        if (params->doDemod > 0) {

	  params->edat = (EphemerisData *)LALMalloc(sizeof(EphemerisData));

	   /* 07/10/02 gam */ /* Call function to transfer ephemeris data input as ilwd from LALSearchInput structure into LAL structures; based on LALInitBarycenter. */
	  LALWRAPPERInitBarycenter(status->statusPtr, inout, params->edat);
	  CHECKSTATUSPTR (status);
	}

      #endif

      /* 10/15/01 gam */ /* Moved here */
      #ifdef INCLUDE_TEST_DATA_CODE
         if (params->generateTestData == 'y') {

	     /* variable used to generate test data */
	     REAL8 ts=0.0;
	     REAL8 tn=0.0;
	     REAL8 ts0=0.0;
	     REAL4 temp = 0.0;
	     REAL4Vector *noise = NULL;
	     static RandomParams *rndParams;
	     INT4 seed=0;
	     REAL8 baseTbary0, tDot, t, tBary;
	     REAL8 dTbary, dTbary2, dTbary3, dTbary4, dTbary5, dTbary6;

            /* IF ABOVE TRUE, USE TEST DATA OR ADD TEST DATA TO INPUT DATA */

            #ifdef DEBUG_TEST_CODE
  		 fprintf(stdout, "\nGENERATING TEST DATA FOR params->testType = %i\n",params->testType);
  		 fflush(stdout);
		 fprintf(stdout, "\ndataFound = %i\n",dataFound);
  		 fflush(stdout);
            #endif

	    /* Regenerate time stamps to allow gaps in test data */
	    deletions =	params->testDeletions;
	    /* MoreTimes(params->tSFT, params->mObsSFT, params->timeStamps, params->timeIntervals, deletions); */ /* 10/10/01 gam */
	    /* MoreTimes(params->tSFT, params->mObsSFT, params->gpsStartTime, params->timeStamps, params->timeIntervals, deletions); */
	    MoreTimes(params->tSFT, params->mObsSFT, params->gpsStartTime, params->timeStamps, &params->sftPerCoh, params->timeIntervals, deletions); /* 08/06/02 gam */

	     /* conversion: detector->barycenter time,  t=GPS_0 */
	    ts0=(REAL8)(params->timeStamps[0].gpsSeconds) + (REAL8)(params->timeStamps[0].gpsNanoSeconds)*1.0E-9;
	    /* (*params->funcName)((REAL8)params->testRA,(REAL8)params->testDec, ts0, &baseTbary0, &tDot, modulation); */
	    (*params->funcName)((REAL8)params->testRA,(REAL8)params->testDec, ts0, &baseTbary0, &tDot); /*  10/08/01 gam */

	    /* if (params->inputN != params->vecInputData->length) {} */ /* 10/30/01 gam */
	    if (params->inputN != nData) {
	    	if (params->testType != 2) {
		        /* Need to resize vecInputData */
			/* 10/30/01 gam */
			if (dataFound) {
	    			LALSDestroyVector(status->statusPtr, &(params->vecInputData));
			}
      			LALSCreateVector(status->statusPtr, &(params->vecInputData),params->inputN);
		} else {
			/* ERROR TEST DATA LENGTH MUST EQUAL INPUT DATA LENGTH FOR CASE 2; ADD ABORT HERE */
			/* "Invalid input data or inputN for test type 2" */
    			ABORT( status, KNOWNPULSARDEMODH_EDATATESTTWO, KNOWNPULSARDEMODH_MSGEDATATESTTWO); /* 10/30/01 gam */
		}
	    }

           #ifdef INCLUDE_TEST_DATA_FROM_FILE_CODE
                /* 10/22/02 gam Add option to read in test data from a file; used in stand-alone mode only. */

	       if (params->testType == 5) {
			FILE *inFID;
                        inFID = fopen("/home/gmendell/searchcode/src/lalwrapper/contrib/knownpulsardemod/test/TestIn.txt", "r");

			for(i=0; i < params->inputN; i++)
			{
	                     fscanf(inFID,"%e\n", &params->vecInputData->data[i]);
                             #ifdef DEBUG_TEST_DATA_FROM_FILE_CODE
			     	fprintf(stdout, "\n params->vecInputData->data[%i] = %g\n",i,params->vecInputData->data[i]);
  		 		fflush(stdout);
		             #endif
			}
		}

           #endif

	   if (params->testType == 5) {
                /* 10/22/02 gam Using option to read in test data from a file above; used in stand-alone mode only. */
	   } else {

    	    LALSCreateVector(status->statusPtr, &noise, params->inputN);
	    for(i=0;i<params->inputN;i++) {
	       noise->data[i]=0.0;  /* initialize */
	    }

	    /* if (params->testType == 1 || params->testType == 4) {} */ /* 11/02/01 gam */
	    if (params->testType == 1 || params->testType == 4) {
	       /* fill noise vector with normal deviates */
	       LALCreateRandomParams(status->statusPtr, &rndParams, seed);
	       LALNormalDeviates(status->statusPtr, noise, rndParams);
	       LALDestroyRandomParams(status->statusPtr, &rndParams);
	    }

	    for(k=0; k<params->mObsSFT; k++)
	    {
		ts=(REAL8)(params->timeStamps[k].gpsSeconds)*1.00;
		tn=(REAL8)(params->timeStamps[k].gpsNanoSeconds)*1.00E-9;

		for(i=0; i < params->nSFT; i++)
		{
			t=ts+tn+(REAL8)i*params->deltaT;

			/* conversion again */
			/* (*params->funcName)((REAL8)params->testRA, (REAL8)params->testDec, (REAL8)t, &tBary, &tDot, modulation); */
			(*params->funcName)((REAL8)params->testRA, (REAL8)params->testDec, (REAL8)t, &tBary, &tDot); /*  10/08/01 gam */

	    		if (params->testType <= 2) {
			   dTbary=tBary-baseTbary0;
			} else {
			   dTbary=t-ts0;  /* Do not amplitude or phase modulate data due to Earth's motion */ /* 11/02/01 gam */
			}
			/* pipeline for better performance */
			dTbary2=dTbary*dTbary;
			dTbary3=dTbary2*dTbary;
			dTbary4=dTbary3*dTbary;
			dTbary5=dTbary4*dTbary;
			dTbary6=dTbary5*dTbary;

			/* create time series according to test phase model */
			temp = params->testAmplitude*cos(LAL_TWOPI*(
				  params->testPhase0 +
			          params->testFreq0*dTbary +
				  0.5*params->testFreqDeriv0*dTbary2 +
				  (1.0/3.0)*params->testFreqSecondDeriv0*dTbary3 +
				  0.25*params->testFreqThirdDeriv0*dTbary4 +
				  0.20*params->testFreqFourthDeriv0*dTbary5 +
				  (1.0/6.0)*params->testFreqFifthDeriv0*dTbary6
				));
	    		/* if (params->testType == 0) {} */ /* 11/02/01 gam */
	    		if (params->testType == 0 || params->testType == 3) {
				params->vecInputData->data[i + k*params->nSFT] = temp;
	    		        /* {} else if (params->testType == 1) {} */ /* 11/02/01 gam */
	    		} else if (params->testType == 1 || params->testType == 4) {
				params->vecInputData->data[i + k*params->nSFT] = temp + noise->data[k*params->nSFT + i];
			} else if (params->testType == 2) {
				params->vecInputData->data[i + k*params->nSFT] = params->vecInputData->data[i + k*params->nSFT] + temp;
			}
		} /* end for(i=0; i < params->nSFT; i++) */
	     }	/* end for(k=0; k<params->mObsSFT; k++) */
	     LALDestroyVector(status->statusPtr, &noise);
	    }
	    /* END if (params->testType == 5) */ /* 10/22/02 gam Using option to read in test data from a file above; used in stand-alone mode only. */

             nData = params->inputN;
             /* strcpy(params->channel, "test"); */ /* 11/05/01 gam */
	     switch(params->testType) {
			case 0: strcpy(params->channel, "test0"); break;
			case 1: strcpy(params->channel, "test1"); break;
			case 2: strcpy(params->channel, "test2"); break;
			case 3: strcpy(params->channel, "test3"); break;
			case 4: strcpy(params->channel, "test4"); break;
			case 5: strcpy(params->channel, "test5"); break;
	     }
	     params->times.timeStepSize = params->deltaT;
	     params->times.startSec = params->timeIntervals[0].startSec;
	     params->times.startNan = params->timeIntervals[0].startNan;
	     params->times.stopSec = params->timeIntervals[params->mObsSFT - 1].stopSec;
	     params->times.stopNan = params->timeIntervals[params->mObsSFT -1].stopNan;
             params->unitSFT = lalVoltUnit;  /* STILL NEED TO DECIDE ON UNIT FOR TEST DATA */
             dataFound = 1;

         } /* END if (params->generateTestData == 'y') */
   #endif

   /* 10/10/01 gam */
   /* Check that data requested was found (if not generating test data) */
   if (params->generateTestData == 'n') {

	/* "No input data was found" */
        if (!dataFound) {
    		ABORT( status, KNOWNPULSARDEMODH_ENODATA, KNOWNPULSARDEMODH_MSGENODATA);
	}

        #ifdef DEBUG_KNOWNPULSARDEMOD
               fprintf(stdout,"\nnData = %i \n", nData);
               fflush(stdout);
        #endif

	/* "Invalid number of input data points" */
	/* if (params->inputN != nData) */ /* 07/09/02 gam */
	if (params->inputN != nData && params->doDemod < 2) {
    		ABORT( status, KNOWNPULSARDEMODH_ENDATA, KNOWNPULSARDEMODH_MSGENDATA);
	}

        #ifdef DEBUG_KNOWNPULSARDEMOD
               fprintf(stdout,"\ntimeStepSize = %g \n", params->times.timeStepSize);
               fflush(stdout);
        #endif

	/* "Incorrect input data time step" */
	if (params->times.timeStepSize != params->deltaT) {
    		ABORT( status, KNOWNPULSARDEMODH_ETIMESTEP, KNOWNPULSARDEMODH_MSGETIMESTEP);
	}

        #ifdef DEBUG_KNOWNPULSARDEMOD
               fprintf(stdout,"\ntimes.startSec = %i \n", params->times.startSec);
               fprintf(stdout,"times.startNan = %i \n", params->times.startNan);
               fflush(stdout);
        #endif

	/* "Incorrect input data start time" */
	/* 02/03/03 gam Check only if not inputting SFTs; actual input start time may differ from that requested if missing SFTs (gaps) */
        if (params->doDemod < 2) {
	  if (params->gpsStartTime != params->times.startSec || params->times.startNan != 0) {
    		ABORT( status, KNOWNPULSARDEMODH_ESTARTTIME, KNOWNPULSARDEMODH_MSGESTARTTIME);
	  }
	}

        #ifdef DEBUG_KNOWNPULSARDEMOD
               /* fprintf(stdout,"\nComputed stop time = %f \n",(REAL8)((REAL8)params->times.startSec + params->times.timeStepSize*(REAL8)(nData - 1))); */
 	       /* fprintf(stdout,"Input stop time    = %f \n",(REAL8)((REAL8)params->times.stopSec + (REAL8)1.0e-9*(REAL8)params->times.stopNan)); */
               /* fprintf(stdout,"Difference         = %g \n", (REAL8)((REAL8)params->times.stopSec + (REAL8)1.0e-9*(REAL8)params->times.stopNan)
	                 -  (REAL8)((REAL8)params->times.startSec + params->times.timeStepSize*(REAL8)(nData - 1)) ); */
	       /* 10/11/01 gam */
               fprintf(stdout,"\nComputed stop time = %f \n",(REAL8)((REAL8)params->times.startSec + params->times.timeStepSize*(REAL8)nData));
 	       fprintf(stdout,"Input stop time    = %f \n",(REAL8)((REAL8)params->times.stopSec + (REAL8)1.0e-9*(REAL8)params->times.stopNan));
               fprintf(stdout,"Difference         = %g \n", (REAL8)((REAL8)params->times.stopSec + (REAL8)1.0e-9*(REAL8)params->times.stopNan)
	                 -  (REAL8)((REAL8)params->times.startSec + params->times.timeStepSize*(REAL8)nData) );
               fflush(stdout);
        #endif

	/* "Incorrect input data stop time" */ /* NOTE 1.0e-10 SAFETY FACTOR */
	/* if ( fabs( (REAL8)((REAL8)params->times.startSec + params->times.timeStepSize*(REAL8)(nData - 1))
		   - (REAL8)((REAL8)params->times.stopSec + (REAL8)1.0e-9*(REAL8)params->times.stopNan) ) > 1.e-10*params->times.timeStepSize ) */
	/* 10/11/01 gam */
	/* 11/17/01 gam */ /* Do not vet stop time since it can be ambigous, e.g, can = start time + N*deltaT or start time + (N - 1)*deltaT. */
	/*
	if ( fabs( (REAL8)((REAL8)params->times.startSec + params->times.timeStepSize*(REAL8)nData)
	   - (REAL8)((REAL8)params->times.stopSec + (REAL8)1.0e-9*(REAL8)params->times.stopNan) )
	   > 1.0e-10*((REAL8)((REAL8)params->times.startSec + params->times.timeStepSize*(REAL8)nData)) ) {
   		ABORT( status, KNOWNPULSARDEMODH_ESTOPTIME, KNOWNPULSARDEMODH_MSGESTOPTIME);
	}
	*/

   } /* END if (params->generateTestData == 'n') */


   #ifdef DEBUG_QUALITYCHANNEL_CODE
     fprintf(stdout, "\nQuality Channel Info:  qc1Found = %i, qc1Index = %i \n",qc1Found,qc1Index);
     fflush(stdout);
   #endif

   /* 11/27/01 gam */ /* Handle the quality channel */ /* IN THE FUTURE ADD gapsFound code here too. */
   if (qc1Found) {

      gpsTimeInterval qc1Times = inout->sequences[qc1Index].range.dTime; /* The GPS start and end time of the quality channel */
      REAL8 qc1DeltaT = (REAL8) qc1Times.timeStepSize;                   /* Quality channel time step size */
      UINT4 gwSRateToqc1SRate = (UINT4)floor(qc1DeltaT/params->deltaT);  /* Ratio of gravity wave sample rate to quality channel sample rate */
      REAL4 gwMeanValue = 0.0;                                           /* Mean value with cutoffs stored in the gw channel */
      UINT4 qc1InputN = inout->sequences[qc1Index].dimensions[0];        /* Number of data points in quality channel from start time */
      UINT4 kpadStart = 0;
      UINT4 kpadEnd = 0;
      /* REAL4 gwChannelMinExpected = (REAL4)GW_CHANNEL_MIN_VALUE_EXPECTED; */ /* 02/07/02 gam */ /* disable use of cutoffs for now */
      /* REAL4 gwChannelMaxExpected = (REAL4)GW_CHANNEL_MAX_VALUE_EXPECTED; */ /* 02/07/02 gam */ /* disable use of cutoffs for now */
      REAL4 gwChannelSum = 0.0;  /* Sum of points of good quality */ /* And not drop out points in the future */
      UINT4 gwChannelCount = 0;  /* Count points of good quality */ /* And not drop out points in the future */

      iStartTime = 0;  /* Reinitialize */
      /* For now assume that quality channel starts at GPS time requested */
      /* GetStartTimeIndex(&qc1Times, &params->gpsStartTime, &iStartTime); */  /* Find the index that corresponds to the starting time */

      /* CalcMeanWithCutoffs( params->vecInputData, (REAL4)GW_CHANNEL_MIN_VALUE_EXPECTED, (REAL4)GW_CHANNEL_MAX_VALUE_EXPECTED, &gwMeanValue); */
      /* Calculate the mean, avoiding points of low quality and using cutoffs */
      /* Outer loop goes throught the quality channel, inner loop sums data for each point of good quality */
      for (i = iStartTime; i < qc1InputN; ++i)
      {
	    #ifdef DEBUG_QUALITYCHANNEL_CODE
              fprintf(stdout, "In sum code: i= %i, qc1 = %g \n",i, inout->sequences[qc1Index].data.real4[i]);
              fflush(stdout);
	    #endif

	    /* Check if quality OK */
      	    if (inout->sequences[qc1Index].data.real4[i] > 0.0) {

	       kpadStart = (i - iStartTime)*gwSRateToqc1SRate;
	       if (kpadStart >= params->vecInputData->length) break; /* We are past the end of the input gw data */
	       kpadEnd = (i + 1 - iStartTime)*gwSRateToqc1SRate;
	       if (kpadEnd > params->vecInputData->length) kpadEnd = params->vecInputData->length;

	      /* Sum up the good quality data */
              for (k = kpadStart; k < kpadEnd; ++k)
              {
                    /* 02/07/02 gam Fix bug in quality channel code; cutoffs were incorrectly applied; disable use of cutoffs for now. */
		    /* fabs was missing from if below.  However, not sure how fabs which for REAL8 casts REAL4 input. */ /* 02/07/02 gam */
    	            /* if ( fabs(params->vecInputData->data[k]) >= gwChannelMinExpected && fabs(params->vecInputData->data[k]) <= gwChannelMaxExpected ) { */
                       #ifdef DEBUG_QUALITYCHANNEL_CODE
                          fprintf(stdout, "k= %i, Summing vecInputData->data[k] = %g \n",k, params->vecInputData->data[k]);
                          fflush(stdout);
                       #endif
                       gwChannelSum += params->vecInputData->data[k];
		       gwChannelCount += 1;
                    /* } */ /* 02/07/02 gam */
              }
	      if (kpadEnd >= params->vecInputData->length) break; /* We are past the end of the input gw data */
	    }

      } /* END for (i = iStartTime; i < qc1InputN; ++i) */

      params->percentPad = (REAL4)( ( ((REAL4)params->vecInputData->length) - ((REAL4)gwChannelCount) )/((REAL4)params->vecInputData->length) );

      if (params->percentPad < 0.0) params->percentPad = 0.0; /* 10/10/02 gam Fix round-off error problems when computing percentPad and percent clean lock */

      if (gwChannelCount > 0) {
      	gwMeanValue = gwChannelSum/((REAL4)gwChannelCount);
      } else {
      	gwMeanValue = 0.0;  /* In the future should abort here ! */
      }

      #ifdef DEBUG_QUALITYCHANNEL_CODE
          fprintf(stdout, "\nqcDeltaT = %g \n",qc1DeltaT);
          fprintf(stdout, "gwSRateToqc1SRate = %i \n",gwSRateToqc1SRate);
          fprintf(stdout, "Quality Channel iStartTime = %i \n",iStartTime);
          fprintf(stdout, "Quality Channel qc1InputN = %i \n",qc1InputN);
          fprintf(stdout, "percentPad = %g \n",params->percentPad);
          fprintf(stdout, "gwMeanValue = %g \n\n",gwMeanValue);
          fflush(stdout);
      #endif

      /* Pad the data of poor quality */
      /* Outer loop goes throught the quality channel, inner loop pads data for each point of poor quality */
      for (i = iStartTime; i < qc1InputN; ++i)
      {
	    #ifdef DEBUG_QUALITYCHANNEL_CODE
              fprintf(stdout, "In pad code: i= %i, qc1 = %g \n",i, inout->sequences[qc1Index].data.real4[i]);
              fflush(stdout);
	    #endif

      	    /* if (inout->sequences[qc1Index].data.real4[i] == 0.0) {} */ /* 02/07/02 gam */
      	    if (inout->sequences[qc1Index].data.real4[i] <= 0.0) {

	       kpadStart = (i - iStartTime)*gwSRateToqc1SRate;
	       if (kpadStart >= params->vecInputData->length) break; /* We are past the end of the input gw data */
	       kpadEnd = (i + 1 - iStartTime)*gwSRateToqc1SRate;
	       if (kpadEnd > params->vecInputData->length) kpadEnd = params->vecInputData->length;

	      /* Pad the low quality data with the mean value */
              for (k = kpadStart; k < kpadEnd; ++k)
              {
                 #ifdef DEBUG_QUALITYCHANNEL_CODE
                      fprintf(stdout, "k= %i, replacing vecInputData->data[k] = %g \n",k, params->vecInputData->data[k]);
                      fflush(stdout);
	         #endif
	         params->vecInputData->data[k] = gwMeanValue;
              }
	      if (kpadEnd >= params->vecInputData->length) break; /* We are past the end of the input gw data */
	    }

      } /* END for (i = iStartTime; i < qc1InputN; ++i) */

      /* 10/16/02 gam Add option if doDemod < 0 (and qc1Found > 0) then remove mean when generating SFTs. Works only when a quality channel is provided. */
      /*  02/03/03 gam If doDemod == -2 then high pass filter and window time series data before generating SFTs. (Make doDemod == -1 then remove mean) */
      /* if (params->doDemod < 0) */
      if (params->doDemod == -1) {
              for (k = iStartTime*gwSRateToqc1SRate; k < params->vecInputData->length; ++k)
              {
                 #ifdef DEBUG_QUALITYCHANNEL_CODE
                      fprintf(stdout, "Subtracting mean value %g from vecInputData->data[%i] = %g \n",gwMeanValue,k,params->vecInputData->data[k]);
                      fflush(stdout);
	         #endif
	         params->vecInputData->data[k] -= gwMeanValue;  /* Remove the mean value */
              }
      }

   } /* END if (qc1Found) */

   /*  02/03/03 gam If doDemod == -2 then high pass filter and window time series data before generating SFTs. (Make doDemod == -1 then remove mean) */
   if (params->doDemod == -2) {

       #ifdef DEBUG_HIGHPASSFILTERANDWINDOW
	 fprintf(stdout, "\n TEST LINES HIGH PASS FILTER AND WINDOW; DATA BEFORE CALL TO HighPassFilterAndWindow: \n");
	 fflush(stdout);
         for(i=0; i < params->inputN; i++)
         {
	     	fprintf(stdout, "\n params->vecInputData->data[%i] = %g\n",i,params->vecInputData->data[i]);
         }
       #endif

       HighPassFilterAndWindow(status->statusPtr, params->vecInputData, params);

       #ifdef DEBUG_HIGHPASSFILTERANDWINDOW
	 fprintf(stdout, "\n TEST LINES HIGH PASS FILTER AND WINDOW; DATA AFTER CALL TO HighPassFilterAndWindow: \n");
	 fflush(stdout);
         for(i=0; i < params->inputN; i++)
         {
	     	fprintf(stdout, "\n params->vecInputData->data[%i] = %g\n",i,params->vecInputData->data[i]);
         }
       #endif

   }

 }
 /* END if (params->searchMaster) */

  RETURN( status );
}

/* Apply search routine */
void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
    )
{
  DemodSearchParams* const params = (DemodSearchParams *) applyParams->searchParams;

  UINT4          i = 0;         /* all purpose index */
  UINT4          k = 0;         /* another all purpose index */

  /* 07/09/02 gam */ /* Separate test for input and ouput since input now can include SFT data */
  #ifdef DEBUG_KNOWNPULSARDEMODINPUTFILES
   FILE           *inputDataDat = NULL;
  #endif

  #ifdef DEBUG_KNOWNPULSARDEMODOUTPUTFILES
   FILE           *outputDataDat = NULL;
  #endif

  #ifdef DEBUG_OUTPUT_SFT_FILES
   FILE           *SFT0Dat = NULL;
   FILE           *SFT1Dat = NULL;
   FILE           *SFT2Dat = NULL;
   FILE           *SFT3Dat = NULL;
  #endif

  INITSTATUS( status, "LALApplySearch", KNOWNPULSARDEMODC );
  ATTATCHSTATUSPTR(status);

  if ( !(output) ){
    ABORT( status, KNOWNPULSARDEMODH_ENULL, KNOWNPULSARDEMODH_MSGENULL);
  }

  /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
  /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */
  if (params->searchMaster) {

    output->numOutput = 0; /* 07/12/02 gam Initialize */

    #ifdef DEBUG_KNOWNPULSARDEMOD
      printf("\nI am the search master.  params->searchMaster = %i \n",params->searchMaster);
      printf("\nRank of this node = %i \n",params->rank);
      printf("\nNumber of nodes this job = %i \n", params->numNodes); /* 07/12/02 gam */
    #endif

    /* 07/09/02 gam */ /* Separate test for input and ouput since input now can include SFT data */
    #ifdef DEBUG_KNOWNPULSARDEMODINPUTFILES
  	inputDataDat = fopen("InputData.dat","w");
    #endif

    #ifdef DEBUG_KNOWNPULSARDEMODOUTPUTFILES
  	outputDataDat = fopen("OutputData.dat","w");
    #endif

    /* 07/09/02 gam */ /* Separate test for input and ouput since input now can include SFT data */
    #ifdef DEBUG_KNOWNPULSARDEMODINPUTFILES
       for (k = 0; k < params->vecInputData->length; ++k)
       {
  	   /* fprintf(stdout, "\nIn ApplySearch: vecInputData[%i] = %g \n",k,params->vecInputData->data[k]);
  	   fflush(stdout); */
  	   /* fprintf(inputDataDat, "%i %g \n",k,fabs(params->vecInputData->data[k]) + 1.0e-6); */
  	   fprintf(inputDataDat, "%g \n",params->vecInputData->data[k]);
  	   fflush(inputDataDat);
       }
    #endif

    /**** START COMPUTING SFTs modified from LALDemodTest.c ****/

	/* 10/15/01 gam */
        /*  Parameters below are for keeping track of which SFTs to do, and if all SFTs are done; used with params->mCohSFTPerCall */
        /*  params->firstK;  */ /* Index of the first SFT to compute and store in the SFTData structure this call to ApplySearch */
        /*  params->lastK;  */  /* Index of the last SFT to compute and store in the SFTData Structure this call to ApplySearch */
        /*  params->finishedSFTs; */ /* Set equal to true when all SFTS for this job have been computed */

        if (!params->finishedSFTs) {
	   params->lastK = params->firstK + params->mCohSFTPerCall;
	   if (params->lastK > params->mObsSFT) params->lastK = params->mObsSFT;
    	   /* for(k=0; k<params->mObsSFT; k++) */
    	   for(k=params->firstK; k<params->lastK; k++)
    	   {

		for(i=0;i<params->nSFT;i++)
		{
			/*
			params->tvec->data[i].re = params->vecInputData->data[i + k*params->nSFT];
			params->tvec->data[i].im = 0.0;
			*/
			params->tvec->data[i] = params->vecInputData->data[i + k*params->nSFT]; /* 09/28/01 gam */

			#ifdef DEBUG_SFT_CODE
			  /* fprintf(stdout,"k = %i, params->tvec->data[%i].re = %g\n",k,i,params->tvec->data[i].re); */
			  fprintf(stdout,"k = %i, tvec->data[%i] = %g\n",k,i,params->tvec->data[i]);/* 09/28/01 gam */
   			  fflush(stdout);
			#endif
		}

		/* Perform FFTW-LAL Fast Fourier Transform */
		/* LALCOMPLEX8VectorFFT(status->statusPtr, params->fvec, params->tvec, params->pfwd); */
    		LALForwardRealFFT( status->statusPtr, params->fvec, params->tvec, params->pfwdReal );

		/* write the data to the SFTData structure and normalize */
		/* for(i=0;i<(int)params->fvec->length;i++) */ /* 11/02/01 gam */ /* i is of time UINT4 */
		for(i=0;i<(UINT4)params->fvec->length;i++)
		{
                        /* 05/14/02 gam */
			/* params->SFTData[k]->fft->data->data[i].re=(REAL8)params->fvec->data[i].re*params->deltaT/sqrt(params->tSFT);
			params->SFTData[k]->fft->data->data[i].im=(REAL8)params->fvec->data[i].im*params->deltaT/sqrt(params->tSFT); */
			params->SFTData[k]->fft->data->data[i].re=(REAL4)(((REAL8)params->fvec->data[i].re)*params->deltaT/sqrt(params->tSFT));
			params->SFTData[k]->fft->data->data[i].im=(REAL4)(((REAL8)params->fvec->data[i].im)*params->deltaT/sqrt(params->tSFT));
		}

 		#ifdef DEBUG_SFT_CODE
		  for(i=0;i<params->fvec->length;i++)
		  {
			fprintf(stdout,"k = %i i = %i f = %g ps = %g\n",k,i,(params->ifMin+i)*params->dfSFT,params->SFTData[k]->fft->data->data[i].re*params->SFTData[k]->fft->data->data[i].re+params->SFTData[k]->fft->data->data[i].im*params->SFTData[k]->fft->data->data[i].im);
			fflush(stdout);
		  }
                #endif

		#ifdef DEBUG_OUTPUT_SFT_FILES
	         switch(k) {
			case 0: SFT0Dat = fopen("SFT0.dat","w"); break;
			case 1: SFT1Dat = fopen("SFT1.dat","w"); break;
			case 2: SFT2Dat = fopen("SFT2.dat","w"); break;
			case 3: SFT3Dat = fopen("SFT3.dat","w"); break;
		 }
		 for(i=0;i<params->fvec->length;i++) {
	            switch(k) {
			case 0: fprintf(SFT0Dat,"%f\t%f\n",params->SFTData[k]->fft->data->data[i].re,params->SFTData[k]->fft->data->data[i].im); fflush(SFT0Dat); break;
			case 1: fprintf(SFT1Dat,"%f\t%f\n",params->SFTData[k]->fft->data->data[i].re,params->SFTData[k]->fft->data->data[i].im); fflush(SFT1Dat); break;
			case 2: fprintf(SFT2Dat,"%f\t%f\n",params->SFTData[k]->fft->data->data[i].re,params->SFTData[k]->fft->data->data[i].im); fflush(SFT2Dat); break;
			case 3: fprintf(SFT3Dat,"%f\t%f\n",params->SFTData[k]->fft->data->data[i].re,params->SFTData[k]->fft->data->data[i].im); fflush(SFT3Dat); break;
		    }
		 }
	         switch(k) {
			case 0: fclose(SFT0Dat); break;
			case 1: fclose(SFT1Dat); break;
			case 2: fclose(SFT2Dat); break;
			case 3: fclose(SFT3Dat); break;
		 }
		#endif

		/* assign particulars to each SFT */
		params->SFTData[k]->fft->epoch=params->timeStamps[k];
		params->SFTData[k]->fft->f0=params->fMin;
		params->SFTData[k]->fft->deltaF=params->dfSFT;
		/* params->SFTData[k]->fft->data->length=params->nSFT; */
		params->SFTData[k]->fft->data->length=params->nDeltaF; /* 09/28/01 gam */
		/* 11/10/01 gam */ /* Calculate stat associated with this SFT */
		CalcDemodPowerStats( params->SFTData[k]->fft->data, params->SFTStats[k], params->fMin, params->dfSFT);
	   } /* END for(k=params->firstK; k<params->lastK; k++) */
        } /* END if (!params->finishedSFTs)  */
	params->firstK = params->lastK;
        if (params->firstK >= params->mObsSFT) params->finishedSFTs = 1;

	/* Finished computing SFTS; perform cleanup tasks */
	/* 10/12/01 gam */
	if (params->finishedSFTs) {
	   /* Set up units to agree with normalization of SFT data above */

           /* 01/28/02 gam */ /* Will need to fix code below to use new definition of LALUnitPair */
           /* 04/16/02 gam */ /* Complete code change to fix code to use new definition of LALUnitPair */

	   LALUnitPair  unitPair;   /* A pair of LALUnits */
	   RAT4         unitPower;  /* Rational number and demoninator minus one to compute power on a LALUnit */
           LALUnit     unitTmp;     /* 04/16/02 gam */ /* Temporary place holder for units. */

  	   /* unitPair.unitOne = lalADCCountUnit; */

	   /* 01/28/02 gam */ /* Will need to fix code below to use new definition of LALUnitPair */
           /* 04/16/02 gam */ /* Complete code change to fix code to use new definition of LALUnitPair */
	   /* unitPair.unitOne = params->unitSFT; */ /* initial unitSFT is set up in ConditionData; Now add in units due to normalization */
	   unitPair.unitOne = &(params->unitSFT);  /* 04/16/02 gam */ /* initial unitSFT is set up in ConditionData; Now add in units due to normalization */
  	   unitPower.numerator = -1;
  	   unitPower.denominatorMinusOne = 1;
  	   /* LALUnitRaise(status->statusPtr, &(unitPair.unitTwo), &lalHertzUnit, &unitPower ); */
  	   LALUnitRaise(status->statusPtr, &unitTmp, &lalHertzUnit, &unitPower ); /* 04/16/02 gam */
    	   CHECKSTATUSPTR (status);
	   unitPair.unitTwo = &unitTmp; /* 04/16/02 gam */
  	   LALUnitMultiply(status->statusPtr, &params->unitSFT, &unitPair);
    	   CHECKSTATUSPTR (status);

	   /* 07/09/02 gam */ /* Only need to deallocate if SFT generated within code */
           if (params->doDemod < 2) {
	      /* Free memory */
	      LALCDestroyVector(status->statusPtr, &params->fvec);
	      /* LALCDestroyVector(status->statusPtr, &params->tvec); */
	      LALSDestroyVector(status->statusPtr, &params->tvec);  /* 09/28/01 gam */
	      /* LALDestroyComplexFFTPlan(status->statusPtr, &params->pfwd); */
              LALDestroyRealFFTPlan( status->statusPtr, &params->pfwdReal ); /* 09/28/01 gam */
	   }
	}

    /**** END COMPUTING SFTs modified from LALDemodTest.c ****/

   /* 10/31/02 gam */  /* START Calibrate SFTs (normalizeSFTs == 2) */
   /* if (params->finishedSFTs && params->normalizeSFTs == 2) */
   /*  01/15/03 gam If normalizeSFTs > 1 then calibrate; if normalizeSFTs == 3 then calibrate only; do not normalize. */
   /*               Thus if normalizeSFTs == 1 normalize only; if normalizeSFTs == 2 then calibrate and normalize; if normalizeSFTs == 3 calibrate only */

   if (params->finishedSFTs && params->normalizeSFTs > 1) {

	const LALUnit strainPerCount = {0,{0,0,0,0,0,1,-1},{0,0,0,0,0,0,0}};
	LALUnitPair upair;
	COMPLEX8FrequencySeries response;

	response.data = NULL;
	strncpy( response.name, "response", sizeof( response.name ) );

 	LALCCreateVector( status->statusPtr, &response.data, params->nDeltaF );
  	CHECKSTATUSPTR( status );

	#ifdef DEBUG_CALIBRATESFTS
 		fprintf(stdout, "Calibrating SFTs \n");
   		fflush(stdout);
	#endif

	/* Copied and modified from src/lalwrapper/test/Test.c */

	for(k=0;k<params->mObsSFT;k++)
	{

		#ifdef DEBUG_CALIBRATESFTS
 			fprintf(stdout, "Calibrating SFTData[%i]\n",k);
   			fflush(stdout);
		#endif

		response.epoch = params->SFTData[k]->fft->epoch;
  		response.f0 = params->SFTData[k]->fft->f0;
  		response.deltaF = params->SFTData[k]->fft->deltaF;
  		response.sampleUnits = strainPerCount;

  		LALExtractResponse( status->statusPtr, &response, input, NULL );
  		/* CHECKSTATUSPTR( status ); */

		#ifdef DEBUG_CALIBRATESFTS
 			fprintf(stdout, "Response extracted; will now multipy times SFTData[%i]\n",k);
   			fflush(stdout);
		#endif

		LALCCVectorMultiply( status->statusPtr, params->SFTData[k]->fft->data, params->SFTData[k]->fft->data, response.data );
  		CHECKSTATUSPTR( status );

	}

	upair.unitOne = &params->unitSFT;
  	upair.unitTwo = &response.sampleUnits;
  	LALUnitMultiply( status->statusPtr, &params->unitSFT, &upair );
	CHECKSTATUSPTR( status );

 	LALCDestroyVector( status->statusPtr, &response.data );
  	CHECKSTATUSPTR( status );
   }
   /* 10/31/02 gam */  /* END Calibrate SFTs */

   /* 10/15/02 gam */  /* START normalizeSFTs */
   /* if (params->finishedSFTs && params->normalizeSFTs > 0) */
   /*  01/15/03 gam If normalizeSFTs > 1 then calibrate; if normalizeSFTs == 3 then calibrate only; do not normalize. */
   /*               Thus if normalizeSFTs == 1 normalize only; if normalizeSFTs == 2 then calibrate and normalize; if normalizeSFTs == 3 calibrate only */

   if (params->finishedSFTs && params->normalizeSFTs > 0 && params->normalizeSFTs < 3) {

	REAL4 sftNorm = 1.0; /* Default value does not change SFTs. */

	for(k=0;k<params->mObsSFT;k++)
	{
                EstimateMeanSpectralAmplitude(params->SFTData[k]->fft->data, params->if0Min, params->if0Max, &sftNorm);
                 #ifdef DEBUG_NORMALIZESFTS
                      fprintf(stdout, "Normalizing SFTData[%i], with MeanSpectralAmplitude = %g \n",k, sftNorm);
                      fflush(stdout);
	         #endif
		if (sftNorm > 0.0) {
			sftNorm = 1.0/sftNorm; /* Divide just once; then multiply below. */

			for(i=0;i<params->nDeltaF;i++)
			{
				params->SFTData[k]->fft->data->data[i].re = sftNorm*params->SFTData[k]->fft->data->data[i].re;
				params->SFTData[k]->fft->data->data[i].im = sftNorm*params->SFTData[k]->fft->data->data[i].im;
			}

		} else {
		       /* SFT could be all zeros; just continue */
		}
	}

        params->unitSFT = lalDimensionlessUnit; /* SFTData is now dimensionless */
   }
   /* end if (params->finishedSFTs && params->normalizeSFTs > 0) */
   /* 10/15/02 gam */  /* END normalizeSFTs */

 #ifdef INCLUDE_DEMOD_CODE
    /**** LALDemod CODE SECTION ENDS HERE ****/
    /* if (params->doDemod) */ 	/* 10/15/01 gam */
    /* if (params->doDemod && params->finishedSFTs) */ /* 07/07/02 gam */
    if (params->doDemod > 0 && params->finishedSFTs) {
    /**** START COMPUTE demParams based on LALDemodTest.c ****/

        /* variables for computing DeFTs */
	/* FFT **xHat; */ /* Container for output from LALDemod */
	DeFTPeriodogram **xHat;  /* 07/12/02 gam */ /* Container for output from LALDemod */ /* 05/16/02 gam */

	DemodPar *demParams;          /* Parameters needed for stitching together SFTs to produce coherent demodulated DFT = DeFT */
	ParameterSet *templateParams; /* For holding sky position and spin down parameters */
	CSParams *csParams;           /* Container for ComputeSky */
	INT4 iSkyCoh;                 /* For ComputeSky */
	/* const CHAR *modulation=NULL; */ /* For ComputeSky */ /* 05/14/02 gam */

	/* variables for computing maximum power, frequency for which maximum power occurs, and mean power */
	REAL8 pw = 0.0;
	INT4 ipwMax = 0;
	REAL8 pwSum= 0.0;
	INT4 ipwF0 = 0;  /* 11/05/01 gam */
	REAL8 pwSum2 = 0.0; /* 11/05/01 gam */
	REAL8 tmpF = 0.0; /* 11/05/01 gam */
	REAL8 tmpdF = 0.0; /* 11/05/01 gam */

        /* 06/02/02 gam  Copied from LALDemodTest.c */
        /* Quantities for use in LALBarycenter package */
        /* EphemerisData *edat=NULL; */  /* 07/10/02 gam Now a parameter; initialized in LALInitSearch */

        LALDetector cachedDetector;
        BarycenterInput baryinput;
        EarthState earth;
        EmissionTime emit;

        /* 05/17/02 gam Copied from LALDemodTest.c */
        /*  Variables for AM correction */
        AMCoeffsParams *amParams;
        AMCoeffs amc;

	/* Allocate space for template parameters */
	templateParams=LALMalloc(sizeof(ParameterSet));
	templateParams->spind=LALMalloc(sizeof(Spindown));
	templateParams->skyP=LALMalloc(2*sizeof(SkyPos));
	templateParams->spind->spParams=LALMalloc(5*sizeof(REAL8));

	templateParams->spind->m = 5;
	templateParams->spind->spParams[0] = params->templateFreqDeriv0;
	templateParams->spind->spParams[1] = params->templateFreqSecondDeriv0;
	templateParams->spind->spParams[2] = params->templateFreqThirdDeriv0;
	templateParams->spind->spParams[3] = params->templateFreqFourthDeriv0;
	templateParams->spind->spParams[4] = params->templateFreqFifthDeriv0;
	templateParams->skyP->alpha = params->templateRA;
	templateParams->skyP->delta = params->templateDec;

	/* BEGIN BARYCENTER */ /* 06/02/02 gam */
        /* 06/02/02 gam Include code for barycentering; copied from LALDemodTest.c */


        #ifdef INCLUDE_LALINITBARYCENTER
       /*  07/01/02 gam */ /* Copied from LALInitBarycenter.c in the LAL support package.  Author: Curt Cutler */

            /* Quantities computed for barycentering */
	    /* edat=(EphemerisData *)LALMalloc(sizeof(EphemerisData)); */
            params->edat = (EphemerisData *)LALMalloc(sizeof(EphemerisData)); /* 07/10/02 gam */
	    /* (*edat).ephiles.earthEphemeris = "earth98.dat";
            (*edat).ephiles.sunEphemeris = "sun98.dat"; */
	    /* 06/06/02 gam */
            /* 06/11/02 gam */

	    /* (*edat).ephiles.earthEphemeris = "/home/gmendell/searchcode/share/lal/earth01.dat";
            (*edat).ephiles.sunEphemeris = "/home/gmendell/searchcode/share/lal/sun01.dat"; */

	    (*params->edat).ephiles.earthEphemeris = "/home/gmendell/searchcode/share/lal/earth02.dat"; /* 07/10/02 gam */
            (*params->edat).ephiles.sunEphemeris = "/home/gmendell/searchcode/share/lal/sun02.dat";     /* 07/10/02 gam */

	    /* (*params->edat).ephiles.earthEphemeris = "/home/gmendell/searchcode/share/lal/earth98.dat"; */ /* 07/10/02 gam */
            /* (*params->edat).ephiles.sunEphemeris = "/home/gmendell/searchcode/share/lal/sun98.dat";     */ /* 07/10/02 gam */

            /* Read in ephemerides */
            /* LALInitBarycenter(&status, edat); */
            /* LALInitBarycenter(status->statusPtr, edat); */
            LALInitBarycenter(status->statusPtr, params->edat); /* 07/10/02 gam */
    	    CHECKSTATUSPTR (status);

	#endif

        /*Getting detector coords from DetectorSite module of tools package */

        /* Cached options are:
           LALDetectorIndexLHODIFF, LALDetectorIndexLLODIFF,
           LALDetectorIndexVIRGODIFF, LALDetectorIndexGEO600DIFF,
           LALDetectorIndexTAMA300DIFF,LALDetectorIndexCIT40DIFF
        */
        if (strstr(params->IFO, "LHO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexLHODIFF];
        } else if (strstr(params->IFO, "LLO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexLLODIFF];
        } else if (strstr(params->IFO, "GEO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexGEO600DIFF];
        } else if (strstr(params->IFO, "VIRGO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexVIRGODIFF];
        } else if (strstr(params->IFO, "TAMA")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexTAMA300DIFF];
        } else {
             /* "Invalid or null IFO" */
             ABORT( status, KNOWNPULSARDEMODH_EIFO, KNOWNPULSARDEMODH_MSGEIFO);
        }

        baryinput.site.location[0]=cachedDetector.location[0]/LAL_C_SI;
        baryinput.site.location[1]=cachedDetector.location[1]/LAL_C_SI;
        baryinput.site.location[2]=cachedDetector.location[2]/LAL_C_SI;
        baryinput.alpha=params->templateRA;
        baryinput.delta=params->templateDec;
        baryinput.dInv=0.e0;  /* 06/02/02 gam  Inverse distance to the source.  Not sure what 0.0 does. */

	/* END BARYCENTER */ /* 06/02/02 gam */

	/* BEGIN AMPLITUDE MODULATION */ /* 05/17/02 gam */

        /* 05/17/02 gam Include code for amplitude demodulation; copied from LALDemodTest.c */
	/* 06/02/02 gam Starting on this again from scratch; was not finished on 05/17/02 */

	/* Allocate space for amParams stucture */
        /* Here, amParams->das is the Detector and Source info */
        amParams = (AMCoeffsParams *)LALMalloc(sizeof(AMCoeffsParams));
        amParams->das = (LALDetAndSource *)LALMalloc(sizeof(LALDetAndSource));
        amParams->das->pSource = (LALSource *)LALMalloc(sizeof(LALSource));
        /* amParams->das->pSource->name = NULL; */ /* 07/07/02 gam */
        /* amParams->das->pSource->name = (CHAR *) LALMalloc(sizeof(CHAR)); */ /* 07/07/02 gam */
        /* amParams->das->pSource->name = params->targetName; */ /* 07/07/02 gam */
        /* strcpy(amParams->das->pSource->name, params->targetName); */ /* 07/07/02 gam */
	/* LALRealloc(amParams->das->pSource->name,0); */ /* 07/07/02 gam */
        /* Fill up AMCoeffsParams structure */
        amParams->baryinput = &baryinput;
        amParams->earth = &earth;
        /* amParams->edat = edat; */
        amParams->edat = params->edat; /* 07/10/02 gam */
        amParams->das->pDetector = &cachedDetector;
        /* LALDemodTest has: */
        /*  amParams->das->pSource->equatorialCoords.latitude = signalParams->skyP->alpha; */
        /*  amParams->das->pSource->equatorialCoords.longitude = signalParams->skyP->delta; */
	/* But, using my parameters, it should be: */
        /* amParams->das->pSource->name = NULL;  */ /* 07/07/02 gam */
        amParams->das->pSource->equatorialCoords.latitude = params->templateDec;
        amParams->das->pSource->equatorialCoords.longitude = params->templateRA;
        amParams->das->pSource->equatorialCoords.system = COORDINATESYSTEM_EQUATORIAL;
        amParams->das->pSource->orientation = 0.0;  /* 06/02/02 gam  Another form of the polarization angle. See LALSource documentation. See note below. */
        amParams->polAngle = 0.0; /* 06/02/02 gam  Polarization angle. */
	/* Any angle should work since Jaranowski, Krolak, and Schutz a(t) and b(t) are independent of psi and only need a(t) and b(t) */
	/* amParams->tObs = (REAL4)(((REAL8)(params->mObsCoh))*params->tCoh); */ /* 09/18/02 gam No longer in this structure */
	/* 09/18/02 gam Parameter amParams->mObsSFT is now in LALComputeAM.h (= params->sftPerCoh[params->mObsCoh]). */
        amParams->mObsSFT = params->sftPerCoh[params->mObsCoh]; /* 09/18/02 gam */

        /* Allocate space for AMCoeffs */
        amc.a = NULL;
        amc.b = NULL;
        /* LALSCreateVector(&status, &(amc.a), (UINT4)params->mObsSFT);
        LALSCreateVector(&status, &(amc.b), (UINT4)params->mObsSFT); */
        LALSCreateVector(status->statusPtr, &(amc.a), (UINT4)params->mObsSFT);
        CHECKSTATUSPTR (status);
        LALSCreateVector(status->statusPtr, &(amc.b), (UINT4)params->mObsSFT);
        CHECKSTATUSPTR (status);

        /* LALDemodTest says:
         * Compute timestamps for middle of each ts chunk
         * Note, we have decided to use the values of 'a' and 'b'
         * that correspond to the midpoint of the timeSeries for
         * which the SFT denotes.  In practice, this can be a flaw
         * since 'a' is a sinusoid with T=45ksec and T_b=90ksec; both
         * of these have amplitude about 0.4.  Thus, for 'a', on a timescale
         * of 10ksec, the value changes significantly.
        */ /* I DO NOT THINK THIS IS RIGHT. 06/02/02 gam */

       {
          LIGOTimeGPS midTS[params->mObsSFT];

	  REAL8 tempT = 0.0;  /* 09/18/02 gam  Used below as temporary storage for a time; called just t in LALDemodTest.c */

          /* 09/18/02 gam Add in params: amParams->ts0 and amParams->tsEnd as in LALDemodTest.c */
          /* compute timestamps of end and beg */
          amParams->ts0 = params->timeStamps[0];
          TimeToFloat(&tempT, &(params->timeStamps[0]));
          tempT += ((REAL8)(params->mObsCoh))*params->tCoh;  /* The observation time */
          FloatToTime(&(amParams->tsEnd), &tempT);

          for(k=0; k<params->mObsSFT; k++)
          {
	    REAL8 teemp=0.0;

	    TimeToFloat(&teemp, &(params->timeStamps[k]));
	    /* teemp += (REAL8)nDeltaF*dfSFT; */  /* 06/02/02 gam In LALDemodTest.c */
	    teemp += (REAL8)0.5*params->tSFT;  /* Go to the time midway through this SFT */
	    FloatToTime(&(midTS[k]), &teemp);
	    /* Test line printf("\n timeStamp[k] = %i %i , teemp = %g, midTS[k] = %i %i \n",
	           params->timeStamps[k].gpsSeconds, params->timeStamps[k].gpsNanoSeconds,teemp,midTS[k].gpsSeconds,midTS[k].gpsNanoSeconds); */
          }
          /* Compute the AM coefficients */
          /* LALComputeAM(&status, &amc, midTS, amParams); */
          LALComputeAM(status->statusPtr, &amc, midTS, amParams);
    	  CHECKSTATUSPTR (status);
       }

       /* END AMPLITUDE MODULATION */


	/* Allocate space and set quantity values for demodulation parameters */

	demParams=(DemodPar *)LALMalloc(sizeof(DemodPar));
	demParams->skyConst=(REAL8 *)LALMalloc((2*templateParams->spind->m*(params->mObsSFT+1)+2*params->mObsSFT+3)*sizeof(REAL8));
	demParams->spinDwn=(REAL8 *)LALMalloc(templateParams->spind->m*sizeof(REAL8));
	demParams->if0Max=params->if0Max;
	demParams->if0Min=params->if0Min;
	demParams->mCohSFT=params->mCohSFT;
	demParams->mObsCoh=params->mObsCoh;
	demParams->ifMin=params->ifMin;
	demParams->spinDwnOrder=templateParams->spind->m;
        demParams->amcoe = &amc; /* 05/16/01 gam */
        demParams->sftPerCoh = params->sftPerCoh; /* 08/06/02 gam */ /* Array with mCohSFT, unless gaps are present */

	/* Changed this from signalParams to templateParams */
       	for(i=0;i<templateParams->spind->m;i++)
	{
		demParams->spinDwn[i]=templateParams->spind->spParams[i];
	}

        #ifdef DEBUG_DEMOD_CODE
         for(i=0;i<demParams->spinDwnOrder;i++)
         {
      	   fprintf(stdout,"demParams->spinDwn[%i] = %g \n",i,demParams->spinDwn[i]);
      	   fflush(stdout);
         }
        #endif


	/* Allocate space and set quantities for call to ComputeSky() */
	csParams=(CSParams *)LALMalloc(sizeof(CSParams));
	csParams->skyPos=(REAL8 *)LALMalloc(2*sizeof(REAL8));
	csParams->skyPos[0]=templateParams->skyP->alpha;
	csParams->skyPos[1]=templateParams->skyP->delta;
	csParams->tGPS=params->timeStamps;
	csParams->spinDwnOrder=templateParams->spind->m;
	csParams->mObsSFT=params->mObsSFT;
	csParams->tSFT=params->tSFT;
	/* 06/02/02 gam  Added from LALDemodTest.c */
        /* csParams->edat=edat; */
        csParams->edat=params->edat; /* 07/10/02 gam */
        csParams->emit=&emit;
        csParams->earth=&earth;
        csParams->baryinput=&baryinput;

	/* 05/14/02 gam */ /* No longer used */
	/* csParams->sw=modulation;
	csParams->funcName=params->funcName; */

	iSkyCoh=0;  /* Index for sky patch NOT USED IN THIS CODE */

        for(i=0;i<(2*templateParams->spind->m*(params->mObsSFT+1)+2*params->mObsSFT+3);i++)
        {
      	  demParams->skyConst[i] = 0.0; /* Initialize */
        }


	/* Call COMPUTESKY() */
	ComputeSky(status->statusPtr, demParams->skyConst, iSkyCoh, csParams);
    	CHECKSTATUSPTR (status);

	/* Deallocate space for ComputeSky parameters */
	LALFree(csParams->skyPos);
	LALFree(csParams);

    #ifdef DEBUG_DEMOD_CODE
      for(i=0;i<(2*templateParams->spind->m*(params->mObsSFT+1)+2*params->mObsSFT+3);i++)
      {
      	fprintf(stdout,"demParams->skyConst[%i] = %g \n",i,demParams->skyConst[i]);
      	fflush(stdout);
      }
    #endif

    /**** END COMPUTE demParams based on LALDemodTest.c ****/

    /* Allocate memory for demodulated data */
/* 05/16/02 gam Change xHat to type DeFTPeriodogram which contains a REAL8FrequencySeries and a ParameterSet. */
/*    xHat=(FFT **)LALMalloc(params->mObsCoh*sizeof(FFT *));
    for(i=0; i<params->mObsCoh; i++)
    {
	   xHat[i]=(FFT *)LALMalloc(sizeof(FFT)); */
           /* 05/14/02 gam */
	   /*xHat[i]->fft=(COMPLEX16FrequencySeries *)LALMalloc(sizeof(COMPLEX16FrequencySeries));
	   xHat[i]->fft->data=(COMPLEX16Vector *)LALMalloc(sizeof(COMPLEX16Vector));
	   xHat[i]->fft->data->data=(COMPLEX16*)LALMalloc((params->nDeFT)*sizeof(COMPLEX16)); */
/*	   xHat[i]->fft=(COMPLEX8FrequencySeries *)LALMalloc(sizeof(COMPLEX8FrequencySeries));
	   xHat[i]->fft->data=(COMPLEX8Vector *)LALMalloc(sizeof(COMPLEX8Vector));
	   xHat[i]->fft->data->data=(COMPLEX8*)LALMalloc((params->nDeFT)*sizeof(COMPLEX8));

    } */
/* 05/16/02 gam Change xHat to type DeFTPeriodogram which contains a REAL8FrequencySeries and a ParameterSet. */
    xHat=(DeFTPeriodogram **)LALMalloc(params->mObsCoh*sizeof(DeFTPeriodogram *));
    for(i=0; i<params->mObsCoh; i++)
    {
	   xHat[i]=(DeFTPeriodogram *)LALMalloc(sizeof(DeFTPeriodogram));
	   xHat[i]->fft=(REAL8FrequencySeries *)LALMalloc(sizeof(REAL8FrequencySeries));
	   xHat[i]->fft->data=(REAL8Vector *)LALMalloc(sizeof(REAL8Vector));
	   xHat[i]->fft->data->data=(REAL8*)LALMalloc((params->nDeFT)*sizeof(REAL8));
           /* 06/02/02 gam */ /* Include memory to return DeFT for each polarization basis function */
           xHat[i]->fA=(COMPLEX16FrequencySeries *)LALMalloc(sizeof(COMPLEX16FrequencySeries));
           xHat[i]->fA->data=(COMPLEX16Vector *)LALMalloc(sizeof(COMPLEX16Vector));
           xHat[i]->fA->data->data=(COMPLEX16 *)LALMalloc(params->nDeFT*sizeof(COMPLEX16));
           xHat[i]->fA->data->length=(UINT4)(params->nDeFT);
           xHat[i]->fB=(COMPLEX16FrequencySeries *)LALMalloc(sizeof(COMPLEX16FrequencySeries));
           xHat[i]->fB->data=(COMPLEX16Vector *)LALMalloc(sizeof(COMPLEX16Vector));
           xHat[i]->fB->data->data=(COMPLEX16 *)LALMalloc(params->nDeFT*sizeof(COMPLEX16));
           xHat[i]->fB->data->length=(UINT4)(params->nDeFT);
    }

    k = 0; /* Need to set index of which coherent DFT = DeFT */
    #ifdef DEBUG_DEMOD_CODE
      for(i=0;i<params->nDeFT;i++)
      {
         /* 05/16/02 gam */
	 /* xHat[k]->fft->data->data[i].re = 0.0;
	 xHat[k]->fft->data->data[i].im = 0.0; */
	 xHat[k]->fft->data->data[i] = 0.0;
      }
    #endif
    #ifdef DEBUG_DEMOD_CODE
      for(i=0;i<params->nDeFT;i++)
      {
         /* 05/16/02 gam */
	 /* fprintf(stdout,"initialized xHat[%i] f = %g xHat.re = %g xHat.im = %g\n",i,params->f0Min+(REAL8)i/params->tCoh,
		xHat[k]->fft->data->data[i].re, xHat[k]->fft->data->data[i].im); */
	 fprintf(stdout,"initialized xHat[%i] f = %g xHat data = %g \n",i,params->f0Min+(REAL8)i/params->tCoh,
		xHat[k]->fft->data->data[i]);

	 fflush(stdout);
	 /* if (params->f0Min+(REAL8)i/params->tCoh > params->templateFreq0) break; */
      }
    #endif
    demParams->iCoh=k; /* Index of which coherent DFT = DeFT is being generated; used if calling LALDemod more than once. */

    /**** CALL LALDemod !!!! ****/
/* 05/13/02 gam */
/*    LALDemod(status->statusPtr, xHat, params->SFTData, demParams); */
/* 05/16/02 gam */ /* Call using new form of xHat. */
    LALDemod(status->statusPtr, xHat[k], params->SFTData, demParams);
    CHECKSTATUSPTR (status);

    #ifdef DEBUG_DEMOD_CODE
      fprintf(stdout,"\nparams->nDeFT = %i \n",params->nDeFT);
      fflush(stdout);
      /* for(i=0;i<xHat[k]->fft->data->length;i++) */
      /* NOTE: CANNOT USE ABOVE; LENGTH NEVER INITIALIZED !!! */
      for(i=0;i<params->nDeFT;i++)
      {
         /* 05/16/02 gam */
	 /* fprintf(stdout,"%i %g %g %g\n",i,params->f0Min+(REAL8)i/params->tCoh,
		xHat[k]->fft->data->data[i].re, xHat[k]->fft->data->data[i].im); */
	 fprintf(stdout,"i = %i f = %g fft->data->data[i] = %g \n",i,params->f0Min+(REAL8)i/params->tCoh,
		xHat[k]->fft->data->data[i]);
	 fflush(stdout);
	 /* if (params->f0Min+(REAL8)i/params->tCoh > params->templateFreq0) break; */
      }
    #endif

    /* Analyze results */
    /* 05/16/02 gam */ /* !!! NOTE xHat now contains the F statistic from Jaranowski, Krolak, and Schutz (I think Eq. 110) */
    /* Calculate the maximum power and the mean power */
    /* params->pwMax = 0.0; */ /* 11/05/01 gam already initialized */
    pwSum = 0.0;
    pwSum2= 0.0; /* 11/05/01 gam */
    tmpdF = params->f0Max - params->f0Min; /* 11/05/01 gam */
    for(i=0;i<params->nDeFT;i++)
    {
                /* 05/16/02 gam */
		/* pw=xHat[k]->fft->data->data[i].re*xHat[k]->fft->data->data[i].re+
				xHat[k]->fft->data->data[i].im*xHat[k]->fft->data->data[i].im; */
		pw=(REAL8)xHat[k]->fft->data->data[i];

                tmpF = params->f0Min + ((REAL8)i)/params->tCoh; /* 11/05/01 gam */

		#ifdef DEBUG_DEMOD_CODE
                        /* 05/14/02 gam */
  			/* fprintf(stdout, "freq, pw = %g %g \n",params->f0Min+(REAL8)i/params->tCoh,pw); */
  			fprintf(stdout, "freq, Jaranowski, Krolak, and Schutz F = %g %g \n",params->f0Min+(REAL8)i/params->tCoh,pw);
  			fflush(stdout);
		#endif
		#ifdef DEBUG_KNOWNPULSARDEMODOUTPUTFILES
  			/* fprintf(outputDataDat, "%g %g \n",params->f0Min+(REAL8)i/params->tCoh,pw); */
  			   fprintf(outputDataDat, "%g %g \n",params->f0Min+(REAL8)i/params->tCoh,4.0*pw); /* 10/22/02 gam Correct for missing factor of 4 in LALDemod */
  			fflush(outputDataDat);
    		#endif

	        /* pwSum = pwSum + pw; */ /* 11/05/01 gam */
	        /* pwSum2 = pwSum2 + pw*pw; */ /* 11/05/01 gam */

		if(params->pwMax<pw)
		{
			params->pwMax=pw;
			ipwMax=i;
		}

		/* 11/05/01 gam */
		if( fabs(params->templateFreq0 - tmpF) < tmpdF)
		{
			tmpdF = fabs(params->templateFreq0 - tmpF);
			ipwF0=i;
		}
    }
    /* 11/05/01 gam */ /* Compute mean and std dev for all bins except ipwF0 */
    for(i=0;i<params->nDeFT;i++)
    {
                /* 05/14/02 gam */
		/* pw=xHat[k]->fft->data->data[i].re*xHat[k]->fft->data->data[i].re+
				xHat[k]->fft->data->data[i].im*xHat[k]->fft->data->data[i].im; */
                /* 05/16/02 gam */
		/* pw=(REAL8)(xHat[k]->fft->data->data[i].re*xHat[k]->fft->data->data[i].re+
				xHat[k]->fft->data->data[i].im*xHat[k]->fft->data->data[i].im); */
		pw=(REAL8)xHat[k]->fft->data->data[i];


		if (i != ipwF0)
		{
	           pwSum = pwSum + pw;
	           pwSum2 = pwSum2 + pw*pw;
		}
    }
    /* 11/05/01 gam */
    /* 05/14/02 gam */
    /* params->pwF0 = pw=xHat[k]->fft->data->data[ipwF0].re*xHat[k]->fft->data->data[ipwF0].re +
				xHat[k]->fft->data->data[ipwF0].im*xHat[k]->fft->data->data[ipwF0].im; */
    /* 05/16/02 gam */
    /* params->pwF0 = pw = (REAL8)(xHat[k]->fft->data->data[ipwF0].re*xHat[k]->fft->data->data[ipwF0].re +
				xHat[k]->fft->data->data[ipwF0].im*xHat[k]->fft->data->data[ipwF0].im); */
    /* params->pwF0 = pw = (REAL8)xHat[k]->fft->data->data[ipwF0]; */
    params->pwF0 = (REAL8)xHat[k]->fft->data->data[ipwF0]; /* 01/15/03 gam Fix possible bug in computing pwF0 */

    params->freqPWF0 = params->f0Min + ((REAL8)ipwF0)/params->tCoh;
    params->pwMean = pwSum/((REAL8)(params->nDeFT - 1));
    params->pwStdDev = pwSum2/((REAL8)(params->nDeFT - 1)) - params->pwMean*params->pwMean;
    #ifdef DEBUG_BUILDOUTPUT_CODE
        /* 05/16/02 gam */
    	/* fprintf(stdout,"\npwStdDev squared = %g \n",params->pwStdDev); */
    	fprintf(stdout,"\nStd Dev F Jaranowski, Krolak, and Schutz squared = %g \n",params->pwStdDev);
	fflush(stdout);
    #endif
    if ( params->pwStdDev > 0.0 ) {
    	params->pwStdDev = sqrt( params->pwStdDev );
    } else {
        params->pwStdDev = -1.0; /* Should not happen except due to round off error in test data */
    }
    if (params->pwMean > 0.0) {
    	params->pwF0OverpwMean = params->pwF0/params->pwMean;
    } else {
    	params->pwF0OverpwMean = -1.0;  /* Should not happen */
    }
    /* 05/14/02 gam  */
    /* params->phaseForPWF0 = ComputePhase(xHat[k]->fft->data->data[ipwF0].re, xHat[k]->fft->data->data[ipwF0].im); */
    /* 05/16/02 gam */
    /* params->phaseForPWF0 = ComputePhase((REAL8)xHat[k]->fft->data->data[ipwF0].re, (REAL8)xHat[k]->fft->data->data[ipwF0].im); */
    /* params->phaseForPWF0 = ComputePhase((REAL8)xHat[k]->fft->data->data[ipwF0], (REAL8)0.0); */
       params->phaseForPWF0 = 0.0;

    #ifdef DEBUG_BUILDOUTPUT_CODE
        /* 05/16/02 gam */
    	/*fprintf(stdout,"\nfreqPWF0 = %g, pwF0 = %g, pwMean = %g, pwF0/pwMean = %g \n",params->freqPWF0,params->pwF0,params->pwMean,params->pwF0OverpwMean);
    	fprintf(stdout,"\npwStdDev = %g, phaseForPWF0 = %g \n",params->pwStdDev, params->phaseForPWF0); */
    	fprintf(stdout,"\nf_0 = %g, F statistic for f_0 = %g, FMean = %g, F/FMean = %g \n",params->freqPWF0,params->pwF0,params->pwMean,params->pwF0OverpwMean);
    	fprintf(stdout,"\nStd Dev of F stat = %g, Phase should be 0 since F is real = %g \n",params->pwStdDev, params->phaseForPWF0);
	fflush(stdout);
    #endif

    /* 07/09/02 gam */ /* Separate test for input and ouput since input now can include SFT data */
    #ifdef DEBUG_KNOWNPULSARDEMODINPUTFILES
  	fclose(inputDataDat);
    #endif

    #ifdef DEBUG_KNOWNPULSARDEMODOUTPUTFILES
  	fclose(outputDataDat);
    #endif

	/* Deallocate memory used */

	/* 06/02/02 gam  Free fA and fB.  Copied from LALDemodTest.c */
	/* 07/12/02 gam Moved after call to BuildDemodFrameOutput below */
	/* for(i=0; i<params->mObsCoh; i++)
	{
		LALFree(xHat[i]->fft->data->data);
		LALFree(xHat[i]->fft->data);
		LALFree(xHat[i]->fft);

                LALFree(xHat[i]->fA->data->data);
                LALFree(xHat[i]->fA->data);
                LALFree(xHat[i]->fA);
                LALFree(xHat[i]->fB->data->data);
                LALFree(xHat[i]->fB->data);
                LALFree(xHat[i]->fB);

		LALFree(xHat[i]);
	}
	LALFree(xHat); */

	/* Deallocate template params */
	LALFree(templateParams->spind->spParams);
	LALFree(templateParams->spind);
	LALFree(templateParams->skyP);
	LALFree(templateParams);

	/* Deallocate demodulation params */
	LALFree(demParams->skyConst);
	LALFree(demParams->spinDwn);
	LALFree(demParams);

	/* 06/02/02 gam  Copied from LALDemodTest.c */
        /* Deallocate AM  */
        /* LALSDestroyVector(&status, &(amc.a));
        LALSDestroyVector(&status, &(amc.b)); */
        LALSDestroyVector(status->statusPtr, &(amc.a));
    	CHECKSTATUSPTR (status);
        LALSDestroyVector(status->statusPtr, &(amc.b));
    	CHECKSTATUSPTR (status);
        /* LALFree(amParams->das->pSource->name); */ /* 07/07/02 gam */
        LALFree(amParams->das->pSource); /* 07/07/02 gam */
        LALFree(amParams->das);
        LALFree(amParams);

	/* 06/02/02 gam  Copied from LALDemodTest.c */
        /* LALFree(edat->ephemS);
        LALFree(edat->ephemE);
        LALFree(edat); */

        LALFree(params->edat->ephemS); /* 07/10/02 gam */
        LALFree(params->edat->ephemE); /* 07/10/02 gam */
        LALFree(params->edat); /* 07/10/02 gam */

	/* 07/12/01 gam */ /* BuildOutput before freeing memory for xHat */

	/* Next are used when allocating memory in BuildOutput and are also needed in BuildDemodFrameOutput */
	params->numDBOutput = 3;  /* FOR NOW HARD CODE for 1 row in 3 tables: search summary + search summary vars + dperiod tables */
        if (params->outputDeFT > 0) {
  	    output->numOutput = 3 + params->mObsCoh; /* FOR NOW HARD CODE */
	} else {
            output->numOutput = 3; /* FOR NOW HARD CODE */
	}

	BuildOutput(status->statusPtr, params, output);
        CHECKSTATUSPTR (status);

        if (params->outputDeFT > 0) {
           BuildDemodFrameOutput(status->statusPtr, params, xHat, params->timeIntervals, output);
	   CHECKSTATUSPTR (status);
	}

        for(i=0; i<params->mObsCoh; i++)
        {
		LALFree(xHat[i]->fft->data->data);
		LALFree(xHat[i]->fft->data);
		LALFree(xHat[i]->fft);

                LALFree(xHat[i]->fA->data->data);
                LALFree(xHat[i]->fA->data);
                LALFree(xHat[i]->fA);
                LALFree(xHat[i]->fB->data->data);
                LALFree(xHat[i]->fB->data);
                LALFree(xHat[i]->fB);

		LALFree(xHat[i]);
	 }
	 LALFree(xHat);

        /* LALCheckMemoryLeaks(); */ /* NOT SURE IF I CAN CALL THIS HERE */

 } /* END if (params->doDemod ...) */
   /**** LALDemod CODE SECTION ENDS HERE ****/
 #endif

   /***** Set output data structure STARTS HERE *****/

    	/* output->numOutput = 0; */ /* 07/12/02 gam Move above */

	/* 10/15/01 gam */
	if (params->finishedSFTs) {

	  /* if (params->doDemod) */ /* 07/07/02 gam */
	  /* if (params->doDemod > 0)
    		BuildOutput(status->statusPtr, params, output);
	  else */
	  /* 07/12/02 gam BuildOutput and BuildFrameOutput need to know how many rows are going into the database */
	  if (params->doDemod < 1) {
              params->numDBOutput = 2;  /* FOR NOW HARD CODE */
  	      output->numOutput = 2 + params->mObsSFT;  /* FOR NOW HARD CODE BUT NEED TO MAKE ROOM FOR OUTPUT SFTS AS WELL; THESE ARE BUILT IN BuildFrameOutput */

	      BuildOutput(status->statusPtr, params, output); /* 07/12/02 gam */ /* Always call to generate Search Summary table entry */
    	      CHECKSTATUSPTR (status);

	      /* Just output SFTs into frame files (ilwd files for stand alone wrapper */
    	      /* BuildFrameOutput(status->statusPtr, params, params->SFTData, params->timeIntervals, output); */
              BuildFrameOutput(status->statusPtr, params, params->SFTData, params->SFTStats, params->timeIntervals, output);
    	      CHECKSTATUSPTR (status);
	  }
	  /* 07/12/02 gam Else if params->doDemod > 0 then already have called BuildOutput and BuildDemodFrameOutput above */

    	  /* CHECKSTATUSPTR (status); */ /* 07/12/02 gam */

	  LALFree(params->timeStamps);

	  LALFree(params->sftPerCoh); /* 08/06/02 gam */

	  LALFree(params->timeIntervals);

	  /* Deallocate SFTData structure */
	  for(i=0;i<params->mObsSFT;i++)
	  {
		LALFree(params->SFTData[i]->fft->data->data);
		LALFree(params->SFTData[i]->fft->data);
		LALFree(params->SFTData[i]->fft);
		LALFree(params->SFTData[i]);
	  }
	  LALFree(params->SFTData);

	  /* 07/09/02 gam */ /* Only used when generating SFTs in code */
	  if (params->doDemod < 2) {
            /* 11/10/01 gam */ /* DAllocate memory for the SFTStats structure */
    	    for(i=0;i<params->mObsSFT;i++)
    	    {
		LALFree(params->SFTStats[i]);
    	    } /* END for(i=0;i<params->mObsSFT;i++) */
	    LALFree(params->SFTStats);
	  }

	  output->fracRemaining = 0;
    	  output->notFinished	= 0;

	  #ifdef DEBUG_FRACREMAINING_CODE
  	         fprintf(stdout, "\nfracRemaining = %g \n", output->fracRemaining);
  		 fflush(stdout);
	  #endif

	} else {

	      output->fracRemaining = (REAL4)params->lastK/(REAL4)params->mObsSFT;
    	      output->notFinished = 1;

	      #ifdef DEBUG_FRACREMAINING_CODE
  		fprintf(stdout, "\nfracRemaining = %g \n", output->fracRemaining);
  		fflush(stdout);
	      #endif

	} /* END if (params->finishedSFTs) else ... */

   /***** Set output data structure ENDS HERE *****/

  } else {
  /* if NOT (params->searchMaster) */
  /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
  /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */

    /* Just continue for now */
    /* Set output data structure */
    output->numOutput 	= 0;
    output->result	= NULL;
    output->fracRemaining = 0;
    output->notFinished	= 0;

  } /* END if (params->searchMaster) else ... */

  CHECKSTATUSPTR (status);
  DETATCHSTATUSPTR (status);
  RETURN( status );
}

/* Finalize search routine */
void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams
    )
{
  DemodSearchParams* const params = (DemodSearchParams *) *searchParams;

  INITSTATUS( status, "LALFinalizeSearch", KNOWNPULSARDEMODC );
  ATTATCHSTATUSPTR (status);

  /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
  /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */
  if (params->searchMaster) {

     /* 07/09/02 gam */ /* Only used when generating SFTS from time series data */
      if (params->doDemod < 2) {
         LALSDestroyVector(status->statusPtr, &(params->vecInputData));
      }

  }

  /* 10/16/01 gam */ /* 11/05/01 gam */
  LALFree(params->ifoNickName);
  LALFree(params->IFO);
  LALFree(params->targetName);

  LALFree(params->dsoName);

  #ifdef DEBUG_KNOWNPULSARDEMOD
    fprintf(stdout, "\nLALFinalizeSearch, my rank is %i.\n",params->rank);
    fflush(stdout);
  #endif

  LALFree(*searchParams);

  DETATCHSTATUSPTR (status);
  RETURN(status);

}

/********************** local static function definitions ********************/

/* !!!!!!!!!!!!!!! COPIED FROM LALDemodTest !!!!!!!!!!!!! */
/* internal routine to convert detector time to barycentric time */
/* Note: this routine will be replaced by Barycenter(), written  */
/* by C. Cutler, Albert-Einstein-Institut.  			 */
/*  10/08/01 gam; updated latest version form LALDemodTest */
void tdb(REAL8 alpha, REAL8 delta, REAL8 t_GPS, REAL8 *T, REAL8 *Tdot)
{

	/*RA alpha and DEC delta have their usual meanings, but units
  are radians, not hours or degrees */

	REAL8 tdiff;
	REAL8 tdiff_day;
	REAL8 tdiff_yr;
	REAL8 beta;
	REAL8 x;
	REAL8 y;
	REAL8 lam;
	REAL8 phi0;
	REAL8 psi0;
	REAL8 phi;
	REAL8 psi;
	REAL8 cose;
	REAL8 sine;
	REAL8 sind;
	REAL8 cosd;
	REAL8 sina;
	REAL8 cosa;
	REAL8 cosb;

	/*phi0 and psi0 given location and orientation, resp., of Earth at t_GPS =0 */
	REAL8 T_bary, Tdot_bary;
	/* T_bary (what we want to know) is pulse emission time in TDB coords.*/
	/* The lab measures GPS time, but we set t_GPS =0 at first instant of 1998*/
	REAL8 AU = 1.49597870e13/2.99792458e10;
	REAL8 RE = 6.37814e8/2.99792458e10;
	REAL8 eps = (23.0 + 26.0/60 + 21.488/3600)*LAL_PI/180;
	/*    printf("eps is %lf \n",eps); */
	phi0=2.86e0;
	psi0=4.12e0;
	/* Those values of phi0 and psi0 are completely made up */

	/* Now converting from equatorial to ecliptic coords. */
	sine=sin(eps);
	cose=cos(eps);
	sind=sin(delta);
	cosd=cos(delta);
	sina=sin(alpha);
	cosa=cos(alpha);
	beta=cose*sind-sine*cosd*sina;
	cosb=sqrt(1.0-beta*beta);
	x=cosd*cosa/cosb;
	y=(sine*sind+cose*cosd*sina)/cosb;
	lam=atan2(y,x);
	/*  printf("beta, lam are (rad) %lf  %lf \n", beta, lam);  */
	phi=phi0+2*LAL_PI*t_GPS/3.15576e7;
	psi=psi0+2*LAL_PI*t_GPS/8.64e4;
	tdiff_day=RE*cosd*cos(psi-alpha);
	tdiff_yr=AU*cosb*cos(phi-lam);
	tdiff=tdiff_day+tdiff_yr;
	T_bary=t_GPS+tdiff;
	Tdot_bary=1.e0-RE*cosd*sin(psi-alpha)*2*LAL_PI/8.64e4
		-AU*cosb*sin(phi-lam)*2*LAL_PI/3.15576e7;

	/* switch to turn mod off/on */
	/* note that 'off' negates the function of the code! */

	*T=T_bary;
 	*Tdot=Tdot_bary;

}

/*
void tdb(REAL8 alpha, REAL8 delta, REAL8 t_GPS, REAL8 *T, REAL8 *Tdot, const CHAR *sw)
{
*/
 /* RA alpha and DEC delta have their usual meanings, but units are radians, not hours or degrees */
/*
	REAL8 tdiff;
	REAL8 tdiff_day;
	REAL8 tdiff_yr;
	REAL8 beta;
	REAL8 x;
	REAL8 y;
	REAL8 lam;
	REAL8 phi0;
	REAL8 psi0;
	REAL8 phi;
	REAL8 psi;
	REAL8 cose;
	REAL8 sine;
	REAL8 sind;
	REAL8 cosd;
	REAL8 sina;
	REAL8 cosa;
	REAL8 cosb;
*/
	/*phi0 and psi0 given location and orientation, resp., of Earth at t_GPS =0 */
/*
	REAL8 T_bary, Tdot_bary;
*/
	/* T_bary (what we want to know) is pulse emission time in TDB coords.*/
	/* The lab measures GPS time, but we set t_GPS =0 at first instant of 1998*/
/*
	REAL8 AU = 1.49597870e13/2.99792458e10;
	REAL8 RE = 6.37814e8/2.99792458e10;
	REAL8 eps = (23.0 + 26.0/60 + 21.488/3600)*LAL_PI/180;
*/
	/*    printf("eps is %lf \n",eps); */
/*
	phi0=2.86e0;
	psi0=4.12e0;
*/
	/* Those values of phi0 and psi0 are completely made up */

	/* Now converting from equatorial to ecliptic coords. */
/*
	sine=sin(eps);
	cose=cos(eps);
	sind=sin(delta);
	cosd=cos(delta);
	sina=sin(alpha);
	cosa=cos(alpha);
	beta=cose*sind-sine*cosd*sina;
	cosb=sqrt(1.0-beta*beta);
	x=cosd*cosa/cosb;
	y=(sine*sind+cose*cosd*sina)/cosb;
	lam=atan2(y,x);
*/
	/*  printf("beta, lam are (rad) %lf  %lf \n", beta, lam);  */
/*
	phi=phi0+2*LAL_PI*t_GPS/3.15576e7;
	psi=psi0+2*LAL_PI*t_GPS/8.64e4;
	tdiff_day=RE*cosd*cos(psi-alpha);
	tdiff_yr=AU*cosb*cos(phi-lam);
	tdiff=tdiff_day+tdiff_yr;
	T_bary=t_GPS+tdiff;
	Tdot_bary=1.e0-RE*cosd*sin(psi-alpha)*2*LAL_PI/8.64e4
		-AU*cosb*sin(phi-lam)*2*LAL_PI/3.15576e7;
*/
	/* switch to turn mod off/on */
	/* note that 'off' negates the function of the code! */
/*
	if(sw!=NULL){
	*T=T_bary;
 	*Tdot=Tdot_bary;}

	else {
	*T=t_GPS;
	*Tdot=1.0;}

	}
*/

/*
void times(REAL8 tSFT, int howMany, LIGOTimeGPS *ts, INT4 sw)
{
	int i=0, j=0;
	int temp1, temp2;
	#ifdef DEBUG_DEMOD_CODE
	  FILE *fp;
	  fp=fopen("times.blah","w");
	#endif
	while(i<sw*howMany)
	{
		temp1=floor(tSFT*i);
		temp2=(int)((tSFT*(double)i-temp1)*1E9);

		if(i==0) {

			ts[j].gpsSeconds=0;
			ts[j].gpsNanoSeconds=0;

		} else {

			ts[j].gpsSeconds=temp1;
			ts[j].gpsNanoSeconds=temp2;
		}
		#ifdef DEBUG_DEMOD_CODE
		  fprintf(fp,"%d %d %d %d %d %d\n",ts[j].gpsSeconds, ts[j].gpsNanoSeconds,
		     timeIntervals[j].startSec, timeIntervals[j].startNan, timeIntervals[j].stopSec,timeIntervals[j].stopNan );
		#endif
		i=i+sw;
		j++;
	}
	#ifdef DEBUG_DEMOD_CODE
	  fclose(fp);
	#endif
}
*/

/* void MoreTimes(REAL8 tSFT, int howMany, LIGOTimeGPS *ts, gpsTimeInterval *timeIntervals, INT4 sw) */ /* 10/10/01 gam */
/* void MoreTimes(REAL8 tSFT, int howMany, UINT4 gpsStartTime, LIGOTimeGPS *ts, gpsTimeInterval *timeIntervals, INT4 sw) */  /* 11/02/01 gam */
/* void MoreTimes(REAL8 tSFT, INT4 howMany, UINT4 gpsStartTime, LIGOTimeGPS *ts, gpsTimeInterval *timeIntervals, INT4 sw) */
/* 08/06/02 gam */ /* Will compute sftPerCoh in MoreTimes, as in LALDemodTest times2 */
void MoreTimes(REAL8 tSFT, INT4 howMany, UINT4 gpsStartTime, LIGOTimeGPS *ts, INT4 **sftPerCoh, gpsTimeInterval *timeIntervals, INT4 sw)
{
        /*
	int i=0, j=0;
	int temp1, temp2;
	int temp3, temp4;
	*/

	/* 11/02/01 gam */
        INT4 i=0, j=0;
	INT4 temp1, temp2;
	INT4 temp3, temp4;

	#ifdef DEBUG_DEMOD_CODE
	  FILE *fp;
	  fp=fopen("times.blah","w");
	#endif

	/* 08/06/02 gam */ /* For now this is hard coded for just mObsCoh = 1 */
        *sftPerCoh=(INT4 *)LALCalloc(1+1, sizeof(INT4)); /* 08/06/02 gam */
        (*sftPerCoh)[0]=0;                               /* 08/06/02 gam */
        (*sftPerCoh)[1] = howMany;                       /* 08/06/02 gam */

	while(i<sw*howMany)
	{
		temp1=floor(tSFT*i);
		/* temp2=(int)((tSFT*(double)i-temp1)*1E9); */
		/* temp2=(INT4)((tSFT*(double)i-temp1)*1E9); */ /* 11/02/01 gam */
		temp2=(INT4)((tSFT*(REAL8)i-temp1)*1E9); /* 11/17/01 gam */

		temp3=floor(tSFT*(i + 1));
		/* temp4=(int)((tSFT*(double)(i + 1) - temp3)*1E9); */
		/* temp4=(INT4)((tSFT*(double)(i + 1) - temp3)*1E9); */ /* 11/02/01 gam */
		temp4=(INT4)((tSFT*(REAL8)(i + 1) - temp3)*1E9); /* 11/17/01 gam */

		if(i==0) {

		        /* 10/10/01 gam */ /*
			ts[j].gpsSeconds=0;
			ts[j].gpsNanoSeconds=0;

			timeIntervals[j].startSec = 0;
			timeIntervals[j].startNan = 0;
			timeIntervals[j].stopSec = temp3;
			timeIntervals[j].stopNan = temp4;
			*/

			ts[j].gpsSeconds = gpsStartTime;
			ts[j].gpsNanoSeconds = 0;

			timeIntervals[j].startSec = gpsStartTime;
			timeIntervals[j].startNan = 0;
			timeIntervals[j].stopSec = gpsStartTime + temp3;
			timeIntervals[j].stopNan = temp4;

		} else {

		        /* 10/10/01 gam */ /*
			ts[j].gpsSeconds=temp1;
			ts[j].gpsNanoSeconds=temp2;

			timeIntervals[j].startSec = temp1;
			timeIntervals[j].startNan = temp2;
			timeIntervals[j].stopSec = temp3;
			timeIntervals[j].stopNan = temp4;
			*/

			ts[j].gpsSeconds = gpsStartTime + temp1;
			ts[j].gpsNanoSeconds = temp2;

			timeIntervals[j].startSec = gpsStartTime + temp1;
			timeIntervals[j].startNan = temp2;
			timeIntervals[j].stopSec = gpsStartTime + temp3;
			timeIntervals[j].stopNan = temp4;

		}
		#ifdef DEBUG_DEMOD_CODE
		  fprintf(fp,"%d %d %d %d %d %d\n",ts[j].gpsSeconds, ts[j].gpsNanoSeconds,
		     timeIntervals[j].startSec, timeIntervals[j].startNan, timeIntervals[j].stopSec,timeIntervals[j].stopNan );
		#endif
		i=i+sw;
		j++;
	}
	#ifdef DEBUG_DEMOD_CODE
	  fclose(fp);
	#endif
}

/* 06/02/02 gam  Copied from LALDemodTest.c */
static void TimeToFloat(REAL8 *f, LIGOTimeGPS *tgps)
{
  INT4 x, y;

  x=tgps->gpsSeconds;
  y=tgps->gpsNanoSeconds;
  *f=(REAL8)x+(REAL8)y*1.e-9;
}

/* 06/02/02 gam Copied from LALDemodTest.c */
static void FloatToTime(LIGOTimeGPS *tgps, REAL8 *f)
{
  REAL8 temp0, temp2, temp3;
  REAL8 temp1, temp4;

  temp0 = floor(*f);     /* this is tgps.S */
  temp1 = (*f) * 1.e10;
  temp2 = fmod(temp1, 1.e10);
  temp3 = fmod(temp1, 1.e2);
  temp4 = (temp2-temp3) * 0.1;

  tgps->gpsSeconds = (INT4)temp0;
  tgps->gpsNanoSeconds = (INT4)temp4;
}


/* For building output for the database */
/* Copied and modified from Moments.c   */
/* Author: Charlton, Philip; Brady, P R */
/*
 * Function BuildOutput
 *
 * Create a linked list containing the database entries and put it in
 * the search output structure
 */
static
void
BuildOutput(LALStatus*            const status,
	    const DemodSearchParams* const params,
	    LALSearchOutput*      const output)
{

  /* const CHAR* const PROGRAM = "KnownPulsarDemod"; */ /* 10/16/01 gam */
  UINT4  i = 0;        /* all purpose index */

  dataBase* dbEntry = 0;
  outPut* outputArray = 0;

  /* SearchSummaryInfo info; */ /* 07/12/02 gam */ /* Copied from PingSearchWrappper.c */
  UINT4 stringLength; /* 07/12/02 gam */ /* Copied from StochasticBuildOutput.c */
  dataBase  *prev; /* 07/12/02 gam */ /* Copied from StochasticBuildOutput.c */

  /* 02/03/03 gam Define these temporary constants used below when setting in_end_time and in_end_time_ns */
  REAL8 tempReal8tSFT1 = 0.0;
  LIGOTimeGPS tempLIGOTimeGPStSFT;

  /* 07/12/02 gam add macro to attach status pointer */
  INITSTATUS( status, "BuildOutput", KNOWNPULSARDEMODC );
  ATTATCHSTATUSPTR( status );

  /* Number of output objects */
  /* 07/12/02 gam output->numOutput and params->numDBOutput are now set up in before these routines are called */
  /* output->numOutput = 1;  */ /* FOR NOW HARD CODE AS 1 */

  #ifdef DEBUG_BUILDOUTPUT_CODE
    	fprintf(stdout,"\nIn BuildOutput: output->params->numDBOutput = %i \n",params->numDBOutput); /* 07/12/02 gam */
    	fprintf(stdout,"\nIn BuildOutput: output->numOutput = %i \n",output->numOutput);
	fflush(stdout);
  #endif

  /* make check fails if below is used */
  /* outputArray = output->result = (outPut *) LALMalloc(1 * sizeof(outPut) ); */

  /* outputArray = output->result = (outPut *) LALCalloc( 1, sizeof(outPut) ); */
  outputArray = output->result = (outPut *) LALCalloc( output->numOutput, sizeof(outPut) );   /* 07/12/02 gam */
  if ( ! outputArray )
  {
    ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
  }

  /*
   * Start a linked list of database entries - the head of the list
   * starts with the "results" field
   */

  /* 07/12/02 gam */ /* Write to the search summary table.  Copied and modified from PingSearchWrapper.c */
  /* outputArray[0].search         = periodic;
  outputArray[0].significant    = 1;
  outputArray[0].states         = 0;
  outputArray[0].optional       = 0;
  outputArray[0].indexNumber    = 0;
  info.dsoName         = "knownpulsardemod";
  info.inputStartTime.gpsSeconds     = params->times.startSec;
  info.inputStartTime.gpsNanoSeconds = params->times.startNan;
  info.inputEndTime.gpsSeconds       = params->times.stopSec;
  info.inputEndTime.gpsNanoSeconds   = params->times.stopNan;
  info.outputStartTime = info.inputStartTime;
  info.outputEndTime   = info.inputEndTime;
  info.numEvents       = 1; */
  /* info.numNodes        = 1; */ /* HARD CODE FOR NOW; SHOULD EVENTUALLY USE SOMETHING LIKE BELOW */
  /* MPI_Comm_size( *(applyParams->mpiParams->comm), &info.numNodes  ); */
  /* TRY( LALCreateSearchSummaryDataBase( status->statusPtr, &output->result[0].results, &info ), status ); */
  /* TRY( LALCreateSearchSummaryDataBase( status->statusPtr, &outputArray[0].results, &info ), status ); */
  /* LALCreateSearchSummaryDataBase( status->statusPtr, &outputArray[0].results, &info); */
  /* CHECKSTATUSPTR (status); */

  /* 07/12/02 gam */ /* Write to the search summary table.  Copied and modified from StochasticBuildOutput.c */

      /* this is the same for every output */

      outputArray[0].search         = periodic;
      outputArray[0].significant    = 1;
      outputArray[0].states         = 0;
      outputArray[0].optional       = 0;
      outputArray[0].indexNumber    = 0;

      /* Initiate linked list */
      dbEntry = outputArray[0].results = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->previous = NULL;

     /* Write the search summary to the database */

     /* Shared object name */

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "shared_object", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen("knownpulsardemod");

     dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars,"knownpulsardemod", stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* LAL CVS tag */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "lal_cvs_tag", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen(LAL_CVS_TAG);

     dbEntry->rows.chars =
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, LAL_CVS_TAG, stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* LALWrapper CVS tag */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "lalwrapper_cvs_tag", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen(LAL_WRAPPER_CVS_TAG);

     dbEntry->rows.chars =
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, LAL_WRAPPER_CVS_TAG, stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* start time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
     strncpy( dbEntry->columnName, "in_start_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     /* 02/03/03 gam Add if: */
     if (params->doDemod > 1) {
       *(dbEntry->rows.int4s) = params->gpsStartTime; /* 02/03/03 gam Use input start time; actual start time for out_start_time below */
     } else {
       *(dbEntry->rows.int4s) = params->times.startSec;
     }

     prev = dbEntry;

     /* start time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_start_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     /* 02/03/03 gam Add if: */
     if (params->doDemod > 1) {
       *(dbEntry->rows.int4s) = 0; /* 02/03/03 gam Use input start time; actual start time for out_start_time below */
     } else {
       *(dbEntry->rows.int4s) = params->times.startNan;
     }

     prev = dbEntry;

     /* end time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_end_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     /* 02/03/03 gam Add if: */
     if (params->doDemod > 1) {
        /* 02/03/03 gam Compute requested end time which may differ from actual end time due to missing SFTs (gaps). */
	tempReal8tSFT1 = ((REAL8)params->gpsStartTime) + params->tCoh;  /* Assumes one coherence time in the observation time */
        FloatToTime(&tempLIGOTimeGPStSFT, &tempReal8tSFT1);
        *(dbEntry->rows.int4s) = tempLIGOTimeGPStSFT.gpsSeconds;
     } else {
        *(dbEntry->rows.int4s) = params->times.stopSec;
     }

     prev = dbEntry;

     /* end time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_end_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     /* 02/03/03 gam Add if: */
     if (params->doDemod > 1) {
       /* 02/03/03 gam Use result from above for requested end time which may differ from actual end time due to missing SFTs (gaps). */
       *(dbEntry->rows.int4s) = tempLIGOTimeGPStSFT.gpsNanoSeconds;
     } else {
       *(dbEntry->rows.int4s) = params->times.stopNan;
     }

     prev = dbEntry;

     /* start time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
     strncpy( dbEntry->columnName, "out_start_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.startSec;

     prev = dbEntry;

     /* start time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_start_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.startNan;

     prev = dbEntry;

     /* end time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_end_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.stopSec;

     prev = dbEntry;

     /* end time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_end_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.stopNan;

     dbEntry->next = NULL;

  #ifdef DEBUG_BUILDOUTPUT_CODE
      fprintf(stdout,"\nFINISHED OUTPUT FOR SEARCH SUMMARY TABLE\n");
      fflush(stdout);
  #endif

  /* 07/12/02 gam */ /* Write to the search summary variables table  */

      /* this is the same for every output */

      outputArray[1].search         = periodic;
      outputArray[1].significant    = 1;
      outputArray[1].states         = 0;
      outputArray[1].optional       = 0;
      outputArray[1].indexNumber    = 0;

      /* Initiate linked list */
      dbEntry = outputArray[1].results = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->previous = NULL;

     /* Write the search summary to the database */

     /* name */

     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "name", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen("knownpulsardemod");

     dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars,"knownpulsardemod", stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* string */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "string", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     if (params->doDemod < 1) {
         stringLength = strlen("percent_clean_lock");
     } else {
         stringLength = strlen("snr");
     }

     dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     if (params->doDemod < 1) {
         strncpy( dbEntry->rows.chars,"percent_clean_lock", stringLength*sizeof(CHAR) );
     } else {
         strncpy( dbEntry->rows.chars,"snr", stringLength*sizeof(CHAR) );
     }

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* value */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "value", dbNameLimit );


/* 01/15/03 gam Change to real_8 */

/*     dbEntry->type       = real_4;
     dbEntry->numberRows = 1;
     dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
     if ( ! dbEntry->rows.real4 )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     if (params->doDemod < 1) { */
        /* if (params->percentPad > 0.0)  */ /* 10/10/02 gam Fix round-off error problems when computing percentPad and percent clean lock */
/*        if (params->percentPad >= 0.0) {
           if (1.0 - params->percentPad >= 0.0) { */
                /* 10/10/02 gam Check above to avoid round-off error problems when computing percentPad and percent clean lock */
/*      	   	*(dbEntry->rows.real4) = (REAL4)100.0*(1.0 - params->percentPad);
           } else {
	        *(dbEntry->rows.real4) = 0.0;
           }
        } else {
      	   *(dbEntry->rows.real4) = -1.0; */  /* This case should not happen except in test data; hard code for now */
/*        }
     } else { */
        /* if (params->pwF0OverpwMean > 0.0) */ /* 10/10/02 gam Noticed want >= here. */
/*        if (params->pwF0OverpwMean >= 0.0) {
      	   *(dbEntry->rows.real4) = (REAL4)(sqrt(params->pwF0OverpwMean));
        } else {
      	   *(dbEntry->rows.real4) = -1.0; */  /* This case should not happen except in test data; hard code for now */
/*        }
     } */

     dbEntry->type       = real_8;
     dbEntry->numberRows = 1;
     dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
     if ( ! dbEntry->rows.real8 )
     {
       ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     if (params->doDemod < 1) {
        /* if (params->percentPad > 0.0)  */ /* 10/10/02 gam Fix round-off error problems when computing percentPad and percent clean lock */
        if (params->percentPad >= 0.0) {
           if (1.0 - params->percentPad >= 0.0) {
                /* 10/10/02 gam Check above to avoid round-off error problems when computing percentPad and percent clean lock */
      	   	*(dbEntry->rows.real8) = (REAL8)100.0*(1.0 - params->percentPad);
           } else {
	        *(dbEntry->rows.real8) = 0.0;
           }
        } else {
      	   *(dbEntry->rows.real8) = -1.0;   /* This case should not happen except in test data; hard code for now */
        }
     } else {
        /* if (params->pwF0OverpwMean > 0.0) */ /* 10/10/02 gam Noticed want >= here. */
        if (params->pwF0OverpwMean >= 0.0) {
      	   *(dbEntry->rows.real8) = (REAL8)(sqrt(params->pwF0OverpwMean));
        } else {
      	   *(dbEntry->rows.real8) = -1.0;   /* This case should not happen except in test data; hard code for now */
        }
     }

     dbEntry->next = NULL;

  if (params->doDemod < 1) {
      /* Just doing SFTs, so return */
      /* Terminate linked list */
      DETATCHSTATUSPTR( status );
      RETURN(status);
  }

  /* for (i = 0; i < output->numOutput; i++) */ /* 07/12/02 gam */
  for (i = 2; i < params->numDBOutput; i++) {

      /*  outputArray[0].templateNumber = 0; */    /* not relevant */
      /* outputArray[0].search        = periodic;
      outputArray[0].significant    = 1;
      outputArray[0].states         = 0;
      outputArray[0].optional       = 0; */

      /*  outputArray[0].templateNumber = 0; */    /* not relevant */
      /*  07/12/01 gam */
      outputArray[i].search         = periodic;
      outputArray[i].significant    = 1;   /* 1 - IS significant */
      outputArray[i].states         = 0;
      outputArray[i].optional       = 0;

      /* Just set equal to zero for now */
      outputArray[i].indexNumber = 0;

      /* Put entry into the SNGL_DPERIODIC database table */
      /* DPERIODIC = DIRECTED PERIODIC */

      /* Initiate linked list */
      dbEntry = outputArray[i].results = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->previous = NULL;

      /* ifo */  /*  10/18/01 gam */ /* dbEntry is initialized above */
      /*
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;
      */

      /* ifo */
      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "ifo" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      stringLength = strlen(params->ifoNickName); /* 07/18/02 gam */
      /* dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen( "??" ) + 1) * sizeof(CHAR) ); */
      /* dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen( params->IFO ) + 1) * sizeof(CHAR) ); */ /* 10/16/01 gam */
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) ); /* 07/18/02 gam */
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /* strcpy(dbEntry->rows.chars, "??"); */
      /* strcpy(dbEntry->rows.chars, params->IFO); */ /* 10/16/01 gam */
      strncpy(dbEntry->rows.chars, params->ifoNickName, stringLength*sizeof(CHAR)); /* 07/18/02 gam CAN ONLY BE TWO CHARACTERS */
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /* Don't include terminating null */
      /* *(dbEntry->rowDimensions) = (UINT4) strlen( "??" ); */
      /* *(dbEntry->rowDimensions) = (UINT4) strlen( params->IFO ); */ /* 10/16/01 gam */
      *(dbEntry->rowDimensions) = stringLength; /* 07/18/02 gam */

      /* search */ /* 12/13/01 gam */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "search" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      stringLength = strlen("knownpulsardemod");  /* 07/18/02 gam */
      /* dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen( "knownpulsardemod" ) + 1) * sizeof(CHAR) ); */
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) ); /* 07/18/02 gam */
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /* strcpy(dbEntry->rows.chars, "knownpulsardemod"); */
      strncpy(dbEntry->rows.chars, "knownpulsardemod",stringLength*sizeof(CHAR)); /* 07/18/02 gam */
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /* Don't include terminating null */
      /* *(dbEntry->rowDimensions) = (UINT4) strlen( "knownpulsardemod" ); */ /* 07/18/02 gam */
      *(dbEntry->rowDimensions) = stringLength;


      /* start_time */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "start_time" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.startSec;

     /* start_time_ns */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "start_time_ns" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.startNan;

     /* end_time */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "end_time" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.stopSec;

     /* end_time_ns */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "end_time_ns" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.stopNan;

      /* duration */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "duration" );

      /* 10/17/01 gam */
      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      /* *(dbEntry->rows.real4) = params->deltaT*params->inputN; */
      /* *(dbEntry->rows.real4) = params->deltaT*((REAL4)(params->inputN - 1)); */
      *(dbEntry->rows.real4) = (REAL4)(((REAL4)params->deltaT)*((REAL4)params->inputN)); /* 10/17/01 gam */

      /* 10/10/01 gam */
      /* 10/17/01 gam */
      /*
      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! dbEntry->rows.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      */
      /* *(dbEntry->rows.real8) = params->deltaT*((REAL8)(params->inputN - 1)); */
      /* *(dbEntry->rows.real8) = params->deltaT*((REAL8)params->inputN); */ /* 10/11/01 gam */

      /* target_name */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "target_name" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      stringLength = strlen(params->targetName ); /* 07/18/02 gam */
      /* dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen( "A known pulsar" ) + 1) * sizeof(CHAR) ); */
      /* dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen( params->targetName ) + 1) * sizeof(CHAR) ); *//* 10/16/01 gam */
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) ); /* 07/18/02 gam */
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /* strcpy(dbEntry->rows.chars, "A known pulsar"); */
      /* strcpy(dbEntry->rows.chars, params->targetName); */ /* 10/16/01 gam */
      strncpy(dbEntry->rows.chars, params->targetName, stringLength*sizeof(CHAR)); /* 07/18/02 gam */
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /* Don't include terminating null */
      /* *(dbEntry->rowDimensions) = (UINT4) strlen( "A known pulsar" ); */
      /* *(dbEntry->rowDimensions) = (UINT4) strlen( params->targetName ); */ /* 10/16/01 gam */
      *(dbEntry->rowDimensions) = stringLength; /* 07/18/02 gam */


      /* sky_ra */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "sky_ra" );

      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! dbEntry->rows.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = params->templateRA;

      /* sky_dec */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "sky_dec" );

      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! dbEntry->rows.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = params->templateDec;

      /* frequency */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "frequency" );

      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! dbEntry->rows.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = (REAL8) params->freqPWF0; /* 11/05/01 gam */

      /* amplitude */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "amplitude" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real4) = (REAL4) sqrt(params->pwF0);  /* 11/05/01 gam */

      /* phase */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "phase" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real4) = (REAL4) params->phaseForPWF0;  /* 11/05/01 gam */

      /* snr */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "snr" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      /* *(dbEntry->rows.real4) = (REAL4) params->pwF0OverpwMean; */ /* 11/05/01 gam */
      /* if (params->pwStdDev > 0.0) {
       	 *(dbEntry->rows.real4) = (REAL4)((params->pwF0 - params->pwMean)/params->pwStdDev);
      } else { */
       	 /* *(dbEntry->rows.real4) = -1.0; */  /* This case should not happen except in test data; hard code for now */
      /* } */
      /* 07/09/02 gam temporarily set SNR = sqrt(pwF0/pwMean). THIS IS REALLY JKS sqrt(Fmax/Fmean) */
      if (params->pwF0OverpwMean > 0.0) {
       	 *(dbEntry->rows.real4) = (REAL4)(sqrt(params->pwF0OverpwMean));
      } else {
       	 *(dbEntry->rows.real4) = -1.0;   /* This case should not happen except in test data; hard code for now */
      }

      /* confidence */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "confidence" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      /* *(dbEntry->rows.real4) = 0.0; */
      /* *(dbEntry->rows.real4) = 1.0 - exp(-1.0*params->pwF0OverpwMean); */ /* 11/05/01 gam */
      *(dbEntry->rows.real4) = 1.0 - (1.0 + 2.0*params->pwF0OverpwMean)*exp(-2.0*params->pwF0OverpwMean); /* 10/16/01 gam see calculation from this date */

      /* stat_name */ /* 10/16/01 gam */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "stat_name" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;

      /*  01/15/03 gam If normalizeSFTs > 1 then calibrate; if normalizeSFTs == 3 then calibrate only; do not normalize. */
      /*               Thus if normalizeSFTs == 1 normalize only; if normalizeSFTs == 2 then calibrate and normalize; if normalizeSFTs == 3 calibrate only */
      if (params->normalizeSFTs == 1) {
      	stringLength = strlen("Norm JKS F Statistic");
      } else if (params->normalizeSFTs == 2) {
      	stringLength = strlen("Calib Norm JKS F Statistic");
      } else if (params->normalizeSFTs == 3) {
      	stringLength = strlen("Calib Only JKS F Statistic");
      } else {
      	stringLength = strlen("JKS F Statistic");
      }
      /* stringLength = strlen("JKS F Statistic"); */
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) );
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /*  01/15/03 gam If normalizeSFTs > 1 then calibrate; if normalizeSFTs == 3 then calibrate only; do not normalize. */
      /*               Thus if normalizeSFTs == 1 normalize only; if normalizeSFTs == 2 then calibrate and normalize; if normalizeSFTs == 3 calibrate only */
      if (params->normalizeSFTs == 1) {
        strncpy(dbEntry->rows.chars, "Norm JKS F Statistic",stringLength*sizeof(CHAR));
      } else if (params->normalizeSFTs == 2) {
        strncpy(dbEntry->rows.chars, "Calib Norm JKS F Statistic",stringLength*sizeof(CHAR));
      } else if (params->normalizeSFTs == 3) {
        strncpy(dbEntry->rows.chars, "Calib Only JKS F Statistic",stringLength*sizeof(CHAR));
      } else {
        strncpy(dbEntry->rows.chars, "JKS F Statistic",stringLength*sizeof(CHAR));
      }
      /* strncpy(dbEntry->rows.chars, "JKS F Statistic",stringLength*sizeof(CHAR)); */
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      /* Don't include terminating null */
      *(dbEntry->rowDimensions) = stringLength;

      /* stat_value */ /* 10/16/01 gam */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "stat_value" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real4) = (REAL4) params->pwF0;

      /* stat_prob */ /* 10/16/01 gam */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "stat_prob" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      /* *(dbEntry->rows.real4) = 0.0; */
      /* *(dbEntry->rows.real4) = 1.0 - exp(-1.0*params->pwF0OverpwMean); */ /* 11/05/01 gam */
      *(dbEntry->rows.real4) = 1.0 - (1.0 + 2.0*params->pwF0OverpwMean)*exp(-2.0*params->pwF0OverpwMean); /* 10/16/01 gam see calculation from this date */

      /* Terminate linked list */
      dbEntry->next = NULL;

  } /* END for (i = 0; i < output->numOutput; i++) */

  /* 07/12/02 gam add macro to detach the status pointer */
  DETATCHSTATUSPTR( status );

  RETURN(status);
}
/* END BuildOutput */

/* Compute the phase of a complex Number */
REAL8 ComputePhase(REAL8 x, REAL8 y)
{
	REAL8 phi = 0.0;
	REAL8 pi = LAL_PI;
	REAL8 twoPi = LAL_TWOPI;
	REAL8 piOverTwo = LAL_PI_2;
        if (x != 0.0) {
		phi = atan(fabs(y/x));
		if (x < 0) {
			if (y < 0) {
				// quad III
				phi = phi + pi;
			} else {
				// quad II
				phi = pi -  phi;
			}
		} else {
			if (y < 0) {
				// quad IV
				phi = twoPi -  phi;
			} else {
				// quad I; just continue.
			}
		}
	} else {
		if (y >= 0.0) {
			phi = piOverTwo;     //return pi/2.
		} else {
			phi = 3.0*piOverTwo; //return 3pi/2.
		}
	}
	return phi;
}

/* Copied and modified from StochasticBuildOutput.c written by: */
/* Anderson, W.G., Romano, J.D., and Whelan, J.T. */
/*void StochasticBuildOutput (
      LALStatus*                  status,
      LALSearchOutput*            output,
      COMPLEX8FrequencySeries*    spec,
      gpsTimeInterval             timeInterval
)
*/
/*
static void
BuildFrameOutput(LALStatus*            const status,
	    const DemodSearchParams*   const params,
	    FFT                        **SFTData,
            gpsTimeInterval            *timeIntervals,
	    LALSearchOutput*           const output)
*/
/* 11/10/01 gam */ /* Add in SFTStats */
static void BuildFrameOutput(LALStatus* const status,
	    const DemodSearchParams*    const params,
	    FFT                         **SFTData,
            DemodPowerStats             **SFTStats,
            gpsTimeInterval             *timeIntervals,
	    LALSearchOutput*            const output)

{
   /* outPut* outputArray = 0; */ /* 07/12/02 gam */
   outPut* outputArray;
   multiDimData*   mddEntry;
   CHARVector     *unitString;

   dcHistory *  historyEntry; /* 11/05/01 gam */

   UINT4 k = 0;    /* index used with SFTData */
   UINT4 iOut = 0; /* index used with outputArray */
   /* UINT4 icopy = 0; */ /* index used to copy SFT data to output structure */ /* 03/06/02 gam */ /* 05/14/02 gam */

   INITSTATUS( status, "BuildFrameOutput", KNOWNPULSARDEMODC );
   ATTATCHSTATUSPTR (status);

    /* output->numOutput = 1; */
    /* output->numOutput = params->inputM; */
    /* output->numOutput = params->mObsSFT; */ /* 07/12/01 gam Already set in BuildOutput when creating search summary table entry */ /* 10/12/01 gam */

   /*
    *
    * create memory for the outPut array
    *
    */

  /* 07/12/01 gam Already set in BuildOutput when creating search summary table entry */
  /* outputArray = output->result = (outPut *) LALCalloc( output->numOutput, sizeof(outPut) );
  if ( ! outputArray )
  {
    ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
  } */

   /*
    *
    * fill the array of output
    *
    */

  outputArray = output->result;

  /* for (k = 0; k < output->numOutput; k++) */ /* 07/12/01 gam */
  for (k = 0; k < params->mObsSFT; k++) {

      /*  outputArray[0].templateNumber = 0; */    /* not relevant */
      /* outputArray[k].search         = periodic;
      outputArray[k].significant    = 1;
      outputArray[k].states         = 0;
      outputArray[k].optional       = 0;
      outputArray[k].indexNumber = 0;  */

      iOut =  k + params->numDBOutput;

      outputArray[iOut].search         = periodic;
      outputArray[iOut].significant    = 1;   /* 1 - IS significant */
      outputArray[iOut].states         = 0;
      outputArray[iOut].optional       = 0;
      outputArray[iOut].indexNumber = 0;

     /*
      *
      * alloc memory for the optional linked list
      *
      */

     /* allocate memory */
     /* mddEntry = outputArray->optional = (multiDimData *) LALMalloc (sizeof (multiDimData)); */
     /* Memory allocation fails if above line used in place of line below */
     /* mddEntry = outputArray->optional = (multiDimData *) LALCalloc (1, sizeof(multiDimData)); */
     /* mddEntry = outputArray[k].optional = (multiDimData *) LALCalloc (1, sizeof(multiDimData)); */ /* 07/12/02 gam */
     mddEntry = outputArray[iOut].optional = (multiDimData *) LALCalloc (1, sizeof(multiDimData));
     if (!mddEntry)
     {
      ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }

     /* write the name and domain of the data */
     /* strncpy (mddEntry->name, "filter_output", maxMultiDimName); */
     strncpy (mddEntry->name, "sft_output", maxMultiDimName);

     /* Set the dimensions from the output frequency series */
     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize), status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &(params->unitSFT)), status );
     strncpy (mddEntry->units, unitString->data, maxMultiDimUnits);
     TRY( LALCHARDestroyVector(status->statusPtr, &unitString), status );
     mddEntry->space = freqD;

     #ifdef DEBUG_BUILDFRAMEOUTPUT_CODE
  	fprintf(stdout, "\ngpsStartTimeSec= %i \n", (UINT4)timeIntervals[k].startSec);
  	fprintf(stdout, "\ngpsStartTimeNan = %i \n",(UINT4)timeIntervals[k].startNan);
  	fprintf(stdout, "\ngpsStopTimeSec = %i \n", (UINT4)timeIntervals[k].stopSec);
  	fprintf(stdout, "\ngpsStopTimeNan = %i \n", (UINT4)timeIntervals[k].stopNan);
  	fflush(stdout);
     #endif

     mddEntry->range.dFreq.gpsStartTimeSec = (UINT4)timeIntervals[k].startSec;
     mddEntry->range.dFreq.gpsStartTimeNan = (UINT4)timeIntervals[k].startNan;
     mddEntry->range.dFreq.gpsStopTimeSec = (UINT4)timeIntervals[k].stopSec;
     mddEntry->range.dFreq.gpsStopTimeNan = (UINT4)timeIntervals[k].stopNan;

     #ifdef DEBUG_BUILDFRAMEOUTPUT_CODE
  	fprintf(stdout, "\nstartFreq = %f \n", SFTData[k]->fft->f0);
  	fprintf(stdout, "\nstopFreq = %f \n", SFTData[k]->fft->f0 + (REAL8)(SFTData[k]->fft->data->length - 1) * SFTData[k]->fft->deltaF);
  	fprintf(stdout, "\nfreqStepSize = %f \n", SFTData[k]->fft->deltaF);
  	fflush(stdout);
     #endif

     mddEntry->range.dFreq.startFreq = SFTData[k]->fft->f0;
     mddEntry->range.dFreq.stopFreq =  SFTData[k]->fft->f0 + (REAL8)(SFTData[k]->fft->data->length - 1) * SFTData[k]->fft->deltaF;
     mddEntry->range.dFreq.freqStepSize = SFTData[k]->fft->deltaF;

     /* mddEntry->range.dFreq.numberSamples = spec->data->length; */
     mddEntry->range.dFreq.numberSamples = SFTData[k]->fft->data->length;

     /* data type is COMPLEX8 */
     mddEntry->type = complex_8; /* 03/06/02 gam */
     /* mddEntry->type = complex_16; */ /* 09/27/01 gam */ /* 03/06/02 gam */

     /* size of the data */
     mddEntry->numberDimensions = 1;
     mddEntry->dimensions = (UINT4 *) LALMalloc (sizeof(UINT4));
     /* Below trying LALCalloc instead */
     /* mddEntry->dimensions = (UINT4 *) LALCalloc (1, sizeof(UINT4)); */
     if (!(mddEntry->dimensions))
     {
      ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     *(mddEntry->dimensions) = SFTData[k]->fft->data->length;

     /* dcHistory is ? */
     /* mddEntry->history = NULL; */ /* 11/02/01 gam */

     /* 11/02/01 gam */
     /* Set up the history structure with meta data about the SFT */

     /* dcHistory data structure */
     /*
	typedef struct dcHistoryTag
	{
   		struct dcHistoryTag* previous;
   		CHAR name[maxHistoryName];
   		CHAR units[maxHistoryUnits];
   		datatype type;
   		UINT4 numberValues;
   		dataPointer value;
   		struct dcHistoryTag* next;
	}dcHistory;
      */

      /* 11/05/01 gam */
      /* Start history linked list */
      historyEntry = mddEntry->history = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      historyEntry->previous = NULL;

      /* Use ifdef below to remove items from history list */
#ifdef HISTORY_test
      /* dso_name */
      sprintf(historyEntry->name, "dso_name" );
      sprintf(historyEntry->units, " " );

      historyEntry->type = char_s_ptr;
      historyEntry->numberValues = strlen( params->dsoName );
      historyEntry->value.chars = (CHAR *) LALMalloc( (strlen( params->dsoName ) + 1) * sizeof(CHAR) );
      if ( ! historyEntry->value.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      strcpy(historyEntry->value.chars, params->dsoName);

      /* datacon_desc */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;


      /* 07/18/02 gam dataConDesc is obsolete; changed this to 2 character ifoNickName */
      /* sprintf(historyEntry->name, "datacon_desc" ); */
      sprintf(historyEntry->name, "ifo_nickname" );
      sprintf(historyEntry->units, " " );

      historyEntry->type = char_s_ptr;
      historyEntry->numberValues = strlen( params->ifoNickName );
      historyEntry->value.chars = (CHAR *) LALMalloc( (strlen( params->ifoNickName ) + 1) * sizeof(CHAR) );
      if ( ! historyEntry->value.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      strcpy(historyEntry->value.chars, params->ifoNickName);

      /* percent_pad */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

#endif

      /* Put percent drop out that was zero padded in time series that produced this sft */
      sprintf(historyEntry->name, "percent_pad" );
      sprintf(historyEntry->units, "percent" );
      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = 100.0*params->percentPad;  /* 11/05/01 gam */

      /* max_power */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "max_power" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(SFTStats[k]->pwMax);

     /* freq_pwmax */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freq_pwmax" );
      sprintf(historyEntry->units, "hz" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(SFTStats[k]->freqPWMax);

     /* power_mean */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "power_mean" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(SFTStats[k]->pwMean);

     /* power_stddev */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "power_stddev" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(SFTStats[k]->pwStdDev);

      /* startSec */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "startSec" );
      sprintf(historyEntry->units, "s" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[k].startSec;

      /* startNan */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "startNan" );
      sprintf(historyEntry->units, "ns" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[k].startNan;

      /* stopSec */
      /* 11/17/01 gam */
      /*
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "stopSec" );
      sprintf(historyEntry->units, "s" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[k].stopSec;
      */

      /* stopNan */
      /* 11/17/01 gam */
      /*
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "stopNan" );
      sprintf(historyEntry->units, "ns" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[k].stopNan;
      */

      /* sample_rate */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "sample_rate" );
      sprintf(historyEntry->units, "hz" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(1.0/params->deltaT);

      /* ndata_input */ /* 11/17/01 gam */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "ndata_input" );
      sprintf(historyEntry->units, " " );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->nSFT;

      /* channel */ /* 11/18/01 gam Will try to include this later */
      /*
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "channel" );
      sprintf(historyEntry->units, " " );

      historyEntry->type = char_s_ptr;
      historyEntry->numberValues = strlen( params->channel );
      historyEntry->value.chars = (CHAR *) LALMalloc( (strlen( params->channel ) + 1) * sizeof(CHAR) );
      if ( ! historyEntry->value.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      strcpy(historyEntry->value.chars, params->channel);
      */

      historyEntry->next = NULL;
      /* End history linked list */

     /* allocate memory for the data */
     mddEntry->data.complex8 = (COMPLEX8 *) LALCalloc (SFTData[k]->fft->data->length, sizeof(COMPLEX8)); /* 03/06/02 gam */
     /* mddEntry->data.complex16 = (COMPLEX16 *) LALCalloc (SFTData[k]->fft->data->length, sizeof(COMPLEX16)); */ /* 09/27/01 gam */ /* 03/06/02 gam */
     /* below trying LALMalloc instead */
     /* mddEntry->data.complex8 = (COMPLEX8 *) LALMalloc (SFTData[k]->fft->data->length*sizeof(COMPLEX8)); */
     /* if (!(mddEntry->data.complex8)) */
     /* 09/27/01 gam */
     /* if (!(mddEntry->data.complex16)) */ /* 03/06/02 gam */
     if (!(mddEntry->data.complex8))
     {
      ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }

     /* and copy the data there */
     /* memcpy (mddEntry->data.complex8, SFTData[k]->fft->data->data, SFTData[k]->fft->data->length * sizeof(COMPLEX8) ); */
     /* memcpy (mddEntry->data.complex16, SFTData[k]->fft->data->data, SFTData[k]->fft->data->length * sizeof(COMPLEX16) ); */ /* 09/27/01 gam */ /* 03/06/02 gam */

     /* 03/06/02 gam */ /* 05/14/02 gam */
     /* for (icopy = 0; icopy < SFTData[k]->fft->data->length; icopy++) {
         mddEntry->data.complex8[icopy].re = (REAL4)SFTData[k]->fft->data->data[icopy].re;
         mddEntry->data.complex8[icopy].im = (REAL4)SFTData[k]->fft->data->data[icopy].im;
     } */
     memcpy (mddEntry->data.complex8, SFTData[k]->fft->data->data, SFTData[k]->fft->data->length * sizeof(COMPLEX8) );

  } /* end for for (k = 0; k < params->mObsSFT; k++) */

   /* normal exit */
   DETATCHSTATUSPTR (status);
   RETURN (status);
}
/* END BuildFrameOutput */

/* 07/12/02 gam */ /* Similar to BuildFrameOutput above */ /* Output Demod JKS F Power Spectrum in Frame Format */
static void BuildDemodFrameOutput(LALStatus* const status,
	    const DemodSearchParams*    const params,
	    DeFTPeriodogram             **xHat,
            gpsTimeInterval             *timeIntervals,
	    LALSearchOutput*            const output)

{
   outPut* outputArray;
   multiDimData*   mddEntry;
   CHARVector     *unitString;

   LALUnitPair  unitPair;   /* A pair of LALUnits */
   LALUnit     unitF;     /* Temporary place holder for units. */

   dcHistory *  historyEntry; /* 11/05/01 gam */

   UINT4 k = 0;    /* index used with SFTData */
   UINT4 iOut = 0; /* index used with outputArray */
   /* UINT4 icopy = 0; */ /* index used to copy SFT data to output structure */ /* 03/06/02 gam */ /* 05/14/02 gam */

   INITSTATUS( status, "BuildDemodFramesOutput", KNOWNPULSARDEMODC );
   ATTATCHSTATUSPTR (status);

   outputArray = output->result;

   /* Set the dimensions from the output JKS F power  */
   unitPair.unitOne = &(params->unitSFT);  /* 04/16/02 gam */ /* initial unitSFT is set up in ConditionData; Now add in units due to normalization */
   unitPair.unitTwo = &(params->unitSFT);
   LALUnitMultiply(status->statusPtr, &unitF, &unitPair);
   CHECKSTATUSPTR (status);

  for (k = 0; k < params->mObsCoh; k++) {

      iOut =  k + params->numDBOutput;

      outputArray[iOut].search         = periodic;
      outputArray[iOut].significant    = 1;   /* 1 - IS significant */
      outputArray[iOut].states         = 0;
      outputArray[iOut].optional       = 0;
      outputArray[iOut].indexNumber = 0;

     mddEntry = outputArray[iOut].optional = (multiDimData *) LALCalloc (1, sizeof(multiDimData));
     if (!mddEntry)
     {
      ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }

     /* write the name and domain of the data */
     /* strncpy (mddEntry->name, "filter_output", maxMultiDimName); */
     strncpy (mddEntry->name, "DeFT_output", maxMultiDimName);

     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize), status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &(unitF)), status );
     strncpy (mddEntry->units, unitString->data, maxMultiDimUnits);
     TRY( LALCHARDestroyVector(status->statusPtr, &unitString), status );
     mddEntry->space = freqD;

     #ifdef DEBUG_BUILDDEMODFRAMEOUTPUT_CODE
  	fprintf(stdout, "\ngpsStartTimeSec= %i \n", (UINT4)timeIntervals[0].startSec);
  	fprintf(stdout, "\ngpsStartTimeNan = %i \n",(UINT4)timeIntervals[0].startNan);
  	fprintf(stdout, "\ngpsStopTimeSec = %i \n", (UINT4)timeIntervals[params->mObsSFT - 1].stopSec);
  	fprintf(stdout, "\ngpsStopTimeNan = %i \n", (UINT4)timeIntervals[params->mObsSFT - 1].stopNan);
  	fflush(stdout);
     #endif

     /* 07/12/02 gam THIS ONLY WORKS WHEN THERE IS ONLY ONE DEFT, WHICH IS TRUE FOR NOW; ALSO USED IN HISTORY BELOW */
     /* 07/12/02 gam OTHERWISE INTERVALS MUST BE CHANGED TO BE DEFT INTERVALS */
     mddEntry->range.dFreq.gpsStartTimeSec = (UINT4)params->timeIntervals[0].startSec;
     mddEntry->range.dFreq.gpsStartTimeNan = (UINT4)params->timeIntervals[0].startNan;
     mddEntry->range.dFreq.gpsStopTimeSec = (UINT4)params->timeIntervals[params->mObsSFT - 1].stopSec;
     mddEntry->range.dFreq.gpsStopTimeNan = (UINT4)params->timeIntervals[params->mObsSFT -1].stopNan;

     #ifdef DEBUG_BUILDDEMODFRAMEOUTPUT_CODE
  	fprintf(stdout, "\nstartFreq = %f \n", params->f0Min);
  	fprintf(stdout, "\nstopFreq = %f  \n", params->f0Min + (REAL8)(params->nDeFT - 1) * 1.0/params->tCoh);
  	fprintf(stdout, "\nfreqStepSize = %f \n",1.0/params->tCoh);
  	fflush(stdout);
     #endif

     /* 07/18/02 gam Correct below */
     /* mddEntry->range.dFreq.startFreq = xHat[k]->fft->f0;
     mddEntry->range.dFreq.stopFreq =  xHat[k]->fft->f0 + (REAL8)(params->nDeFT - 1) * xHat[k]->fft->deltaF;
     mddEntry->range.dFreq.freqStepSize = xHat[k]->fft->deltaF; */

     mddEntry->range.dFreq.startFreq = params->f0Min;
     mddEntry->range.dFreq.stopFreq  = params->f0Min + (REAL8)(params->nDeFT - 1) * 1.0/params->tCoh;
     mddEntry->range.dFreq.freqStepSize = 1.0/params->tCoh;

     mddEntry->range.dFreq.numberSamples = params->nDeFT;

     /* data type is REAL8 */
     mddEntry->type = real_8; /* 03/06/02 gam */

     /* size of the data */
     mddEntry->numberDimensions = 1;
     mddEntry->dimensions = (UINT4 *) LALMalloc (sizeof(UINT4));
     /* Below trying LALCalloc instead */
     /* mddEntry->dimensions = (UINT4 *) LALCalloc (1, sizeof(UINT4)); */
     if (!(mddEntry->dimensions))
     {
      ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }
     *(mddEntry->dimensions) = (UINT4)params->nDeFT;

      /* 11/05/01 gam */
      /* Start history linked list */
      historyEntry = mddEntry->history = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      historyEntry->previous = NULL;

      /* Use ifdef below to remove items from history list */
#ifdef HISTORY_test
      /* dso_name */
      sprintf(historyEntry->name, "dso_name" );
      sprintf(historyEntry->units, " " );

      historyEntry->type = char_s_ptr;
      historyEntry->numberValues = strlen( params->dsoName );
      historyEntry->value.chars = (CHAR *) LALMalloc( (strlen( params->dsoName ) + 1) * sizeof(CHAR) );
      if ( ! historyEntry->value.chars )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      strcpy(historyEntry->value.chars, params->dsoName);
#endif

      /* Put percent drop out during the coherence time */
      /* SHOULD INCLUDE THIS ONCE THIS CAN BE CALCULATED */
      /* sprintf(historyEntry->name, "percent_pad" );
      sprintf(historyEntry->units, "percent" );
      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = 100.0*params->percentPad;  */ /* 11/05/01 gam */

      /* RA */
      sprintf(historyEntry->name, "RA" );
      sprintf(historyEntry->units, "radians" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateRA);

      /* Dec */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "Dec" );
      sprintf(historyEntry->units, "radians" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateDec);

      /* freq0 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freq0" );
      sprintf(historyEntry->units, "Hz" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateFreq0);

      /* freqDeriv1 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv1" );
      sprintf(historyEntry->units, "Hz2" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateFreqDeriv0);

      /* freqDeriv2 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv2" );
      sprintf(historyEntry->units, "Hz3" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateFreqSecondDeriv0);

      /* freqDeriv3 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv3" );
      sprintf(historyEntry->units, "Hz4" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateFreqThirdDeriv0);

      /* freqDeriv4 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv4" );
      sprintf(historyEntry->units, "Hz5" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateFreqFourthDeriv0);

      /* freqDeriv5 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv5" );
      sprintf(historyEntry->units, "Hz6" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->templateFreqFifthDeriv0);

      /* power_f0 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;


      sprintf(historyEntry->name, "power_f0" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(params->pwF0);

     /* power_mean */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "power_mean" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(params->pwMean);

     /* power_stddev */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "power_stddev" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(params->pwStdDev);

      /* startSec */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "startSec" );
      sprintf(historyEntry->units, "s" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[0].startSec;

      /* startNan */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "startNan" );
      sprintf(historyEntry->units, "ns" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[0].startNan;

      /* sample_rate */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "sample_rate" );
      sprintf(historyEntry->units, "hz" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(1.0/params->deltaT);

      /* duration */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "duration" );
      sprintf(historyEntry->units, "sec" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->tCoh);

     historyEntry->next = NULL;
      /* End history linked list */

     /* allocate memory for the data */
     mddEntry->data.real8 = (REAL8 *) LALCalloc (params->nDeFT, sizeof(REAL8));
     if (!(mddEntry->data.complex8))
     {
      ABORT( status, KNOWNPULSARDEMODH_EALOC, KNOWNPULSARDEMODH_MSGEALOC );
     }

     memcpy (mddEntry->data.real8, xHat[k]->fft->data->data, params->nDeFT * sizeof(REAL8) );

  } /* end for (k = 0; k < params->mObsCoh; k++) */

   /* normal exit */
   DETATCHSTATUSPTR (status);
   RETURN (status);
}
/* END BuildDemodFrameOutput */

/* 11/10/01 gam */ /* Calculate stats associated with frequency series */
/* 05/14/02 gam */
/* void CalcDemodPowerStats( const COMPLEX16Vector *fsVec, DemodPowerStats *stats, REAL8 fStart, REAL8 df ) */
void CalcDemodPowerStats( const COMPLEX8Vector *fsVec, DemodPowerStats *stats, REAL8 fStart, REAL8 df )
{

    UINT4 i = 0;         /* all purpose index */
    INT4 ipwMax = 0;     /* Index max power */
    REAL8 pw = 0.0;      /* power in one bin */
    REAL8 pwSum= 0.0;    /* Sum of power */
    REAL8 pwSum2 = 0.0;  /* Sum of power squared */

    stats->pwMax = 0.0;  /* Initialize */

    for(i=0;i<fsVec->length;i++)
    {
		/* pw = fsVec->data[i].re*fsVec->data[i].re + fsVec->data[i].im*fsVec->data[i].im; */ /* 05/14/02 gam */
		pw = (REAL8)(fsVec->data[i].re*fsVec->data[i].re + fsVec->data[i].im*fsVec->data[i].im);  /* 05/14/02 gam */

	        pwSum += pw;
	        pwSum2 += pw*pw;

		/* Update maximum power */
		if(stats->pwMax < pw)
		{
			stats->pwMax=pw;
			ipwMax=i;
		}
    }
    stats->freqPWMax = fStart + ((REAL8)ipwMax)*df; /* Freq associated with max power */
    stats->pwMean = pwSum/((REAL8)fsVec->length);   /* Mean power */
    /* Standard Deviation of the power = sqrt of the variance */
    stats->pwStdDev = pwSum2/((REAL8)fsVec->length) - stats->pwMean*stats->pwMean;
    if ( stats->pwStdDev > 0.0 ) {
    	stats->pwStdDev = sqrt( stats->pwStdDev );
    } else {
        stats->pwStdDev = -1.0; /* Should not happen except due to round off error in test data */
    }
}

/* 10/15/02 gam Next is used when params->normalizeSFTs > 0; finds mean spectral amplitude in a band specified by if0Min and if0Max from an input frequency series. */
void EstimateMeanSpectralAmplitude( const COMPLEX8Vector *fsVec, INT4 if0Min, INT4 if0Max, REAL4 *meanSpectralAmplitude )
{

    UINT4 i = 0;         /* all purpose index */
    REAL8 pwSum= 0.0;    /* Sum of power */

    for(i=if0Min;i<=if0Max;i++)
    {
	        pwSum += (REAL8)(fsVec->data[i].re*fsVec->data[i].re + fsVec->data[i].im*fsVec->data[i].im);
    }
    /* *meanSpectralAmplitude = (REAL4)sqrt(2.0*pwSum/((REAL8)(if0Max - if0Min + 1))); */ /* Note factor of 2 for one-sided power spectral density */
    *meanSpectralAmplitude = (REAL4)sqrt(pwSum/((REAL8)(if0Max - if0Min + 1))); /* 01/15/03 gam Removed factor of 2 */
}

/* 11/27/01 gam */  /* Calculate the mean value of a time series with cutoffs */
/* void CalcMeanWithCutoffs( const REAL4Vector *tInputVec, REAL4 cutoffMin, REAL4 cutoffMax, REAL4 *meanWithCuttoffs)
{
*/
/*    UINT4 i = 0;    */     /* all purpose index */
/*    REAL4 sum= 0.0; */   /* Sum of power */

    /* printf(stdout, "cutoffMax, cutoffMin = %g %g \n",cutoffMax,cutoffMin); */
/*
    for(i=0;i<tInputVec->length;i++)
    {
    	if ( tInputVec->data[i] <= cutoffMax && tInputVec->data[i] >= cutoffMin ) {
*/
	    /* fprintf(stdout, "i = %i, tInputVec->data[i] = %g \n",i,tInputVec->data[i]); */
/*
            sum += tInputVec->data[i];
        }
        *meanWithCuttoffs = sum/((REAL4)tInputVec->length);
    }
}
*/

/* 11/11/01 gam */ /* Find index of input data that corresponds to gpsStartTime */
void GetStartTimeIndex(const gpsTimeInterval *times, const UINT4 *gpsStartTime, UINT4 *iStartTime)
{
  UINT4 diffSec = *gpsStartTime - times->startSec;
  *iStartTime = floor((REAL8)((REAL8)diffSec - (REAL8)(1.0e-9*(REAL8)times->startNan))/((REAL8)times->timeStepSize));
}

#ifdef INCLUDE_LALINITBARYCENTER
/*  07/01/02 gam */ /* Copied from LALInitBarycenter.c in the LAL support package.  Author: Curt Cutler */

 void LALInitBarycenter(LALStatus *stat, EphemerisData *edat)
 {

    FILE *fp1, *fp2; /* fp1 is table of Earth location; fp2 is for Sun*/

    INT4 j; /*dummy index*/
    INT4 gpsYr;  /*gpsYr + leap is the time on the GPS clock
                          at first instant of new year, UTC; equivalently
                          leap is # of leap secs added between Jan.6, 1980 and
                          Jan. 2 of given year */

/*  07/01/02 gam Comment out since LALINITBARYCENTERC not defined */
/*     INITSTATUS(stat,"LALInitBarycenter",LALINITBARYCENTERC);
    ATTATCHSTATUSPTR(stat); */

/*  07/01/02 gam  Change for local stand-alone code */
    /* fp1 = LALOpenDataFile(edat->ephiles.earthEphemeris);
    fp2 = LALOpenDataFile(edat->ephiles.sunEphemeris); */
    fp1 = fopen(edat->ephiles.earthEphemeris, "r");
    fp2 = fopen(edat->ephiles.sunEphemeris, "r");

    /*
    fp1 = fopen(edat->ephiles.earthEphemeris,"r");
    fp2 = fopen(edat->ephiles.sunEphemeris,"r");
    */

    /* CHECK THAT fp1 and fp2 are not NULL: */
    if ( ( fp1 == NULL ) || ( fp2 == NULL ) ) {
      /*  07/01/02 gam Comment out since LALINITBARYCENTERC not defined */
      /* ABORT (stat, LALINITBARYCENTERH_EOPEN, LALINITBARYCENTERH_MSGEOPEN); */
      return;
    }

/*reading first line of each file */

	 fscanf(fp1,"%d %le %d\n", &gpsYr, &edat->dtEtable, &edat->nentriesE);
	 fscanf(fp2,"%d %le %d\n", &gpsYr, &edat->dtStable, &edat->nentriesS);

     edat->ephemE  = (PosVelAcc *)LALMalloc(edat->nentriesE*sizeof(PosVelAcc));
     edat->ephemS  = (PosVelAcc *)LALMalloc(edat->nentriesS*sizeof(PosVelAcc));

/*first column in earth.dat or sun.dat is gps time--one long integer
  giving the number of secs that have ticked since start of GPS epoch
  +  on 1980 Jan. 6 00:00:00 UTC
*/
       for (j=0; j < edat->nentriesE; ++j){

	 /* check return value of fscanf */
	 fscanf(fp1,"%le %le %le %le %le %le %le %le %le %le\n",
		&edat->ephemE[j].gps,&edat->ephemE[j].pos[0],
		&edat->ephemE[j].pos[1],&edat->ephemE[j].pos[2],
		&edat->ephemE[j].vel[0],&edat->ephemE[j].vel[1],
		&edat->ephemE[j].vel[2],&edat->ephemE[j].acc[0],
		&edat->ephemE[j].acc[1],
		&edat->ephemE[j].acc[2] );
       }

       for (j=0; j < edat->nentriesS; ++j){
	 fscanf(fp2,"%le %le %le %le %le %le %le %le %le %le\n",
		&edat->ephemS[j].gps,&edat->ephemS[j].pos[0],
		&edat->ephemS[j].pos[1],&edat->ephemS[j].pos[2],
		&edat->ephemS[j].vel[0],&edat->ephemS[j].vel[1],
		&edat->ephemS[j].vel[2],&edat->ephemS[j].acc[0],
		&edat->ephemS[j].acc[1],
		&edat->ephemS[j].acc[2] );
       }


/*Test to make sure last entries for gpsE,S are
  reasonable numbers; specifically, checking that they are
  of same order as t2000
  --within factor e
Machine dependent: to be fixed!

    if ( (fabs(log(1.e0*(edat->ephemE[edat->nentriesE -1].gps)/t2000)) > 1.e0)
        ||(fabs(log(1.e0*(edat->ephemS[edat->nentriesS -1].gps)/t2000)) > 1.e0) ){
      LALFree(edat->ephemE);
      LALFree(edat->ephemS);
      ABORT(stat,LALINITBARYCENTERH_EEPHFILE, LALINITBARYCENTERH_MSGEEPHFILE);
    }
*/
	fclose(fp1);
        fclose(fp2);

	/*curt: is below the right return???*/
        /*  07/01/02 gam Comment out since LALINITBARYCENTERC not defined */
	/*
	DETATCHSTATUSPTR(stat);
	RETURN(stat);
	*/
 }
#endif

/* 07/10/02 gam */ /* Function to transfer ephemeris data input as ilwd from LALSearchInput structure into LAL structures; based on LALInitBarycenter. */
void LALWRAPPERInitBarycenter(LALStatus *stat, LALSearchInput *inout, EphemerisData *edat)
{

    /* Author: Greg Mendell; based on LALInitBarycenter. Started: 07/10/02. */

    BOOLEAN earthDataFound = 0;
    BOOLEAN sunDataFound = 0;

    INT4 i; /*dummy index*/
    INT4 j; /*dummy index*/

    /* Loop through sequence again to find SFT data */
    for (i = 0; i < inout->numberSequences; ++i)
    {
         if (strstr(inout->sequences[i].name, "earth")) {

		earthDataFound = 1;

		edat->dtEtable = inout->sequences[i].range.dTime.timeStepSize; /* time step between entries */

		edat->nentriesE = inout->sequences[i].dimensions[1]; /* number of entries */

                #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nFound earth data; dtEtable = %g, nentriesE = %i \n", edat->dtEtable, edat->nentriesE);
			fflush(stdout);
		#endif

     	        edat->ephemE = (PosVelAcc *)LALMalloc(edat->nentriesE*sizeof(PosVelAcc));

                for (j=0; j < edat->nentriesE; ++j){

		     /* initialize time for first entry or increment with time step from previous entry */
                     if (j == 0) {
		         edat->ephemE[j].gps = inout->sequences[i].range.dTime.startSec;
		     } else {
		         edat->ephemE[j].gps = edat->ephemE[j-1].gps + edat->dtEtable;
		     }

		     /* successive 9 values are 3D position, velocity, and acceleration */
		     edat->ephemE[j].pos[0] = inout->sequences[i].data.real4[0 + j*9];
		     edat->ephemE[j].pos[1] = inout->sequences[i].data.real4[1 + j*9];
		     edat->ephemE[j].pos[2] = inout->sequences[i].data.real4[2 + j*9];
		     edat->ephemE[j].vel[0] = inout->sequences[i].data.real4[3 + j*9];
		     edat->ephemE[j].vel[1] = inout->sequences[i].data.real4[4 + j*9];
		     edat->ephemE[j].vel[2] = inout->sequences[i].data.real4[5 + j*9];
		     edat->ephemE[j].acc[0] = inout->sequences[i].data.real4[6 + j*9];
		     edat->ephemE[j].acc[1] = inout->sequences[i].data.real4[7 + j*9];
		     edat->ephemE[j].acc[2] = inout->sequences[i].data.real4[8 + j*9];

                     #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nEarth pos,vel,and acc data for GPS time = %f: %19.16f %19.16f %19.16f %23.16e %23.16e %23.16e %23.16e %23.16e %23.16e \n",edat->ephemE[j].gps,
			        edat->ephemE[j].pos[0],edat->ephemE[j].pos[1],edat->ephemE[j].pos[2],
				edat->ephemE[j].vel[0],edat->ephemE[j].vel[1],edat->ephemE[j].vel[2],
				edat->ephemE[j].acc[0],edat->ephemE[j].acc[1],edat->ephemE[j].acc[2]);
			fflush(stdout);
		     #endif

                }

	    } else if (strstr(inout->sequences[i].name, "sun")) {

		sunDataFound = 1;

                edat->dtStable = inout->sequences[i].range.dTime.timeStepSize; /* time step between entries */

		edat->nentriesS = inout->sequences[i].dimensions[1]; /* number of entries */

                #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nFound sun data; dtStable = %g, nentriesS = %i \n", edat->dtStable, edat->nentriesS);
			fflush(stdout);
		#endif

	        edat->ephemS = (PosVelAcc *)LALMalloc(edat->nentriesS*sizeof(PosVelAcc));

                for (j=0; j < edat->nentriesS; ++j){

		     /* initialize time for first entry or increment with time step from previous entry */
                     if (j == 0) {
		         edat->ephemS[j].gps = inout->sequences[i].range.dTime.startSec;
		     } else {
		         edat->ephemS[j].gps = edat->ephemS[j-1].gps + edat->dtStable;
		     }

		     /* successive 9 values are 3D position, velocity, and acceleration */
		     edat->ephemS[j].pos[0] = inout->sequences[i].data.real4[0 + j*9];
		     edat->ephemS[j].pos[1] = inout->sequences[i].data.real4[1 + j*9];
		     edat->ephemS[j].pos[2] = inout->sequences[i].data.real4[2 + j*9];
		     edat->ephemS[j].vel[0] = inout->sequences[i].data.real4[3 + j*9];
		     edat->ephemS[j].vel[1] = inout->sequences[i].data.real4[4 + j*9];
		     edat->ephemS[j].vel[2] = inout->sequences[i].data.real4[5 + j*9];
		     edat->ephemS[j].acc[0] = inout->sequences[i].data.real4[6 + j*9];
		     edat->ephemS[j].acc[1] = inout->sequences[i].data.real4[7 + j*9];
		     edat->ephemS[j].acc[2] = inout->sequences[i].data.real4[8 + j*9];

                     #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nSun pos,vel,and acc data for GPS time = %f: %19.16f %19.16f %19.16f %23.16e %23.16e %23.16e %23.16e %23.16e %23.16e \n",edat->ephemS[j].gps,
			        edat->ephemS[j].pos[0],edat->ephemS[j].pos[1],edat->ephemS[j].pos[2],
				edat->ephemS[j].vel[0],edat->ephemS[j].vel[1],edat->ephemS[j].vel[2],
				edat->ephemS[j].acc[0],edat->ephemS[j].acc[1],edat->ephemS[j].acc[2]);
			fflush(stdout);
		     #endif

                }

         }

	 if (earthDataFound && sunDataFound) {
                #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nFound earth and sun data.\n");
			fflush(stdout);
		#endif
	 	break;
	 }
    }
    /* END for (i = 0; i < inout->numberSequences; ++i) */
}
/* END void LALWRAPPERInitBarycenter(LALStatus *stat, LALSearchInput *inout, EphemerisData *edat) */

/*  02/03/03 gam If doDemod == -2 then high pass filter and window time series data before generating SFTs. (Make doDemod == -1 then remove mean) */
void HighPassFilterAndWindow(LALStatus* const status, REAL4Vector *tvec, const DemodSearchParams* const params)
{

      /* 02/03/03 gam COPY OF CODE FROM BRUCE ALLEN */

#define WINSTART 150
#define WINEND   200
#define WINLEN (WINEND-WINSTART)
// with i ranging from 0 to WINLEN...
#define WINFUN(i) ((sin(i*M_PI/(WINLEN)-M_PI_2)+1.0)/2.0)

      INT4 i;
      static REAL4TimeSeries chan; /* must zero the f0 field */
      PassBandParamStruc filterpar;
      REAL8 window[WINLEN];

      INITSTATUS( status, "BuildDemodFramesOutput", KNOWNPULSARDEMODC );
      ATTATCHSTATUSPTR (status);

      /* initialize chan data = input time series data */
      chan.data = tvec;
      // set up LAL time series object sample interval, time epoch
      strcpy(chan.name, params->channel);
      chan.deltaT = params->deltaT;
      chan.epoch = params->timeStamps[0];
      chan.data->length = params->inputN;

      /* set filtering parameters */
      filterpar.name = "Butterworth High Pass";
      filterpar.nMax  = 5;
      filterpar.f2 = 100.0;
      filterpar.a2 = 0.5;
      /* values that are 'not given' = out of range */
      filterpar.f1 = 0.0;
      filterpar.a1 = 0.0;

      /* init window function */
      for(i=0; i<WINLEN; i++)
    	window[i]=WINFUN(i);

      /* apply butterworth filter */
      LALButterworthREAL4TimeSeries(status->statusPtr, &chan, &filterpar);
      CHECKSTATUSPTR (status);

      // window data.  Off portion
      for (i=0; i<WINSTART; i++) {
	chan.data->data[i] = 0.0;
	chan.data->data[chan.data->length - 1 - i] = 0.0;
      }
      // window data, smooth turn-on portion
      for (i=WINSTART; i<WINEND; i++) {
	double win=window[i - WINSTART];
	chan.data->data[i] *= win;
	chan.data->data[chan.data->length - 1 - i]  *= win;
      }

      DETATCHSTATUSPTR (status);
      RETURN (status);

}




